#if !defined( CAM_EXP_H )
#define CAM_EXP_H

#include "gen_typs.h"
#include "cam_tsk.h"
#include "osd_scsi.h"

struct _CAM_DISK;
typedef struct _CAM_DISK CAM_DISK, buf_space * bpCAM_DISK;

#if defined(USE_EXPORT)
#define CAM_EXP_API(funcName) EXP_CAM_##funcName
#else
#define CAM_EXP_API(funcName) CAM_##funcName
#endif

#define DECLARE_CAM_EXP(funcName) GEN_EXPORT CAM_EXP_API(funcName)

void DECLARE_CAM_EXP(ComponentGetExtensionSize)( 
		OUT bpMEMORY_REQUIREMENT pMemRequirement );

void DECLARE_CAM_EXP(ComponentGetDMASafeMemoryRequirement)(
		IN U32						VenDevID,
		IN U32						SubVenDevID,
		OUT bpMEMORY_REQUIREMENT	pMemRequirement );

void DECLARE_CAM_EXP(ComponentGetDIMMRequirement)(
		IN U32						VenDevID,
		IN U32						SubVenDevID,
		OUT bpMEMORY_REQUIREMENT	pMemRequirement );

void DECLARE_CAM_EXP(ComponentInitialize)(
		IN bpVOID				ComponentExtension,
		IN bpMEMORY_AVAILABLE	pMemAvail,
		IN U32					VenDevID,
		IN U32					SubVenDevID,
		IN bpVOID				ComponentSpecificContext );

void DECLARE_CAM_EXP(ComponentStart)(
		IN bpVOID	ComponentExtension,
		IN U32		DIMMOffset,
		IN U32		DIMMSize );

void DECLARE_CAM_EXP(ComponentStop)( 
		IN bpVOID ComponentExtension );

void DECLARE_CAM_EXP(ComponentRestart)( 
		IN bpVOID ComponentExtension );

void DECLARE_CAM_EXP(ComponentBist)( 
		IN bpVOID ComponentExtension, 
		IN U32 BistFlag );

#if 0//def _X8616B/*============================================================*/

void DECLARE_CAM_EXP(MakeTransferPacket)( 
		IN bpVOID ComponentExtension, 
		IN U8 PortIndex );

#endif/*============================================================*/

void DECLARE_CAM_EXP(SubmitATATask)( 
		IN bpVOID ComponentExtension, 
		IN bpCAM_ATATASK pAtaTask );

void DECLARE_CAM_EXP(SubmitXORTask)( 
		IN bpVOID ComponentExtension, 
		IN bpCAM_XORTASK pXorTask );

void DECLARE_CAM_EXP(SubmitHDMATask)( 
		IN bpVOID ComponentExtension, 
		IN bpCAM_XORTASK pXorTask );

U8 DECLARE_CAM_EXP(CheckInterrupt)( 
		IN bpVOID ComponentExtension );

void DECLARE_CAM_EXP(HandleInterrupt)( 
		IN bpVOID ComponentExtension );

#if !defined(_X8616B)
void DECLARE_CAM_EXP(ComponentReset)( 
		IN bpVOID ComponentExtension );
#endif	/*!_X8616B*/

#if defined(_WINAPP)
U8 DECLARE_CAM_EXP(GetMaximumDiskCount)( 
		IN bpVOID ComponentExtension );
#endif

struct _CAM_DISK
{
	U32 Flags;		/* see below CAM_DISK_F_xxx */
	U32 State;
	
	U8	ChannelID;
	U8	TargetID;	/* can be used as port address defined in SATA II spec for port
					 * multiplier */
	U8	AssociatedEnclosureID;
	U8	CarrierIDInEnclosure;
	
	U64 MaximumLBA; /* This is definitely necessary */
	U32 Cylinders;	/* # of cyl */
	
	U8	Heads;		/* # of hd per cyl */
	U8	Sectors;	/* # of sctr per hd */
	U8	Features;	/* CAM_DEV_F_xxx */
	U8	PIOMode;
	
	U8	MDMAMode;
	U8	UDMAMode;
	U8	NCQQueueDepth;	
	U8	SerialNo[21];
	
	U8	Model[22];
	U8	FWVersion[10];
//#if defined(SUPPORT_SAS)
//	U8	LUN;
//#endif
/* jeffgforsafte */
#if defined(SUPPORT_SAFTE) && defined(SUPPORT_ENCLOSURE)
	U32	flags_safte;
#endif
};

/* _CAM_DISK.Flags */
#define CAM_DISK_F_DEVEXIST			0x0001
#define CAM_DISK_F_48BIT				0x0002
#define CAM_DISK_F_NCQ_ENABLED		0x0004	/* NCQ is enabled */
#define CAM_DISK_F_TCQ_ENABLED		0x0008	/* TCQ is enabled */
#define CAM_DISK_F_WC_ENABLED		0x0010	/* write cache enabled */
#define CAM_DISK_F_LA_ENABLED			0x0020	/* read lookahead enabled */
#define CAM_DISK_F_SMART_ENABLED		0x0040
#define CAM_DISK_F_ERROR				0x0080
#define CAM_DISK_F_ATAPI				0x0100	/* ATAPI device */
#define CAM_DISK_F_3G					0x0200
#define CAM_DISK_F_SCSI				0x0400	/* SCSI device*/
#define CAM_DISK_F_SCSI_TAPE				0x0800	/* SCSI TAPE device*/
#if defined(SUPPORT_PM)
#define CAM_DISK_F_Removing			0x0800
#endif

/* jeffgforsafte */
#if defined(SUPPORT_SAFTE) && defined(SUPPORT_ENCLOSURE)
#define CAM_DISK_F_SAFTE_BYTE1_NOERROR		0x010000
#define CAM_DISK_F_SAFTE_BYTE1_DEVICEFAULT	0x020000
#define CAM_DISK_F_SAFTE_BYTE1_REBUILDING	0x040000
#define CAM_DISK_F_SAFTE_BYTE1_NOHDD		0x080000

#define CAM_DISK_F_SAFTE_BYTE2_ACTIVITY		0x400000
#endif

/* _CAM_DISK.Features */
#define CAM_DEV_F_SMART_SUPPORTED	0x01	/* SMART feature set supported */
#define CAM_DEV_F_WC_SUPPORTED		0x02	/* write cache supported */
#define CAM_DEV_F_LA_SUPPORTED		0x04	/* lookahead supported */
#define CAM_DEV_F_NCQ_SUPPORTED		0x08	/* NCQ  supported */
#define CAM_DEV_F_TCQ_SUPPORTED		0x10	/* TCQ  supported */
#define CAM_DEV_F_3G_SUPPORTED		0x20	/* 3G speed */

void	DECLARE_CAM_EXP(DiskGetInformation)(
			IN bpVOID		ComponentExtension,
			IN U8			DiskID,
			OUT bpCAM_DISK	pCamDisk );

#if defined(CAM_ATAPI_SUPPORT)
void	DECLARE_CAM_EXP(DiskGetInformationEx)(
			IN bpVOID		ComponentExtension,
			IN U8			DiskID,
			OUT bpCAM_DISK	pCamDisk );
#endif	/**/


U32	DECLARE_CAM_EXP(DIMMGetSize)( 
			IN bpVOID ComponentExtension, 
			IN bpU8 DIMMType );

void	DECLARE_CAM_EXP(DIMMRead)(
			IN bpVOID		ComponentExtension,
			IN OUT void*	pTarget,
			IN U32			Offset,
			IN U32			SizeInByte );

void	DECLARE_CAM_EXP(DIMMWrite)(
			IN bpVOID		ComponentExtension,
			IN const void*	pSource,
			IN U32			Offset,
			IN U32			SizeInByte );

U32	DECLARE_CAM_EXP(NVRAMGetSize)( 
			IN bpVOID ComponentExtension );

void	DECLARE_CAM_EXP(NVRAMRead)(
			IN bpVOID		ComponentExtension,
			IN OUT void*	pTarget,
			IN U32			Offset,
			IN U32			SizeInByte );

void	DECLARE_CAM_EXP(NVRAMWrite)(
			IN bpVOID		ComponentExtension,
			IN const void*	pSource,
			IN U32			Offset,
			IN U32			SizeInByte );

#if defined(SUPPORT_BUZZER)
void	DECLARE_CAM_EXP(StartBuzzer)( 
			IN bpVOID ComponentExtension );

void	DECLARE_CAM_EXP(StopBuzzer)( 
			IN bpVOID ComponentExtension );
#endif

#if defined(_WINAPP)
void	DECLARE_CAM_EXP(Terminate)( 
			IN bpVOID ComponentExtension );
#endif

#if defined(SUPPORT_ENCLOSURE)
/**************************************************************** 
 *			Enclosure Related Definition and Interface			*
 ****************************************************************/
/* Enclosure Related Definitions */
#define CAM_MAX_CARRIER					0x04	/* SS4100 is 4 */
#define CAM_MAX_LED_PER_CARRIER			0x02	/* SS1100 is 2 */
#define CAM_MAX_LED_PER_ENCLOSURE		0x03	/* NEC is 3 */
#define CAM_MAX_CARRIER_PER_ENCLOSURE	0x04	/* SS4100 */

#if defined(SUPPORT_SAFTE) && defined(SUPPORT_ENCLOSURE)
		#define CAM_MAX_CARRIER_PER_SAFTE_ENCLOSURE	4	/* T3 is 4 */
#endif

enum LEDType
{
	LED_TYPE_ACTIVITY,
	LED_TYPE_STATUS,
	LED_TYPE_GLOBAL
};

enum FlashType
{
	FLASH_OFF,
	FLASH_SLOW,
	FLASH_FAST,
	FLASH_STEADYON,
	FLASH_INVALIDE
};

enum HDActivity
{
	HDActivityOFF,
	HDActivityON
};

#define CAM_DISK_ACTIVITY_FLAG   0x80

#define LED_CHANGED 0x80
typedef U8 LEDCOLOR;
//enum LEDCOLOR
//{
#define	LED_OFF			0x1
#define LED_GREEN		0x2
#define LED_AMBER		0x3
#define LED_RED			0x4
#define LED_INVALIDE	0x5
//};

//enum FAULTLED
#define	TURN_OFF		0x0L
#define 	TURN_ON		0x1L
#define 	BLINKING		0x2L

/* Interface between Engine and CAM
 * In order to control the Enclosure LED, here is the list of possible disk status. 
 * It is based on SS4100 and SS1100 PRD. 
 * I may need organize it after analyzing NEC, FSC and HPQ requirement.
 */
#define ENC_DISK_POWERON				0x1		/* Power applied and drive is getting ready */
#define ENC_DISK_NODISK					0x2		/* Drive not installed */
#define ENC_DISK_HEALTHY				0x3		/* Drive ready and healthy */
#define ENC_DISK_SETDOWN				0x4		/* (*) Drive fails and offline(engine set down the drive due to error) */
#define ENC_DISK_HEALTHY_ARRAY_OFFLINE	0x5		/* Healthy drive in an offline array */
#define ENC_DISK_REBUILD				0x6		/* (*) Drive under rebuild process or in critical mode */
#define ENC_SENSOR_ERROR				0x7		/* Temperature, voltage or FAN error */
#define ENC_DISK_SMARTERROR				0x8		/* (*) Drive SMART status error */
#define ENC_DISK_REBUILDPAUSED			0x9		/* (*) Target drive during pause when rebuilding */
#define ENC_ARRAY_MORPHINGPAUSED		0xA		/* (*) Pause when morphing */
#define ENC_DISK_DOWN_ARRAY_MORPHING	0xB		/* Drive fail in a morphing array */
#define ENC_DISK_HEALTHY_ARRAY_MORPHING	0xC		/* Healthy drive in a morphing array */
#define ENC_DISK_LOCATE					0xD		/* Locate the HDD */
/* jeffgforsafte */
#if defined (SUPPORT_SAFTE)
#define ENC_DISK_SETACTIVITY				0xE	
#endif

void CAM_EnclosureDiskStatusNotification(
	LPVOID CAMExtension,
	U8 DiskID, 
	U8 DiskStatus
	);

void CAM_EnclosureDiskActivityNotification(
	LPVOID CAMExtension,
	U8 DiskID, 
	enum HDActivity ActivityStatus
	);

U8 CAM_Enclosure_GetType(
	IN OUT LPVOID CAMExtension,
	IN U8 DiskID 
	);

/* CAM_EnclosureGetEncCount returns number of enclosure boxes detected by CAM. */
U8 CAM_EnclosureGetEncCount(
	IN bpVOID ComponentExtension
);

/* Driver will call CAM_EnclosureGetSensorInfo to get sensor value for each sensor. */
U8 CAM_EnclosureGetSensorInfo(
	IN bpVOID ComponentExtension,
	IN U8 EncID,
	IN U8 SensorID,
	OUT PU32 pValue
);

/* These two functions are provided for application to locate a disk in the enclosure. */
#define CAM_ENCLOSURE_STOP_LOCATE		0x0
#define CAM_ENCLOSURE_START_LOCATE		0x1
#define CAM_ENCLOSURE_RETRIEVE_LOCATE	0x2
U8 CAM_EnclosureLocateDiskWithDiskID(
	IN bpVOID ComponentExtension,
	IN U8 DiskID,
	IN U8 LocateControl
);

U8 CAM_EnclosureLocateDiskWithChannel(
	IN bpVOID ComponentExtension,
	IN U8 Channel,
	IN U8 TargetID,
	IN U8 Start /* 1 to start locating, 0 to stop */
);

void CAM_StopEnclosureModule(
	IN OUT bpVOID CamExtension
);

void CAM_EnclosureEnumerate(
	IN bpVOID ComponentExtension,
	IN PAPIFRAME_ENCLOSURE_INFORMATION pAPI_Enclosure
);

U8 CAM_EnclosureMapDiskID(
	IN bpVOID ComponentExtension,
	IN U8 DiskID,
	IN bpU8 EnclosureID,
	IN bpU8 CarrierID
);

/* Enclosure Event */
#define EVENT_ENCLOSURE_FAN_OUTOFORDER			0x1
#define EVENT_ENCLOSURE_TEMP_OUTOFORDER			0x2
#define EVENT_ENCLOSURE_VOLTAGE33_OUTOFORDER	0x3
#define EVENT_ENCLOSURE_VOLTAGE5_OUTOFORDER		0x4
#define EVENT_ENCLOSURE_VOLTAGE12_OUTOFORDER	0x5

#endif /* SUPPORT_ENCLOSURE */

/**************************************************************** 
 *			Battery Related Definition and Interface			*
 ****************************************************************/
#ifdef SUPPORT_BATTERY

U8 CAM_HasBattery(
	IN bpVOID ComponentExtension
	);

/* Battery Events*/
#define	BATTERY_STATUS_NORMAL						0x00
#define BATTERY_STATUS_TEMP_OUTOFRANGE				0x01
#define BATTERY_STATUS_TEMP_UNSTABLE				0x02
#define BATTERY_STATUS_VOLT_OUTOFRANGE				0x04
#define BATTERY_STATUS_VOLT_UNSTABLE				0x08
#define BATTERY_STATUS_COMMUNICATION_ERROR			0x10
#define BATTERY_STATUS_VOLT_ISCHARGING				0x20	/* It's not an event. */
#define	BATTERY_STATUS_FINAL_ABNORMAL				0x40	/* Error Count > threshold */
#define	BATTERY_STATUS_NOT_DETECTED				    0x80	/*  */

#define	BATTERY_STATUS_UNKNOWN 				        BATTERY_STATUS_VOLT_ISCHARGING

void CAM_BatteryGetInformation(
	IN bpVOID ComponentExtension,
	IN PAPIFRAME_BATTERY_INFORMATION pAPI_Battery
);

#endif /* SUPPORT_BATTERY */

#ifdef INCLUDE_PCI_MEM_EVENT
/* CAM PCI/MEM Events*/
#define CAM_ERR_PCI_PARITY			0x00 /*PCI parity error*/
#define CAM_ERR_PCI_SYSTEM			0x01 /*PCI system error*/
#define CAM_ERR_MEM_SINGLEBIT	    0x02 /*Memory single bit error*/
#define CAM_ERR_MEM_MULTIBIT	    0x03 /*Memory multi bit error*/

#endif

/* Engine Export Function Table Definition */
#if defined(USE_EXPORT)/*VVVVVVVVVVVVVVVVV<USE_EXPORT>VVVVVVVVVVVVVVVVVVVVVVVVVVVVVV*/

#define DEFINE_CAM_EXP_FUNC(funcName)	(GEN_EXPORT* CAM_EXP_API(funcName))

typedef struct
{
	void DEFINE_CAM_EXP_FUNC(ComponentGetExtensionSize)( 
			OUT bpMEMORY_REQUIREMENT pMemRequirement );

	void DEFINE_CAM_EXP_FUNC(ComponentGetDMASafeMemoryRequirement)(
			IN U32						VenDevID,
			IN U32						SubVenDevID,
			OUT bpMEMORY_REQUIREMENT	pMemRequirement );

	void DEFINE_CAM_EXP_FUNC(ComponentGetDIMMRequirement)(
			IN U32						VenDevID,
			IN U32						SubVenDevID,
			OUT bpMEMORY_REQUIREMENT	pMemRequirement );

	void DEFINE_CAM_EXP_FUNC(ComponentInitialize)(
			IN bpVOID				ComponentExtension,
			IN bpMEMORY_AVAILABLE	pMemAvail,
			IN U32					VenDevID,
			IN U32					SubVenDevID,
			IN bpVOID				ComponentSpecificContext );

	void DEFINE_CAM_EXP_FUNC(ComponentStart)(
			IN bpVOID	ComponentExtension,
			IN U32		DIMMOffset,
			IN U32		DIMMSize );

	void DEFINE_CAM_EXP_FUNC(ComponentStop)( 
			IN bpVOID ComponentExtension );

	void DEFINE_CAM_EXP_FUNC(ComponentRestart)( 
			IN bpVOID ComponentExtension );

	void DEFINE_CAM_EXP_FUNC(ComponentBist)( 
			IN bpVOID ComponentExtension, 
			IN U32 BistFlag );

#ifdef _X8616B/*============================================================*/

	void DEFINE_CAM_EXP_FUNC(MakeTransferPacket)( 
			IN bpVOID ComponentExtension, 
			IN U8 PortIndex );

#endif/*============================================================*/

	void DEFINE_CAM_EXP_FUNC(SubmitATATask)( 
			IN bpVOID ComponentExtension, 
			IN bpCAM_ATATASK pAtaTask );

	void DEFINE_CAM_EXP_FUNC(SubmitXORTask)( 
			IN bpVOID ComponentExtension, 
			IN bpCAM_XORTASK pXorTask );

	void DEFINE_CAM_EXP_FUNC(SubmitHDMATask)( 
			IN bpVOID ComponentExtension, 
			IN bpCAM_XORTASK pXorTask );

	U8 DEFINE_CAM_EXP_FUNC(CheckInterrupt)( 
			IN bpVOID ComponentExtension );

	void DEFINE_CAM_EXP_FUNC(HandleInterrupt)( 
			IN bpVOID ComponentExtension );

#if defined(_WINAPP)
	U8 DEFINE_CAM_EXP_FUNC(GetMaximumDiskCount)( 
			IN bpVOID ComponentExtension );
#endif

	void	DEFINE_CAM_EXP_FUNC(DiskGetInformation)(
			IN bpVOID		ComponentExtension,
			IN U8			DiskID,
			OUT bpCAM_DISK	pCamDisk );

#if defined(CAM_ATAPI_SUPPORT)
	void	DEFINE_CAM_EXP_FUNC(DiskGetInformationEx)(
			IN bpVOID		ComponentExtension,
			IN U8			DiskID,
			OUT bpCAM_DISK	pCamDisk );
#endif	/*CAM_ATAPI_SUPPORT*/

	U32		DEFINE_CAM_EXP_FUNC(DIMMGetSize)( 
			IN bpVOID ComponentExtension,
			IN bpU8 DIMMType);

	void	DEFINE_CAM_EXP_FUNC(DIMMRead)(
			IN bpVOID		ComponentExtension,
			IN OUT bpVOID	pTarget,
			IN U32			Offset,
			IN U32			SizeInByte );

	void	DEFINE_CAM_EXP_FUNC(DIMMWrite)(
			IN bpVOID		ComponentExtension,
			IN const void GEN_NEAR* pSource,
			IN U32			Offset,
			IN U32			SizeInByte );

	U32		DEFINE_CAM_EXP_FUNC(NVRAMGetSize)( 
			IN bpVOID ComponentExtension );

	void	DEFINE_CAM_EXP_FUNC(NVRAMRead)(
			IN bpVOID		ComponentExtension,
			IN OUT void*	pTarget,
			IN U32			Offset,
			IN U32			SizeInByte );

	void	DEFINE_CAM_EXP_FUNC(NVRAMWrite)(
			IN bpVOID		ComponentExtension,
			IN const void*	pSource,
			IN U32			Offset,
			IN U32			SizeInByte );

#if defined(SUPPORT_BUZZER)
	void	DEFINE_CAM_EXP_FUNC(StartBuzzer)( 
		IN bpVOID ComponentExtension );
	void	DEFINE_CAM_EXP_FUNC(StopBuzzer)( 
		IN bpVOID ComponentExtension );
#endif	/*SUPPORT_BUZZER*/

#if defined(_WINAPP)
	void	DEFINE_CAM_EXP_FUNC(Terminate)( 
			IN bpVOID ComponentExtension );
#endif

#if !defined(_X8616B)
	void	DEFINE_CAM_EXP_FUNC(ComponentReset)( 
		IN bpVOID ComponentExtension );
#endif	/*!_X8616B*/


} CAM_EXPORT_FUNC_TABLE, buf_space * bpCAM_EXPORT_FUNC_TABLE;

#if defined(_X8616B) && !defined(TestcamOnDOS)/*============================================================*/
extern "C" bpCAM_EXPORT_FUNC_TABLE PSEUDOBASE g_pCamApi;
extern "C" CAM_EXPORT_FUNC_TABLE PSEUDOBASE g_CamApi;
#else/*============================================================*/
extern bpCAM_EXPORT_FUNC_TABLE PSEUDOBASE g_pCamApi;
extern CAM_EXPORT_FUNC_TABLE PSEUDOBASE g_CamApi;
#endif/*============================================================*/

#define CAM_ComponentGetExtensionSize				g_pCamApi->CAM_EXP_API(ComponentGetExtensionSize)
#define CAM_ComponentGetDMASafeMemoryRequirement	g_pCamApi->CAM_EXP_API(ComponentGetDMASafeMemoryRequirement)
#define CAM_ComponentGetDIMMRequirement				g_pCamApi->CAM_EXP_API(ComponentGetDIMMRequirement)
#define CAM_ComponentInitialize						g_pCamApi->CAM_EXP_API(ComponentInitialize)
#define CAM_ComponentStart							g_pCamApi->CAM_EXP_API(ComponentStart)
#define CAM_ComponentStop							g_pCamApi->CAM_EXP_API(ComponentStop)
#define CAM_ComponentRestart						g_pCamApi->CAM_EXP_API(ComponentRestart)
#define CAM_ComponentBist							g_pCamApi->CAM_EXP_API(ComponentBist)
#ifdef _X8616B/*============================================================*/
#define CAM_MakeTransferPacket						g_pCamApi->CAM_EXP_API(MakeTransferPacket)
#endif/*============================================================*/
#define CAM_SubmitATATask							g_pCamApi->CAM_EXP_API(SubmitATATask)
#define CAM_SubmitXORTask							g_pCamApi->CAM_EXP_API(SubmitXORTask)
#define CAM_SubmitHDMATask							g_pCamApi->CAM_EXP_API(SubmitHDMATask)
#define CAM_CheckInterrupt							g_pCamApi->CAM_EXP_API(CheckInterrupt)
#define CAM_HandleInterrupt							g_pCamApi->CAM_EXP_API(HandleInterrupt)
#if defined(_WINAPP)
#define CAM_GetMaximumDiskCount						g_pCamApi->CAM_EXP_API(GetMaximumDiskCount)
#endif
#define CAM_DiskGetInformation						g_pCamApi->CAM_EXP_API(DiskGetInformation)
#if defined(CAM_ATAPI_SUPPORT)
#define CAM_DiskGetInformationEx					g_pCamApi->CAM_EXP_API(DiskGetInformationEx)
#endif	/*CAM_ATAPI_SUPPORT*/
#define CAM_DIMMGetSize								g_pCamApi->CAM_EXP_API(DIMMGetSize)
#define CAM_DIMMRead								g_pCamApi->CAM_EXP_API(DIMMRead)
#define CAM_DIMMWrite								g_pCamApi->CAM_EXP_API(DIMMWrite)
#define CAM_NVRAMGetSize							g_pCamApi->CAM_EXP_API(NVRAMGetSize)
#define CAM_NVRAMRead								g_pCamApi->CAM_EXP_API(NVRAMRead)
#define CAM_NVRAMWrite								g_pCamApi->CAM_EXP_API(NVRAMWrite)

#if defined(SUPPORT_BUZZER)
#define CAM_StartBuzzer								g_pCamApi->CAM_EXP_API(StartBuzzer)
#define CAM_StopBuzzer								g_pCamApi->CAM_EXP_API(StopBuzzer)
#endif

#if defined(_WINAPP)
#define CAM_Terminate								g_pCamApi->CAM_EXP_API(Terminate) 
#endif
#if !defined(_X8616B)
#define CAM_ComponentReset							g_pCamApi->CAM_EXP_API(ComponentReset)
#endif


#endif /*USE_EXPORT*//*^^^^^^^^^^^^^^^^^<USE_EXPORT>^^^^^^^^^^^^^^^^^^^^^^^^^^*/

#if defined(DIMM_TO_VIRT_SUPPORTED)
bpVOID
CAM_GetDimmVirtualAddress(
	bpVOID CamComponentExtension,
	DIMMADDR DimmAddr );
#endif

#if defined(CAM_SUPPORT_SYNCHRONIZED_SETTING)

/* ActionCode definitions */
#define CAM_SS_AC_SET_NCQ	0	/* Param1: DiskID, Param2: 0 to disable, 1 to enable */
#define CAM_SS_AC_SET_TCQ	1	/* Param1: DiskID, Param2: 0 to disable, 1 to enable */

#define	CAM_SS_RET_SUCCEEDED			0
#define CAM_SS_RET_INVALID_PARAMETERS	1
#define CAM_SS_RET_NOT_CHANGED			2
#define CAM_SS_RET_NOT_SUPPORTED		3

U32
CAM_SynchronizedSetting(
	bpVOID		CamComponentExtension,
	U32			ActionCode,
	PTR_SIZE	Param1,
	PTR_SIZE	Param2,
	PTR_SIZE	Param3,
	PTR_SIZE	Param4 );

#endif	/* CAM_SUPPORT_SYNCHRONIZED_SETTING */

#if defined(CAM_SUPPORT_HW_TRIGGER)
/* hardware debugging interface */
void
CAM_HardwareTrigger( 
	bpVOID	CamComponentExtension,
	U32		TriggerValue );
#endif

#ifdef	SUPPORT_FAULTLED
void CAM_FaultLEDStatusStatusNotification(
	IN OUT LPVOID CAMExtension,
	IN U8 DiskID, 
	IN U32 DiskStatus
	);
	
void CAM_FaultLEDReset(
	IN OUT LPVOID CAMExtension	
);	

//by jordan for asus oem
#define VERSION_CHNANELL 	0x0
#define VERSION_ASUS_OEM 	0x1
#define VERSION_SAS_TAPE 	0x2
#define VERSION_SATAPI 		0x4
#define VERSION_BUZZER 		0x8

U8 CAM_CheckVersion(
	IN OUT LPVOID CAMExtension	
);

#endif

#endif /* CAM_EXP_H */
