/****************************************************************************
 *  Header:
 *
 *  Contributor:     Copyright (c) 1999-2005  Promise Technology Inc.
 *
 ****************************************************************************
 *
 *  Description:
 *  -------------------------------------------------------------------------
 *  R02 05/20   Adam    To follow the NAPA CAM code about CAM_ATAPI_SUPPORT defination
 *		        to fix the SATAPI can't boot.
 *  R01	04/22	Adam	Init version to merge ATI CAM code to T3 2.5 BIOS
 */


#if !defined( CAM_TASK_H )
#define CAM_TASK_H

struct _CAM_ATATASK;
typedef struct _CAM_ATATASK CAM_ATATASK, buf_space * bpCAM_ATATASK;

struct _CAM_XORTASK;
typedef struct _CAM_XORTASK CAM_XORTASK, buf_space * bpCAM_XORTASK;

typedef void (GEN_CALLBACK *CAM_TASK_CALLBACK ) ( bpVOID pContext, bpCAM_ATATASK pCamTask );

#define ATACMD_F_PSEUDO_CMD		0x0001
#define ATACMD_F_DATA_XFER		0x0002
#define ATACMD_F_DATA_IN			0x0004
#define ATACMD_F_PIO				0x0008
#define ATACMD_F_STATUS_RETURN	0x0010
#define ATACMD_F_DIRECT_CMD		0x0020
#define ATACMD_F_WAIT_FOR_INT	0x0040
#define ATACMD_F_PSEUDO_NCQ_CMD	0x0080

#define ATACMD_F_PIO_DMA			0x0100	/* NAPA ASIC Specific */
#define ATACMD_F_EMERGENT			0x0200	/* CAM Internal usage. an emergent task would skip normal queue management */
#define ATACMD_F_NON_QUEUE		0x0400	/* CAM Internal usage. no NCQ or chaining when this flag on */
#define ATACMD_F_48B_CMD			0x0800	/* CAM Internal usage. */
#define ATACMD_F_SRST				0x1000	/* Software Reset */
#define ATACMD_F_ATAPI				0x2000	/* ATAPI command */

#if defined(SUPPORT_PASSTHROUGH)		//NCQ performance issue by tammy
#define ATACMD_F_PASSTHRO_CMD	0x4000
#endif
#if defined(SUPPORT_ICH6_BUG)
#define ATACMD_F_CH6BUG			0x8000	/*ICH6 BUG*/
#endif

/* definitions for ATAStatus */
#define ATAERR_OVERRUN				0x08		/* indicate total count of S/G table is
											 * larger than HDD requires */
#define ATAERR_UNDERRUN			0x10		/* indicate total count of S/G table is less
											 * than HDD requires */
#define ATAERR_DRV					0x20		/* check bError and bStatus of ATACmd or
											 * check sense data in SenseInfoBuffer to get
											 * the detail information */
#define ATAERR_PCI					0x40		/* Indicates that is due to PCI system error */
#define ATAERR_PARITY				0x80		/* Indicates that is due to PCI parity check
											 * error */
#define ATAERR_HWERROR			( ATAERR_OVERRUN + ATAERR_UNDERRUN + ATAERR_DRV + ATAERR_PCI + ATAERR_PARITY )
#define ATABUF_F_V_ADDR			0x01
#define ATABUF_F_SG_ADDR			0x02
#define ATABUF_F_UNALIGNED_SG		0x04
#if defined(CAM_Performance_Parity_DIMM)
#define ATABUF_F_PARITY_SG			0x08
#endif
/* CAM ATA status (CAM_ATATASK.CamAtaStatus) */
#define CAMATA_STATUS_OK			0x00			/* everything works fine */

/*
 * ATA reports error, AtaStatus and task file registers must be updated. Upper
 * layers relies on them. CAM should set this flag if the ERR bit of ATA task file
 * status register is set.
 */
#define CAMATA_STATUS_ATAERROR	0x01

/*
 * SATA reports error, SataStatus, SataError and task file registers must be
 * updated. Pls refer to related ASIC spec.
 */
#define CAMATA_STATUS_SATAERROR 	0x02

/*
 * the CAM task is aborted because of software error, i.e. queue abort, etc. Note
 * that this is different from ATA ABORT. For ATA ABORT, CAMATA_STATUS_ATAERROR
 * should be set instead. No other status/error registers needs updating.
 */
#define CAMATA_STATUS_ABORT		0x03

/* disk is gone. No other status/error registers needs updating. */
#define CAMATA_STATUS_DISK_OFFLINE	0x04

/* task timeout. No other status/error registers needs updating. */
#define CAMATA_STATUS_TIMEOUT		0x05

/*Retry Task for Tammy handle*/ //some wrong with HDD 2006/10/01 by tammy
#define	CAMATA_STATUS_RETRYTASK		0x06		//R02


#define CAM_ATA_TASK_RETRY_CNT 1

#if defined( _WINAPP )
#define CAM_TASK_ATA		0x00
#define CAM_TASK_XOR		0x01
#define CAM_TASK_HOTPLUG	0x02			/* used for hotplug simulation */
#define CAM_TASK_CRASH		0x03			/* used for crash test */
#endif

typedef struct	_SG_INFO
{
#if defined(MOONFANG)
	U8		Coef;							/* Galosis Multiplication Coefficient */
	U8		bReserved[3];
#endif

	bpVOID	pSG;
#if !defined(CAM_USE_HOSTMEM_FOR_PACKETS)
	DIMMADDR	DimmSGAddr;						/* this field is defined for CAM internal
											 * only, other component should not access it
											 * */
#else
	bpVOID		DimmSGAddr;
#endif
}
SG_INFO, *PSG_INFO;

#if defined( _WINAPP )
#define DEFINE_CAM_TASK_HEADER				\
	U32						Type;			\
	bpCAM_ATATASK			pPrev;			\
	bpCAM_ATATASK			pNext;			\
	bpVOID					pContext;		\
	CAM_TASK_CALLBACK		pCallback;
#else
#define DEFINE_CAM_TASK_HEADER				\
	bpCAM_ATATASK			pPrev;			\
	bpCAM_ATATASK			pNext;			\
	bpVOID					pContext;		\
	CAM_TASK_CALLBACK		pCallback;
#endif

#if defined(DONT_USE_NAPA_CAM_ATATASK)

#include "cam_mvtk.h"

#else	/* DONT_USE_NAPA_CAM_ATATASK */

/*
 * Please keep fields Type to pCallback of both _CAM_ATATASK and
 * _CAM_XORTASK same. This is critical to Shasta firmware.
 */
struct _CAM_ATATASK
{
	DEFINE_CAM_TASK_HEADER

#if defined(CAM_ATAPI_SUPPORT)
	bpVOID			pOriginalReq;	/* upper layers used only, CAM should not
											 * touch it */
#endif	/*CAM_ATAPI_SUPPORT*/

//R01-S
#if defined(SUPPORT_PASSTHROUGH)
	U8			DevHead;
	U8			bReserved_PASSTHROUGH[3];
#endif
//R01-E
	U16			CmdFlags;			/* see below ATACMD_F_xxx */
	U8			DataBufferFlags;		/* see below ATABUF_F_xxx */
	U8			DiskID;

	U8			AtaStatus;			/* OUT valid on return if
								 * CamAtaStatus==CAMATA_STATUS_ATAERROR */
	U8			TimeoutValue;			/* IN timeout value in seconds, 0 for default */
	U8			CamAtaStatus;			/* OUT see above CAMATA_STATUS_xxx */
	U8			CmdTag;

#if 0
	/* 
	 * SataStatus and SataError are obsolete to save task size. Upper layers never will use them,
	 * and actually no CAM implementation fill them.
	 */
	U32				SataStatus;				/* OUT valid on return if
											 * CamAtaStatus==CAMATA_STATUS_SATAERROR */
	U32				SataError;				/* OUT valid on return if
											 * CamAtaStatus==CAMATA_STATUS_SATAERROR */
#endif

	/* ATA task file register start */

	/*
	 * ATAPI support extension:
	 * The following fields will be used by ATAPI command also, so please keep position of Lba 
	 * unchanged and make sure this block size is at least the sizeof(CAM_ATAPI_CMD_BLOCK), which
	 * is defined below.
	 *
	 * To keep existing code untouch, I didn't define a union to support ATAPI here. anonymous 
	 * union/struct is not supported by some compilers.
	 * Use the following compile time assertion to check this restriction.
	 *
	 * OSD_CASSERT( OFFSET_OF(CAM_ATATASK,Pad) - OFFSET_OF(CAM_ATATASK,Lba) + sizeof(CAM_ATATASK.Pad) 
	 *              >= sizeof(CAM_ATAPI_CMD_BLOCK) );
	 *
	 */
	LBA			Lba;
	U16			Feature;
	U16			SectorCount;
	
	U8			Command;
	U8			Control;
#if defined(CAM_ATAPI_SUPPORT)
	U8			Pad[6];
#else
	U8			bReserved1[2];
#endif	/*CAM_ATAPI_SUPPORT*/
	/* ATA task file register end */
	U32			ByteOffset;
	U32			DataTransferLength;

	/* Data buffer is controlled by previous DataBufferFlags */
	union
	{
		LPVOID 		DataBufferAddress;
		SG_INFO 	SGInfo;			/* SG32 or SG64 format depends on
							 * DataBufferFlags */
	} sg;

	/* Cam Internal use, other component should not access this field */
	U32			RetryCnt;
	bpVOID			pPackHead;
#if !defined(NAPA_ASIC)
	DIMMADDR		DimmPackAddr;
#endif
	/* Cam Internal end */
#if !defined( _X8616B )/*============================================================*/
	spVOID					AddressTranslationContext;
#endif/*============================================================*/

#if defined(SORTED_TASK_QUEUE)
	U16 Age;				/* Use for sorting to avoid starving */
	U16 Reserved;
#endif

};

#endif	/* DONT_USE_NAPA_CAM_ATATASK */

#define MAX_XOR_STRAMS	8

/* XOR Mode related definitions */
#define CAM_XORMODE_DMA		0x00
#define CAM_XORMODE_XOR		0x01
#define CAM_XORMODE_FILL	0x02
#define CAM_XORMODE_PARITY	0x04
#define CAM_XORMODE_MASKINT	0x08

#if defined(_FIRMWARE_VER)
#define CAM_XORMODE_EMERGENT	0x10
#endif

/* definitions for XORStatus */
#define XOR_NOERROR			0x00
#define XOR_ERROR				0x01		/* indicate this XOR_TASK is failed */
#define XOR_ABORT				0x02		/* indicate this XOR_TASK is aborted */
#define XOR_TIMEOUT				0x04		/* indicate this XOR_TASK timeout */
#define XORERR_COMPARE			0x20		/* indicate this XOR_TASK found compare error */
#define XORERR_HARDECC			0x40		/* indicate uncorrectable ECC error */
#define XORERR_SOFTECC			0x80		/* indicate correctable ECC error */

#define CAM_XOR_STATUS_CMPERR	XORERR_COMPARE
#define CAM_XOR_STATUS_OK		XOR_NOERROR

/* definitions for HDMAStatus */
#define HDMA_NOERROR			0x00
#define HDMA_ERROR				0x01			/* indicate this HDMA_TASK is failed */
#define HDMA_ABORT				0x02			/* indicate this HDMA_TASK is aborted */
#define HDMA_TIMEOUT			0x04			/* indicate this HDMA_TASK timeout */
#define HDMA_HOST_READ		0x08			/* only used in HDMA TASK to indicate DMA
											 * direction */
#define HDMAERR_PCI_SYSTEM	0x10			/* indicate this HDMA_TASK HW bus error */
#define HDMAERR_PCI_PARITY		0x20			/* indicate this HDMA_TASK PCI parity error */
#define HDMAERR_HARDECC		0x40		/* indicate uncorrectable ECC error */
#define HDMAERR_SOFTECC		0x80		/* indicate correctable ECC error */

#define CAM_XOR_TASK_RETRY_CNT 1

struct _CAM_XORTASK
{
	DEFINE_CAM_TASK_HEADER

	bpVOID					pOriginalReq;	/* upper layers used only, CAM should not
											 * touch it */
	U8						XorMode;		/* see CAM_XORMODE_xxx below */
	U8						XorStatus;		/* CAM should fill correct status here, at
											 * the same time, CAM should fill XorOffset
											 * below */
	U8						SourceSGCount;	/* indicates pSG size below. For HDMA
											 * packets, it should be 1 */
	U8						CmdTag;
	U32						XorOffset;		/* When XOR/Compare is aborted because of
											 * error, CAM should fill the offset where is
											 * aborted, so upper layers know where the
											 * error is */
#if !defined( _X8616B )/*============================================================*/
	spVOID					AddressTranslationContext;
#endif/*============================================================*/

	/* Cam Internal use, other component should not access this field */
	U32						MaskInt;
	U32 					RetryCnt;   
	bpVOID					pPackHead;

#if !defined(NAPA_ASIC)
	DIMMADDR				DimmPackAddr;
#else
	U32						TaskLoad;
#endif

#if defined(_SHASTAFW)
	/*For recorde Next DC address for chainning */
	U32                     DCCount;
	CAM_TASK_CALLBACK		pSavedCallback;
#endif /*_SHASTAFW*/

#if defined(MOONFANG)
	struct _gf_obj_t		*GF;
#endif

	/* Cam internal end */
	SG_INFO					DstSGInfo;
	SG_INFO					SrcSGInfo[1];
};

/*
 =========================================================================================
    CAM_XORTASK is a variable length structure, use CAM_XOR_TASK_SIZE to determine a XOR
    packet size
 =========================================================================================
 */
#define CAM_XOR_TASK_SIZE( sgCount ) \
		( OFFSET_OF(CAM_XORTASK, SrcSGInfo) + sizeof(SG_INFO) * (sgCount) )

typedef struct _CAM_ATAPI_CMD_BLOCK
{
	U8	Cdb[16];	/* ATAPI command is either 12 or 16 bytes */
	U8	CdbLength;
	U8	bReserved[3];
} CAM_ATAPI_CMD_BLOCK, buf_space* bpCAM_ATAPI_CMD_BLOCK; 

/*
 * To get the CAM_ATAPI_CMD_BLOCK from CAM_ATATASK:
 *
 * (bpCAM_ATAPI_CMD_BLOCK) ( ((bpU8)(task)) + OFFSET_OF(CAM_ATATASK,Lba) )
 */

#endif /* CAM_TASK_H */
