#if !defined(CC_EXPORT_H)
#define CC_EXPORT_H


#include "gen_typs.h"
#include "eng_type.h"
#include "osd_exp.h"

#if defined(USE_EXPORT)
#define PBM_EXP_API(funcName) EXP_CC_##funcName
#else
#define PBM_EXP_API(funcName) CC_##funcName
#endif

#define DECLARE_PBM_EXP(funcName) GEN_EXPORT PBM_EXP_API(funcName)

/**
	CC exported common interface
 */
void DECLARE_PBM_EXP(ComponentGetExtensionSize)(
	OUT bpMEMORY_REQUIREMENT pMemRequirement 
);

void DECLARE_PBM_EXP(ComponentGetDMASafeMemoryRequirement)( 
	IN U32 VenDevID,
	IN U32 SubVenDevID,
	OUT bpMEMORY_REQUIREMENT pMemRequirement 
);

void DECLARE_PBM_EXP(ComponentGetDIMMRequirement)( 
	IN U32 VenDevID,
	IN U32 SubVenDevID,
	IN OUT bpMEMORY_REQUIREMENT pMemRequirement 
);

void DECLARE_PBM_EXP(ComponentInitialize)(
	IN bpVOID ComponentExtension,
	IN bpMEMORY_AVAILABLE pMemAvail, 
	IN U32 VenDevID,
	IN U32 SubVenDevID,
	IN bpVOID ComponentSpecificContext
);

void DECLARE_PBM_EXP(ComponentStart)(
	IN bpVOID ComponentExtension,
	IN DIMMADDR DIMMOffset,
	IN U32 DIMMSize
);

U32 DECLARE_PBM_EXP(ComponentState)(
	IN bpVOID ComponentExtension
);

void DECLARE_PBM_EXP(ComponentStop)(
	IN bpVOID ComponentExtension
);

void DECLARE_PBM_EXP(ComponentRestart)(
	IN bpVOID ComponentExtension
);

void DECLARE_PBM_EXP(ComponentBist)(
	IN bpVOID ComponentExtension,
	IN U32 BistFlag
);

void DECLARE_PBM_EXP(SubmitRAID)(
	IN bpVOID CCComponentExtension,
	IN bpREQ p_rq
);

void DECLARE_PBM_EXP(OnLogicalDeviceRemoval) (
	bpVOID CCComponentExtension,
	U8 LogicalDeviceID
);

void DECLARE_PBM_EXP(OnLogicalDeviceArrival) (
	bpVOID CCComponentExtension,
	U8 LogicalDeviceID
);

void DECLARE_PBM_EXP(OnLogicalDeviceChanged) (
	IN bpVOID CCComponentExtension,
	IN U8 LogicalDeviceID
);

#if !defined(BIOS)
void DECLARE_PBM_EXP(ComponentReset)( 
	IN bpVOID ComponentExtension 
);
#endif	/*!BIOS*/

#if !defined(BIOS)
void DECLARE_PBM_EXP(OnBroadcasting)(
	IN bpVOID PBMComponentExtension,
	IN U32 BroadcastType,
	IN PTR_SIZE	Param1,
	IN PTR_SIZE	Param2
);
#endif	/*!BIOS*/

/*=====================================================================================*/
/* PBM Export Function Table Definition */
#if defined(USE_EXPORT)

#define DEFINE_PBM_EXP_FUNC(funcName)	(GEN_EXPORT* PBM_EXP_API(funcName))

typedef struct
{
	void DEFINE_PBM_EXP_FUNC(ComponentGetExtensionSize)(
		OUT bpMEMORY_REQUIREMENT pMemRequirement 
	);

	void DEFINE_PBM_EXP_FUNC(ComponentGetDMASafeMemoryRequirement)( 
		IN U32 VenDevID,
		IN U32 SubVenDevID,
		OUT bpMEMORY_REQUIREMENT pMemRequirement 
	);

	void DEFINE_PBM_EXP_FUNC(ComponentGetDIMMRequirement)( 
		IN U32 VenDevID,
		IN U32 SubVenDevID,
		IN OUT bpMEMORY_REQUIREMENT pMemRequirement 
	);

	void DEFINE_PBM_EXP_FUNC(ComponentInitialize)(
		IN bpVOID ComponentExtension,
		IN bpMEMORY_AVAILABLE pMemAvail, 
		IN U32 VenDevID,
		IN U32 SubVenDevID,
		IN bpVOID ComponentSpecificContext
	);

	void DEFINE_PBM_EXP_FUNC(ComponentStart)(
		IN bpVOID ComponentExtension,
		IN DIMMADDR DIMMOffset,
		IN U32 DIMMSize
	);

	void DEFINE_PBM_EXP_FUNC(ComponentStop)(
		IN bpVOID ComponentExtension
	);

	void DEFINE_PBM_EXP_FUNC(ComponentRestart)(
		IN bpVOID ComponentExtension
	);

	void DEFINE_PBM_EXP_FUNC(ComponentBist)(
		IN bpVOID ComponentExtension,
		IN U32 BistFlag
	);

	void DEFINE_PBM_EXP_FUNC(SubmitRAID)(
		IN bpVOID PBMComponentExtension,
		IN bpREQ p_rq
	);

	void DEFINE_PBM_EXP_FUNC(OnLogicalDeviceRemoval) (
		bpVOID PBMComponentExtension,
		U8 LogicalDeviceID
	);

	void DEFINE_PBM_EXP_FUNC(OnLogicalDeviceArrival)(
		bpVOID PBMComponentExtension,
		U8 LogicalDeviceID
	);

	void DEFINE_PBM_EXP_FUNC(OnLogicalDeviceChanged) (
		IN bpVOID CCComponentExtension,
		IN U8 LogicalDeviceID
	);

#if !defined(BIOS)
	void DEFINE_PBM_EXP_FUNC(ComponentReset)( 
		IN bpVOID ComponentExtension 
	);
#endif	/*!BIOS*/

#if !defined(BIOS)
	void DEFINE_PBM_EXP_FUNC(OnBroadcasting)(
		IN bpVOID PBMComponentExtension,
		IN U32 BroadcastType,
		IN PTR_SIZE	Param1,
		IN PTR_SIZE	Param2
	);
#endif	/*!BIOS*/

} PBM_EXPORT_FUNC_TABLE, buf_space * bpPBM_EXPORT_FUNC_TABLE;

extern bpPBM_EXPORT_FUNC_TABLE PSEUDOBASE g_pPbmApi;

#define PBM_ComponentGetExtensionSize				g_pPbmApi->PBM_EXP_API(ComponentGetExtensionSize)
#define PBM_ComponentGetDMASafeMemoryRequirement	g_pPbmApi->PBM_EXP_API(ComponentGetDMASafeMemoryRequirement)
#define PBM_ComponentGetDIMMRequirement				g_pPbmApi->PBM_EXP_API(ComponentGetDIMMRequirement)
#define PBM_ComponentInitialize						g_pPbmApi->PBM_EXP_API(ComponentInitialize)
#define PBM_ComponentStart							g_pPbmApi->PBM_EXP_API(ComponentStart)
#define PBM_ComponentStop							g_pPbmApi->PBM_EXP_API(ComponentStop)
#define PBM_ComponentRestart						g_pPbmApi->PBM_EXP_API(ComponentRestart)
#define PBM_ComponentBist							g_pPbmApi->PBM_EXP_API(ComponentBist)
#define PBM_SubmitRAID								g_pPbmApi->PBM_EXP_API(SubmitRAID)
#define PBM_OnLogicalDeviceRemoval					g_pPbmApi->PBM_EXP_API(OnLogicalDeviceRemoval)
#define PBM_OnLogicalDeviceArrival					g_pPbmApi->PBM_EXP_API(OnLogicalDeviceArrival)
#define PBM_OnLogicalDeviceChanged					g_pPbmApi->PBM_EXP_API(OnLogicalDeviceChanged)
#if !defined(BIOS)
#define PBM_ComponentReset							g_pPbmApi->PBM_EXP_API(ComponentReset)
#define PBM_OnBroadcasting							g_pPbmApi->PBM_EXP_API(OnBroadcasting)
#endif

#else /*USE_EXPORT*/

#define PBM_ComponentGetExtensionSize				CC_ComponentGetExtensionSize
#define PBM_ComponentGetDMASafeMemoryRequirement	CC_ComponentGetDMASafeMemoryRequirement
#define PBM_ComponentGetDIMMRequirement				CC_ComponentGetDIMMRequirement
#define PBM_ComponentInitialize						CC_ComponentInitialize
#define PBM_ComponentStart							CC_ComponentStart
#define PBM_ComponentState							CC_ComponentState
#define PBM_ComponentStop							CC_ComponentStop
#define PBM_ComponentRestart						CC_ComponentRestart
#define PBM_ComponentBist							CC_ComponentBist
#define PBM_SubmitRAID								CC_SubmitRAID
#define PBM_OnLogicalDeviceRemoval					CC_OnLogicalDeviceRemoval
#define PBM_OnLogicalDeviceArrival					CC_OnLogicalDeviceArrival
#define PBM_OnLogicalDeviceChanged					CC_OnLogicalDeviceChanged
#if !defined(BIOS)
#define PBM_ComponentReset							CC_ComponentReset
#define PBM_OnBroadcasting							CC_OnBroadcasting
#endif

#endif /*USE_EXPORT*/

#endif	/* CC_EXPORT_H */
