#if !defined(ENGINE_EXPORT_H)
#define ENGINE_EXPORT_H

#include "eng_type.h"

/*=====================================================================================*/
/*
 * Data struture and definition for Engine_GetLogicalDeviceInformation
 */
typedef struct _LOGICAL_DEVICE
{
	U8 Flags;				/* see below LD_F_xxx */
	U8 CacheMode;			/* see below LD_CM_xxx */
	U8 DiskWriteCache;		/* 0 for disk write back, 1 for write through */
	U8 DiskReadLookahead;	/* 1 enable, 0 disable disk lookahead */
	U8 RaidLevel;
	U8 StripeBlockPower;
	U16 StripeSize;
	LBA Size;
	U8 Lookahead;
	U8 Consolidation;
	U8 bReserved[2];
} LOGICAL_DEVICE, *PLOGICAL_DEVICE, buf_space* bpLOGICAL_DEVICE;

#define LD_F_VALID			1

#define LD_CM_WRITEBACK		0x00
#define LD_CM_WRITETHRU		0x01
#define LD_CM_BYPASS		0x02
#define LD_CM_AUTO			0x03	/* writeback/writethrough according to battery healthy */
/*=====================================================================================*/

/*=====================================================================================*/


/**
	Engine exported common interface
 */

#if defined(USE_EXPORT)
#define ENG_EXP_API(funcName) EXP_ENG_##funcName
#else
#define ENG_EXP_API(funcName) Engine_##funcName
#endif

#define DECLARE_ENG_EXP(funcName) GEN_EXPORT ENG_EXP_API(funcName)

void DECLARE_ENG_EXP(ComponentGetExtensionSize)(
	OUT bpMEMORY_REQUIREMENT pMemRequirement 
);

void DECLARE_ENG_EXP(ComponentGetDMASafeMemoryRequirement)( 
	IN U32 VenDevID,
	IN U32 SubVenDevID,
	OUT bpMEMORY_REQUIREMENT pMemRequirement 
);

void DECLARE_ENG_EXP(ComponentGetDIMMRequirement)( 
	IN U32 VenDevID,
	IN U32 SubVenDevID,
	OUT bpMEMORY_REQUIREMENT pMemRequirement 
);

typedef struct _ENGINE_INIT_CONFIG
{
	BoOL JbodOnly;
} ENGINE_INIT_CONFIG, buf_space* bpENGINE_INIT_CONFIG;

void DECLARE_ENG_EXP(ComponentInitialize)(
	IN bpVOID ComponentExtension,
	IN bpMEMORY_AVAILABLE pMemAvail, 
	IN U32 VenDevID,
	IN U32 SubVenDevID,
	IN bpVOID ComponentSpecificContext
);

void DECLARE_ENG_EXP(ComponentStart)(
	IN bpVOID ComponentExtension,
	IN DIMMADDR DIMMOffset,
	IN U32 DIMMSize
);

U32 DECLARE_ENG_EXP(ComponentState)(
	IN bpVOID ComponentExtension
);

void DECLARE_ENG_EXP(ComponentStop)(
	IN bpVOID ComponentExtension
);

void DECLARE_ENG_EXP(ComponentRestart)(
	IN bpVOID ComponentExtension
);

void DECLARE_ENG_EXP(ComponentBist)(
	IN bpVOID ComponentExtension,
	IN U32 BistFlag
);

bpLOGICAL_DEVICE DECLARE_ENG_EXP(GetLogicalDeviceInformation)(
	IN bpVOID EngineComponentExtension,
	IN U8 LogicalDeviceID,
	IN OUT PLOGICAL_DEVICE LogicalDevice
);

#if defined(ENGINE_LOG_ERROR_EVENT)
void DECLARE_ENG_EXP(LogEvent)(
	IN bpVOID EngineComponentExtension,
	IN U32 EventID,
	IN U32 EventParamCount,
	IN ...
);
#else	/*ENGINE_LOG_ERROR_EVENT*/
#define Engine_LogEvent	
#endif	/*ENGINE_LOG_ERROR_EVENT*/

void DECLARE_ENG_EXP(SubmitRAID)(
	IN bpVOID EngineComponentExtension,
	IN bpREQ p_rq
);

void DECLARE_ENG_EXP(OnPhysicalDiskArrival)(
	IN bpVOID EngineComponentExtension,
	IN U32 DiskIDMap
);

void DECLARE_ENG_EXP(OnPhysicalDiskRemoval)(
	IN bpVOID EngineComponentExtension,
	IN U32 DiskIDMap
);

void DECLARE_ENG_EXP(OnBroadcasting)(
	IN bpVOID EngineComponentExtension,
	IN U32 BroadcastType,
	IN PTR_SIZE	Param1,
	IN PTR_SIZE	Param2
);

/**
	I finally decided to expose engine internal array structure to outside, especially for
	PBM. Even that, other component should NOT modify the array information manually!!!
 */
bpARY DECLARE_ENG_EXP(GetArrayInfo)(
	IN bpVOID EngineComponentExtension,
	IN U8 ArrayID
	);

void DECLARE_ENG_EXP(InitializeREQ)(
	IN bpVOID EngineComponentExtension,
	IN bpREQ p_rq
	);

#if defined(ENGINE_LOG_ERROR_EVENT)
BoOL DECLARE_ENG_EXP(GetEvent)(
	IN bpVOID			EngineComponentExtension,
	IN OUT bpLOGEVENT	p_logev );
#endif

bpBLOCK DECLARE_ENG_EXP(GetBlockInfo)(
	IN bpVOID EngineComponentExtension,
	IN U8 BlockID
	);

BoOL DECLARE_ENG_EXP(GetMaximumAvailableArraySize) (
	IN bpVOID	EngineComponentExtension,
	IN U32		DiskIDMap,
	IN U8		RaidLevel,
	OUT LBA		*MaximumSize
	);

#if !defined(BIOS)
void DECLARE_ENG_EXP(ComponentReset)( IN bpVOID ComponentExtension );
#endif	/*!BIOS*/

/*=====================================================================================*/

/* Engine Export Function Table Definition */
#if defined(USE_EXPORT)

#define DEFINE_ENG_EXP_FUNC(funcName)	(GEN_EXPORT* ENG_EXP_API(funcName))

typedef struct
{
	void DEFINE_ENG_EXP_FUNC(ComponentGetExtensionSize)(
		OUT bpMEMORY_REQUIREMENT pMemRequirement 
	);

	void DEFINE_ENG_EXP_FUNC(ComponentGetDMASafeMemoryRequirement)( 
		IN U32 VenDevID,
		IN U32 SubVenDevID,
		OUT bpMEMORY_REQUIREMENT pMemRequirement 
	);

	void DEFINE_ENG_EXP_FUNC(ComponentGetDIMMRequirement)( 
		IN U32 VenDevID,
		IN U32 SubVenDevID,
		OUT bpMEMORY_REQUIREMENT pMemRequirement 
	);

	void DEFINE_ENG_EXP_FUNC(ComponentInitialize)(
		IN bpVOID ComponentExtension,
		IN bpMEMORY_AVAILABLE pMemAvail, 
		IN U32 VenDevID,
		IN U32 SubVenDevID,
		IN bpVOID ComponentSpecificContext
	);

	void DEFINE_ENG_EXP_FUNC(ComponentStart)(
		IN bpVOID ComponentExtension,
		IN DIMMADDR DIMMOffset,
		IN U32 DIMMSize
	);

	U32 DEFINE_ENG_EXP_FUNC(ComponentState)(
		IN bpVOID ComponentExtension
	);

	void DEFINE_ENG_EXP_FUNC(ComponentStop)(
		IN bpVOID ComponentExtension
	);

	void DEFINE_ENG_EXP_FUNC(ComponentRestart)(
		IN bpVOID ComponentExtension
	);

	void DEFINE_ENG_EXP_FUNC(ComponentBist)(
		IN bpVOID ComponentExtension,
		IN U32 BistFlag
	);

	bpLOGICAL_DEVICE DEFINE_ENG_EXP_FUNC(GetLogicalDeviceInformation)(
		IN bpVOID EngineComponentExtension,
		IN U8 LogicalDeviceID,
		IN OUT PLOGICAL_DEVICE LogicalDevice
	);

#if defined(ENGINE_LOG_ERROR_EVENT)
	void DEFINE_ENG_EXP_FUNC(LogEvent)(
		IN bpVOID EngineComponentExtension,
		IN U32 EventID,
		IN U32 EventParamCount,
		IN ...
	);
#endif

	void DEFINE_ENG_EXP_FUNC(SubmitRAID)(
		IN bpVOID EngineComponentExtension,
		IN bpREQ p_rq
	);

	void DEFINE_ENG_EXP_FUNC(OnPhysicalDiskArrival)(
		IN bpVOID EngineComponentExtension,
		IN U32 DiskIDMap
	);

	void DEFINE_ENG_EXP_FUNC(OnPhysicalDiskRemoval)(
		IN bpVOID EngineComponentExtension,
		IN U32 DiskIDMap
	);

	void DEFINE_ENG_EXP_FUNC(OnBroadcasting)(
		IN bpVOID EngineComponentExtension,
		IN U32 BroadcastType,
		IN PTR_SIZE	Param1,
		IN PTR_SIZE	Param2
	);

	bpARY DEFINE_ENG_EXP_FUNC(GetArrayInfo)(
		IN bpVOID EngineComponentExtension,
		IN U8 ArrayID
		);

	bpBLOCK DEFINE_ENG_EXP_FUNC(GetBlockInfo)(
		IN bpVOID EngineComponentExtension,
		IN U8 BlockID
		);

	BoOL DEFINE_ENG_EXP_FUNC(GetMaximumAvailableArraySize) (
		IN bpVOID	EngineComponentExtension,
		IN U32		DiskIDMap,
		IN U8		RaidLevel,
		OUT LBA		*MaximumSize
		);

	void DEFINE_ENG_EXP_FUNC(InitializeREQ)(
		IN bpVOID EngineComponentExtension,
		IN bpREQ p_rq
		);

#if defined(ENGINE_LOG_ERROR_EVENT)
	BoOL DEFINE_ENG_EXP_FUNC(GetEvent)(
		IN bpVOID			EngineComponentExtension,
		IN OUT bpLOGEVENT	p_logev );
#endif

#if !defined(BIOS)
	void DEFINE_ENG_EXP_FUNC(ComponentReset)( 
		IN bpVOID ComponentExtension );
#endif	/*!BIOS*/

} ENG_EXPORT_FUNC_TABLE, buf_space * bpENG_EXPORT_FUNC_TABLE;

#ifdef _X8616B
EXTERNC bpENG_EXPORT_FUNC_TABLE  g_pEngineApi;
#else
extern bpENG_EXPORT_FUNC_TABLE PSEUDOBASE g_pEngineApi;
#endif

#define Engine_ComponentGetExtensionSize			g_pEngineApi->ENG_EXP_API(ComponentGetExtensionSize)
#define Engine_ComponentGetDMASafeMemoryRequirement	g_pEngineApi->ENG_EXP_API(ComponentGetDMASafeMemoryRequirement)
#define Engine_ComponentGetDIMMRequirement			g_pEngineApi->ENG_EXP_API(ComponentGetDIMMRequirement)
#define Engine_ComponentInitialize					g_pEngineApi->ENG_EXP_API(ComponentInitialize)
#define Engine_ComponentStart						g_pEngineApi->ENG_EXP_API(ComponentStart)
#define Engine_ComponentStop						g_pEngineApi->ENG_EXP_API(ComponentStop)
#define Engine_ComponentRestart						g_pEngineApi->ENG_EXP_API(ComponentRestart)
#define Engine_ComponentBist						g_pEngineApi->ENG_EXP_API(ComponentBist)
#define Engine_GetLogicalDeviceInformation			g_pEngineApi->ENG_EXP_API(GetLogicalDeviceInformation)
#if defined(ENGINE_LOG_ERROR_EVENT)
#define Engine_LogEvent								g_pEngineApi->ENG_EXP_API(LogEvent)
#endif	/*ENGINE_LOG_ERROR_EVENT*/
#define Engine_SubmitRAID							g_pEngineApi->ENG_EXP_API(SubmitRAID)
#define Engine_OnPhysicalDiskArrival				g_pEngineApi->ENG_EXP_API(OnPhysicalDiskArrival)
#define Engine_OnPhysicalDiskRemoval				g_pEngineApi->ENG_EXP_API(OnPhysicalDiskRemoval)
#define Engine_OnBroadcasting						g_pEngineApi->ENG_EXP_API(OnBroadcasting)
#define Engine_GetArrayInfo							g_pEngineApi->ENG_EXP_API(GetArrayInfo)
#define Engine_GetBlockInfo							g_pEngineApi->ENG_EXP_API(GetBlockInfo)
#define Engine_InitializeREQ						g_pEngineApi->ENG_EXP_API(InitializeREQ)
#if defined(ENGINE_LOG_ERROR_EVENT)
#define Engine_GetEvent								g_pEngineApi->ENG_EXP_API(GetEvent)
#endif
#define Engine_GetMaximumAvailableArraySize			g_pEngineApi->ENG_EXP_API(GetMaximumAvailableArraySize)
#if !defined(BIOS)
#define Engine_ComponentReset						g_pEngineApi->ENG_EXP_API(ComponentReset)
#endif

#endif	/* USE_EXPORT */

#endif /* ENGINE_EXPORT_H */
