/* Engine internal definitions */
#if !defined( ENGINE_INTERNAL_H )
#define ENGINE_INTERNAL_H

#include "eng_type.h"
#include "eng_con.h"
#include "gen_ds.h"
#include "eng_scsi.h"

#if defined(MOONFANG)
#include "moonfang/type.h"
#endif

//extern U8 g_WorkingLDNum;

#define VALID_WATERMARK( watermark ) \
		( ((watermark).u.Low != U32_MAX) || ((watermark).u.High != U32_MAX) )

/*
 -----------------------------------------------------------------------------------------
    (REF_01_29_2004_01) Block ID and Disk ID: We mixed disk ID and block ID before. It's
    time to make it clear. A disk ID represents a physical disk while a block ID
    represents a smallest unit that engine treats as a trunk of data block. A disk/block
    is similar to the conception of disk/partition in the reality world. Currently we
    just support one block per disk so usually Disk ID is duplicated with Block ID. But
    this is supposed to be changed soon. So use them carefully to avoid trouble.
 -----------------------------------------------------------------------------------------
 */
typedef struct	diskRTParam
{
	U8	blDiskFlag;		/* blDISK_xxx below */
	U8	smartStatus;	/* SMART_STATUS_xxx below */
	U8	bQueueCount;
	U8	bMDDFormat;
	U8	DiskWriteCacheEnable;
	U8	bPortId;
	U8	bDeviceId;
	U8	bReserved;
	U8   DiskNCQTCQStatus;
	U8	bReserved2[3];
#if !defined(_X8616B) && !defined(SIMULATE_BIOS_ENV)
	LBA wCurrentLBA;	/* activity information for RAID1/10 load balance */
#endif
}
DISKRTPARAM, bpDISKRTPARAM;

/* DISKRTPARAM.blDiskFlag */
#define blDISK_IS_PRESENT_HERE			bit( 0 )	/* disk presented */
#define blDISK_HAS_NO_SIGNATURE			bit( 1 )	/* there is no MDD signature in the
													 * disk */
#define blDISK_SETDOWN_BY_ERROR			bit( 3 )
#define blDISK_SLEEPING					bit( 4 )
#define blDISK_ATAPI					bit( 5 )
#define blDISK_TAPE					bit( 6 )

#define GetQueueCount( This, DiskID )		( (This)->diskParam[DiskID].bQueueCount )
#define GetDiskFlag( This, DiskID )			( (This)->diskParam[DiskID].blDiskFlag )
#if !defined(_X8616B) && !defined(SIMULATE_BIOS_ENV)
#define GetDiskCurrentLBA( This, DiskID )	( (This)->diskParam[DiskID].wCurrentLBA )
#endif
#define GetDiskMDDFormat( This, DiskID )	( (This)->diskParam[DiskID].bMDDFormat )
#define DiskHasSignature( This, DiskID ) \
		( (GetDiskFlag(This, DiskID) & blDISK_HAS_NO_SIGNATURE) == 0 )
#define DiskPresent( This, DiskID )			( GetDiskFlag(This, DiskID) & blDISK_IS_PRESENT_HERE )
#define DiskSleeping( This, DiskID )		( GetDiskFlag(This, DiskID) & blDISK_SLEEPING )
#define DiskOkForAccessing( This, DiskID ) \
		( (GetDiskFlag(This, DiskID) & (blDISK_IS_PRESENT_HERE | blDISK_SETDOWN_BY_ERROR)) == blDISK_IS_PRESENT_HERE )
#define GetDiskSmartStatus( This, DiskID )	( (This)->diskParam[DiskID].smartStatus )
#define GetDiskNCQTCQStatus( This, DiskID ) ( (This)->diskParam[DiskID].DiskNCQTCQStatus )
#define GetDiskCacheStatus( This, DiskID )	( (This)->diskParam[DiskID].DiskWriteCacheEnable )
#define GetDiskPortId( This, DiskID )		( (This)->diskParam[DiskID].bPortId )
#define GetDiskDeviceId( This, DiskID )		( (This)->diskParam[DiskID].bDeviceId )
#define DiskIsAtapi( This, DiskID )			( GetDiskFlag(This, DiskID) & blDISK_ATAPI )
#define DiskIsTAPE( This, DiskID )			( GetDiskFlag(This, DiskID) & blDISK_TAPE )

#if !defined(DISABLE_JBOD)
#define DiskHasMDD( This, DiskID )			(DiskHasSignature(This,DiskID) && (GetDiskMDDFormat(This,DiskID)!=MDD_FORMAT_JBOD))
#else
#define DiskHasMDD( This, DiskID )			(DiskHasSignature(This,DiskID))
#endif

/*
 * VSERIAL_SIZE is different from SERIAL_SIZE, SERIAL_SIZE is for V2 MDD format
 * only, VSERIAL_SIZE is MDD format independently, dont suppose VSERIAL_SIZE is
 * always equal to SERIAL_SIZE
 */
#define VSERIAL_SIZE	24

typedef struct	strm
{
	U16 blFlag;						/* same as BLOCK.blFlag */
	U8	bIndex;						/* block index, used to reference blkDisk */
	U8	bRedirectIndex;				/* used by RAID1(10) to specify the mirroring stream
									 * ID in the array */
}
STRM, buf_space * bpSTRM, dat_space * dpSTRM;

/* block.blFlag TODO: clean and re-organize following flags */
#define blVALID_BLK		bit( 0 )	/* all other bit should be ignored if blVALID_BLK is
									 * not set */
#define blWORKING_BLK	bit( 1 )
#define blASSIGNED_BLK	bit( 2 )	/* the block is assigned to an array */
#define blSPARE_BLK		bit( 3 )	/* a spare block */

/* bit(4) is reserved to keep compatibility with blDUPLICATE_BLK defined in V2 */
#define blREDIRECT_BLK	bit( 5 )	/* rebuilding target?? */
#define blMARK_DOWN_BLK bit( 6 )	/* block was once set down because of error */

#if defined(ENABLE_BACKUP)
#define blBACKUP_BLK	bit( 7 )	/* Backup target */
#endif
/* bits starting from bit(8) are Napa specific and will not be saved into MDD */
#define blUNRECOGNIZED_BLK	bit( 8 )	/* when blUNRECOGNIZED_BLK is set, all other bits
										 * except blVALID_BLK should be cleared */

/*
 * blMDD_OVERRIDDEN_BLK is a temporary flag that will be set/cleared during
 * configuration checking.
 */
#define blMDD_OVERRIDDEN_BLK	bit( 9 )
#define blSLEEPING_BLK			bit( 10 )
#define blUPDATE_MDD_REQUIRED	bit( 11 )
#define blDELETING_BLK			bit( 12 )
#define blBLOCKING_BLK			bit( 13 )	/*Reserved for Shasta/Aurora */
#if defined(ENABLE_BACKUP)
#define blLATEST_BACKUP			bit( 15 )
#endif

/*
 =========================================================================================
    end of BLOCK/STRM flags
 =========================================================================================
 */
#define IS_WORKING_BLOCK( blkFlag )											 \
		(																	 \
			((blkFlag) & (blVALID_BLK | blWORKING_BLK | blMARK_DOWN_BLK)) == \
				(blVALID_BLK | blWORKING_BLK)								 \
		)
#define IS_VALID_BLOCK( blkFlag )	( (blkFlag) & blVALID_BLK )
#if defined(ENABLE_BACKUP)
#define IS_FREE_WORKING_BLOCK( blkFlag ) \
		( ((blkFlag) & (blVALID_BLK | blWORKING_BLK | blMARK_DOWN_BLK | blASSIGNED_BLK | blBACKUP_BLK)) == (blVALID_BLK | blWORKING_BLK) )
#else
#define IS_FREE_WORKING_BLOCK( blkFlag ) \
		( ((blkFlag) & (blVALID_BLK | blWORKING_BLK | blMARK_DOWN_BLK | blASSIGNED_BLK)) == (blVALID_BLK | blWORKING_BLK) )
#endif
#define IS_ASSIGNED_WORKING_BLOCK( blkFlag ) \
		( ((blkFlag) & (blVALID_BLK | blWORKING_BLK | blMARK_DOWN_BLK | blASSIGNED_BLK)) == (blVALID_BLK | blWORKING_BLK | blASSIGNED_BLK) )
#define IS_SPARE_BLOCK( blkFlag ) \
		( ((blkFlag) & (blSPARE_BLK | blVALID_BLK)) == (blSPARE_BLK | blVALID_BLK) )
#define IS_FREE_BLOCK( blkFlag )	( ((blkFlag) & blASSIGNED_BLK) == 0 )
#define IS_ASSIGNED_BLOCK( blkFlag ) \
		( ((blkFlag) & (blASSIGNED_BLK | blVALID_BLK)) == (blASSIGNED_BLK | blVALID_BLK) )
#define IS_UNRECOG_BLOCK( blkFlag ) ( (blkFlag) & blUNRECOGNIZED_BLK )
#if defined(ENABLE_BACKUP)
#define IS_BACKUP_BLOCK( blkFlag ) ( (blkFlag) & blBACKUP_BLK )
#endif
/*
 -----------------------------------------------------------------------------------------
    BLOCK
 -----------------------------------------------------------------------------------------
 */
struct	block
{
	U16 blFlag;
	U8	bStream;						/* StreamID in an array, used to reference
										 * ary.strm. It will be used as array ID for a
										 * spare block. 0xff for a global spare block. */
	U8	bCh;							/* DiskID, since we abstract channel conception,
										 * channel ID is the same as disk ID now */
	U8	bID;							/* bID is now no longer master/slave as before, it
										 * now defined as block ID in the disk to support
										 * multiple blocks (extent) per disk */
	U8	bControl;						/* MDD save control flags (1<<SAVEMDD_C_xxx) */
	U8	bReserved[2];					/* alignment */
	U8	bSerial[VSERIAL_SIZE];
	LBA wStartLBA;						/* start offset LBA in the disk */
	LBA wSize;
	LBA wWatermark;						/* rebuild watermark, data after watermark is invalid */
	#if defined(DDF_INTEGRATED)
	U8	bForced_GUID;
	#endif
	#if defined(ENABLE_BACKUP)
	U32	bDate;
	U32	wReserved;
	BoOL bRecordTime;
	#endif
};

/* array.blRAID RAID level definitions */
#define blMIRROR					bit( 0 )
#define blPARITY					bit( 1 )
#define blDISTRIBUTED				bit( 2 )
#define blSPAN						bit( 3 )
#define blJBOD						bit( 4 )
#define blRAID0						0
#define blRAID1						blMIRROR
#define blRAID4						blPARITY
#define blRAID5						blDISTRIBUTED
#define blRAID50					bit( 5 ) /* 0x20 */
#define blRAIDATAPI					bit( 6 )	/* 0x40 */
#define blRAIDTAPE					bit( 7 )	/* 0x40 */
#if defined( MOONFANG )
#define blRAID_MF					bit( 7 )
#endif

#define IS_RAID0( blRAID )			( (blRAID)==blRAID0 )
#define IS_RAID1( blRAID )			( (blRAID) & blRAID1 )
#define IS_RAIDSPAN( blRAID )		( (blRAID) & blSPAN )
#define IS_RAID45( blRAID )			( (blRAID) & (blRAID4 | blRAID5) )

#if defined(MOONFANG)
#define IS_RAIDREDUNDANT( blRAID )	( (blRAID) & (blRAID1 | blRAID4 | blRAID5 | blRAID_MF | blRAID50) )
#else
#define IS_RAIDREDUNDANT( blRAID )	( (blRAID) & (blRAID1 | blRAID4 | blRAID5) )
#endif

#if defined(ENABLE_RAID50)
#define IS_RAID50( blRAID )			( (blRAID) & (blRAID50) )
#endif	/* ENABLE_RAID50 */

#define IS_RAIDATAPI( blRAID )		( (blRAID) & blRAIDATAPI )
#define IS_RAIDTAPE( blRAID )		( (blRAID) & blRAIDTAPE )

#if defined( MOONFANG )
#define IS_RAIDMF( blRAID )			( (blRAID) & blRAID_MF )
#endif

#if defined(MOONFANG)
#define IS_RAIDPARITY( blRAID )		( (blRAID) & (blRAID4 | blRAID5 | blRAID_MF | blRAID50) )
#else
#define IS_RAIDPARITY( blRAID )		IS_RAID45( blRAID )
#endif

/* array.blFlag RAID status definitions */
#define blVALID_LUN			bit( 0 )	/* all other bit should be ignored if blVALID_LUN
										 * is not set */
#define blWORKING_LUN		bit( 1 )	/* working LUN */
#define blCONFIGURED_LUN	bit( 2 )	/* configured LUN, usually this bit will be set
										 * when engine initialization is done */
#define blARY_READY			bit( 3 )	/* array is ready for access */
#define blRECOVERING_LUN	bit( 4 )	/* array in critical mode */
#define blSUB_LUN			bit( 5 )
#define blCONV_EXP_LUN		bit( 6 )	/* array in migrating */
#ifdef MOONFANG
#define blDEGRADE_LUN		bit( 7 )	/* It's between critical and functional. At least one parity dropped but at least there is one more. */
#endif

/* flags starting from bit(8) will not be saved into MDD */
#define blPSEUDO_SPARE_LUN		bit( 8 )
#define blSTATUS_CHANGED_LUN	bit( 9 )
#define blSTOPPED_LUN			bit( 10 )
#define blINC_GEN_PENDING_LUN	bit( 11 )

/* helper array flag definitions */
#define blWORKING_LUN_FLAGS		( blVALID_LUN | blWORKING_LUN | blARY_READY | blCONFIGURED_LUN )
#define blCRITICAL_LUN_FLAGS	( blWORKING_LUN_FLAGS | blRECOVERING_LUN )
#ifdef MOONFANG
#define blDEGRADE_LUN_FLAGS		( blWORKING_LUN_FLAGS | blDEGRADE_LUN )
#endif

/*
 =========================================================================================
    helper macros to judge array status
 =========================================================================================
 */
#define IS_VALID_LUN( flag )	( (flag) & blVALID_LUN )
#define IS_WORKING_LUN( flag )	( ((flag) & blWORKING_LUN_FLAGS) == blWORKING_LUN_FLAGS )
#define IS_CRITICAL_LUN( flag ) \
		( ((flag) & blCRITICAL_LUN_FLAGS) == blCRITICAL_LUN_FLAGS )
#define IS_READY_TO_FULL_FUNCTIONAL_LUN( flag )											 \
		(																				 \
			((flag) & (blVALID_LUN | blWORKING_LUN | blARY_READY | blRECOVERING_LUN)) == \
				(blVALID_LUN | blWORKING_LUN | blARY_READY)								 \
		)
#ifndef MOONFANG
#define IS_FULL_FUNCTIONAL_LUN( flag ) \
		( ((flag) & blCRITICAL_LUN_FLAGS) == blWORKING_LUN_FLAGS )
#define IS_DEGRADE_LUN( flag ) \
		( FALSE )
#else
#define IS_FULL_FUNCTIONAL_LUN( flag ) \
		( (((flag) & blCRITICAL_LUN_FLAGS)==blWORKING_LUN_FLAGS) && (((flag) & blDEGRADE_LUN_FLAGS)==blWORKING_LUN_FLAGS) )
#define IS_DEGRADE_LUN( flag )	\
		( ((flag) & blDEGRADE_LUN_FLAGS) == blDEGRADE_LUN_FLAGS )
#endif
#define IS_MIGRATING_LUN( flag )	( (flag) & blCONV_EXP_LUN )
#define IS_CONFIGURED_LUN( flag )	( (flag) & blCONFIGURED_LUN )
#define IS_CONFIGURED_VALID_LUN( flag ) \
		( ((flag) & (blCONFIGURED_LUN | blVALID_LUN)) == (blCONFIGURED_LUN | blVALID_LUN) )
#define IS_STOPPED_LUN(flag)		( (flag) & blSTOPPED_LUN )

#if defined(ENABLE_BACKUP)
/* array.blExtendFlag definitions */
#define blBACKUPING_LUN				bit ( 0 ) /* array is backuping */
#define blBACKUPED_LUN				bit ( 1 ) /* array is finished-backup */
#define blBACKUP_LUN				bit ( 2 ) /* backup array */
#define blINVALID_FLAG 				bit ( 4 )

/* ExtendFlags starting from bit(8) will not be saved into MDD */
#define blUPDATE_MDD_REQUIRED_LUN	bit ( 8 ) /* update mdd required */
#define blACTIVE_BACKUP_LUN			bit ( 9 ) /* need to resume backup */
#define blPREVIOS_FASTCOPY_ENABLE		bit ( 10 )

#define IS_FASTCOPY_ENABLE_LASTTIME( p_ary )	( (p_ary)->blExtendFlag & blPREVIOS_FASTCOPY_ENABLE )
#define NEED_ACTIVE_BACKUP( p_ary )		( (p_ary)->blExtendFlag & blACTIVE_BACKUP_LUN )
#define UPDATE_MDD_REQUIED( p_ary )	( (p_ary)->blExtendFlag & blUPDATE_MDD_REQUIRED_LUN)
#define CLEAR_MDD_UPDATE( p_ary )		( (p_ary)->blExtendFlag &= ~blUPDATE_MDD_REQUIRED_LUN)
#define IS_BACKUPED_LUN( p_ary )		( (p_ary)->blExtendFlag & blBACKUPED_LUN )
#define IS_BACKUPING_LUN( p_ary )		( (p_ary)->blExtendFlag & blBACKUPING_LUN )
#define IS_BACKUP_LUN( p_ary )			( (p_ary)->blExtendFlag & blBACKUP_LUN )
#define IS_BACKUP_SOURCE( p_ary, p_blk )		(((p_ary)->blExtendFlag & blBACKUP_LUN) &&!((p_blk)->blFlag & blBACKUP_BLK))
#endif
/* array.bBGA BGA definitions */
#define blBGA_FOREGRNDINIT		bit( 0 )	/* array in foreground initialization */
#define blBGA_BACKGRNDINIT		bit( 1 )	/* array in background initialization */
#define blBGA_CONSISTENCYCHECK	bit( 2 )	/* array in consistency checking */
#define blBGA_SYNCHRONIZATION	bit( 3 )	/* array in synchronization */
#define blBGA_REBUILD			bit( 4 )	/* array in rebuilding (blRECOVERING_LUN or blDEGRADE_LUN must
											 * be set) */
#define blBGA_MIGRATION			bit( 5 )	/* array in migration (blCONV_EXP_LUN must be
											 * set) */
#if defined(ENABLE_BACKUP)
#define blBGA_BACKUP			bit( 6 )	/* array in backup*/
#endif
#define blBGA_FBINITIALIZED		bit( 7 )

/* helper macro to judge sync/init BGA */
#define blBGA_ANYSYNC				( blBGA_CONSISTENCYCHECK | blBGA_SYNCHRONIZATION )
#define blBGA_ANYINIT				( blBGA_FOREGRNDINIT | blBGA_BACKGRNDINIT )
#define HAS_FBINITED( p_ary )		( (p_ary)->bBGA & blBGA_FBINITIALIZED )
#define HAS_BGA( p_ary )			( (p_ary)->bBGA & (~blBGA_FBINITIALIZED) )
#define HAS_REBUILD( p_ary )		( (p_ary)->bBGA & blBGA_REBUILD )
#define HAS_SYNC( p_ary )			( (p_ary)->bBGA & blBGA_ANYSYNC )
#define HAS_SYNC_NORMAL( p_ary )	( (p_ary)->bBGA & blBGA_SYNCHRONIZATION )
#define HAS_SYNC_CHECK( p_ary )		( (p_ary)->bBGA & blBGA_CONSISTENCYCHECK )
#define HAS_INIT( p_ary )			( (p_ary)->bBGA & blBGA_ANYINIT )
#define HAS_INIT_FORE( p_ary )		( (p_ary)->bBGA & blBGA_FOREGRNDINIT )
#define HAS_INIT_BACK( p_ary )		( (p_ary)->bBGA & blBGA_BACKGRNDINIT )
#define HAS_MIGRATION( p_ary )		( (p_ary)->bBGA & blBGA_MIGRATION )
#if defined(ENABLE_BACKUP)
#define HAS_BACKUP( p_ary )			( (p_ary)->bBGA & blBGA_BACKUP )
#endif

#define BGA_RATE_HIGH		1
#define BGA_RATE_MEDIUM	4
#define BGA_RATE_LOW		0

/*
 =========================================================================================
    Get BGA SYNC kind
 =========================================================================================
 */
#define BGA_SYNC_KIND( p_ary ) \
		( HAS_SYNC_NORMAL(p_ary) ? SYNC_KIND_NORMAL : SYNC_KIND_CHECKONLY )

/*
 =========================================================================================
    Get BGA INIT kind
 =========================================================================================
 */
#define BGA_INIT_KIND( p_ary ) \
		( HAS_INIT_FORE(p_ary) ? INIT_KIND_FULL_FOREGRND : INIT_KIND_FULL_BACKGRND )

/* array.bBGAState BGA state definitions */
#define BGA_STATE_NONE			0	/* no BGA at all */
#define BGA_STATE_RUNNING		1	/* BGA is running on this array */
#define BGA_STATE_ABORTED		2	/* BGA was once aborted */
#define BGA_STATE_PAUSED		3	/* BGA was paused by user */
#define BGA_STATE_AUTOPAUSED	4	/* BGA was paused internally, usually because of ACPI
									 * or system is going to shutdown */

#define BGA_CAN_BE_OVERRIDEN(bgaState)	((bgaState)==BGA_STATE_ABORTED)
/*
 =========================================================================================
    helper macros for BGA state
 =========================================================================================
 */
#define ARRAY_BGA_STATE( p_ary )	( (p_ary)->bBGAState )
#define IS_BGA_RUNNING_ON_ARRAY( p_ary ) \
		( HAS_BGA(p_ary) && (ARRAY_BGA_STATE(p_ary) == BGA_STATE_RUNNING) )
#define IS_BGA_AUTOPAUSED_ON_ARRAY( p_ary ) \
		( HAS_BGA(p_ary) && (ARRAY_BGA_STATE(p_ary) == BGA_STATE_AUTOPAUSED) )
#define IS_BGA_STATE_CONFLICT( p_ary ) \
		( HAS_BGA(p_ary) && (ARRAY_BGA_STATE(p_ary) == BGA_STATE_NONE) )
#define CAN_BGA_AUTO_START( p_ary )		\
		( (ARRAY_BGA_STATE(p_ary) != BGA_STATE_ABORTED) && (ARRAY_BGA_STATE(p_ary) != BGA_STATE_PAUSED) )

#define EngAtapiDiskID(p_ary)	((p_ary)->bLogicalWidth)
#define EngTapeDiskID(p_ary)	((p_ary)->bLogicalWidth)

struct array
{
	U16		hGeneration;
	U16		blFlag;

	U8		blRAID;						/* RAID level */
	U8		bWidth;						/* number of streams, including parity or
										 * mirrored streams */
	U8		bLogicalWidth;				/* for RAID1(0), bLogicalWidth=bWidth/2.
										 * bLogicalWidth=bWidth for other RAID levels.
										 * Sometimes I wondered why bLogicalWidth is not
										 * bWidth-1 for RAID3/5... */
	U8		bID;						/* array ID, usually it is the same as the index
										 * in aryDef[] */

#if defined(ENGINE_SUPPORT_LUN_MAP)
	U8		bTargetId;
	U8		bLun;
#endif

	U8		bSerial[VSERIAL_SIZE];

	U16		nPercentage;				/* BGA percentage, for convinient usage only */
	U16		bBGA;						/* extended for BGA state */
	U8		bBGAState;					/* BGA status */
	U8		Lookahead;
	U8		Consolidation;
	U8		RealCacheMode;

#if defined(NEW_BGA_RATE_CONTROL_SCHEME)
	U8		bCMDLoop;					/* For BGA rate usage, to count how many OS REQ executed*/
	BoOL	BGAReqInQueue;				/* Indicate there's a BGA request in queue */
	U8		bReserved1[2];
#endif

	U16		hCyl;
	U8		bHd;
	U8		bSctr;

	U8		CacheMode;					/* see LUN.CacheMode */
	U8		DiskWriteCache;				/* see LUN.DiskWriteCache */
	U8		DiskReadLookahead;			/* see LUN.DiskReadLookahead */
	U8		OutstandingIO;

	/*
	 * This is used to control MDD format, V2 MDD format or DDF or anything else in
	 * the future
	 */
	U8		bFormat;					/* see MDD_FORMAT_xxx defined in eng_vmdd.h */
	U8		bStripeBlockPower;			/* 1>>bStripeBlockPower = hStripeBlockSize,
										 * defined for convinience. */

	U8		bRounding;					/* rounding scheme, this information is necessary
										 * for migration, see RAID_ROUNDING_xxx */
	U8		SubArrayCount;

	U16		hStripeBlockSize;			/* stripe block size */
	U16		hStripeSize;				/* hStripeBlockSize*bLogicalWidth. For RAID5,
										 * this size includes the parity. */

	LBA		wSize;						/* array size */
	LBA		wWatermark;					/* watermark for BGAs */
	U8		bBlkSize;					/* sectors per Block, for WebPAM/OS information only */

	U8		Reserved1[3];
	STRM	strm[MAX_STREAM_PER_ARRAY]; /* streams of the array */

	bpREQ	p_bgaReq;					/* engine can know BGA by checking p_bgaReq */

	/* array based global locked request queue */
	bpQ		p_LockedReqHead;

#if defined(ENGINE_SEQUENTIAL_SUPPORT)
	LBA		LastAccessLBA;
	U32		SequentialCount;
#endif
	bpU8	Name/*[ARRAY_NAME_LENGTH]*/;

	//
	// bpARYSECINFO pSecAryInfo; /* some other array information that can be discarded
	// during BIOS runtime
	//
#if defined(MOONFANG)
	dev_bmp_t working_stream_bmp;

	/* valid when blRAID==blRAID_MF */
	U8	MfParityCount;
	U8	MfSymmetric;
	I16	MfShift;

	void	*MF_Object;					/* Pointer to Moonfang Object */
#define GET_MF_OBJ_FROM_ARY(ary)		(mf_obj_t *)((ary)->MF_Object)
#endif
#if defined(ENABLE_BACKUP)
	U16		blExtendFlag;
	U8		bFastCopyFlag;
	U8		bRaidTransferFlag;
	U8 		BASEATTR 	bOldName[15];
	U8		Reserved2;
	LBA		wLastTimeWatermark;					/* watermark for BACKUP BGAS */
#endif
};

#define SG_ITEM_COUNT_PER_SECT_AS_CONST

#if defined(_X8616B) || defined(SIMULATE_BIOS_ENV)
	#define SG_ITEM_COUNT_PER_SECT_DEFAULT	4
#elif defined(DISABLE_PBM)
	#define SG_ITEM_COUNT_PER_SECT_DEFAULT	MAX_SG_ITEM_PER_TABLE
#elif defined(CC_BYPASS_SUPPORT)
	#define SG_ITEM_COUNT_PER_SECT_DEFAULT	MAX_SG_ITEM_PER_TABLE
	#define SG_ITEM_COUNT_PER_SECT_MIN		16
	#undef SG_ITEM_COUNT_PER_SECT_AS_CONST
#else
	#define SG_ITEM_COUNT_PER_SECT_DEFAULT	16
#endif


#if defined(SG_ITEM_COUNT_PER_SECT_AS_CONST)
#define SG_ITEM_COUNT_PER_SECT			SG_ITEM_COUNT_PER_SECT_DEFAULT
#else
extern U16 BASEATTR SG_ITEM_COUNT_PER_SECT;
#endif

typedef struct	sect
{
	U32		wLBAOffset;
	U16		hCount;
	U8		bReserved[2];
	bpSG	p_CurrentPRD;
	bpSG	pBlockFirstPRD;
	SG		prd;						/* prd[1] */
	/*U8		prdBuffer[sizeof(SG) * SG_ITEM_COUNT_PER_SECT - 1];*/
}
SECT, buf_space * bpSECT;

#define STRUCT_SECT_SIZE		( OFFSET_OF(SECT,prd) + sizeof(SG) * SG_ITEM_COUNT_PER_SECT )

struct map
{
	bpSECT	p_sect[2*MAX_STREAM_PER_ARRAY + 1];	/* plus 1 is necessary for synchronization */
	bpSG	p_current;
	U32		wOffset;
	U32		wSkipSize;
	U8		bNumber;						/* total number of stream involved */
	U8		bFirstStream;					/* stream number of head */
	U8		bActingStream;					/* for RAID 4/5 rebuild/sync usage only */
	U8		bReserved;
	SECT	sect;							/* sect[1] must be last */
	/*U8		sectBuffer[ STRUCT_SECT_SIZE * MAX_STREAM_PER_ARRAY ];*/
};

#define STRUCT_MAP_SIZE ( OFFSET_OF(MAP,sect) + STRUCT_SECT_SIZE * (MAX_STREAM_PER_ARRAY+1) )

#define GetSectOfMap( p_map, sectIndex ) \
		( (bpSECT) (((bpBUF) & (p_map)->sect) + (sectIndex) * STRUCT_SECT_SIZE) )
#define bpSECTInc( p_st )	( (p_st) = (bpSECT) (((bpBUF) p_st) + STRUCT_SECT_SIZE) )

struct _StripeLines
{
	LBA ArrayLBAOfFirstStripe;				/* LBA information for the first stripe line */
	LBA DiskLBAOfFirstStripe;

	U8	LineOffsetFromFirstStripe;			/* current stripe line no. related to the
											 * first stripe line */
	U8	parityStream;						/* parity stream no. for the current stripe
											 * line */

	U8	stripeCount;
};

struct _AccessMap
{
	StripeLines stripeLine;
#if defined(MOONFANG)
	U32			AvailableSgBufferSize;
#endif
	MAP			map;
};

#define STRUCT_ACCESSMAP_SIZE	( OFFSET_OF(AccessMap,map) + STRUCT_MAP_SIZE )

#if defined(MOONFANG)
#define ENG_REQ_SGTABLE_BUFFER_SIZE	(STRUCT_MAP_SIZE+EXTRA_SSB_SIZE)
bpSGTABLE EngAdvanceReqSgTable( bpREQ p_rq );
bpSGTABLE EngAllocateReqSgTable( bpREQ p_rq, U8 ItemCount );
#endif	/* MOONFANG */

/* locks sections */
#define blREAD_LOCK			bit( 0 )
#define blWRITE_LOCK		bit( 1 )
#define blEXCLUSIVE_LOCK	( blREAD_LOCK | blWRITE_LOCK )

typedef struct	seg
{
	DECLARE_ENG_Q_HEADER
	bpREQ	p_rq;							/* lock holder request */
	LBA		wStart;							/* locked LBA range */
	LBA		wEnd;
	U8		blFlag;							/* see above blLOCK_xxx */
}
SEG, buf_space * bpSEG;

#if defined(MOONFANG)
#define ENG_MAPSGTABLE_ITEM_COUNT	32	/* 16 is not enough for 128K critical RAID MF */
#else
#define ENG_MAPSGTABLE_ITEM_COUNT	16
#endif

typedef struct _MapSgTable
{
	bpSG	p_current;
	U32		wOffset;
	U8		SgTable[ SGTABLE_SIZE(ENG_MAPSGTABLE_ITEM_COUNT) ];
}
MapSgTable, buf_space * bpMapSgTable;

typedef struct auxsgtbl
{
	DECLARE_ENG_Q_HEADER
	MapSgTable MapSgTbl;
} AUX_SGTABLE, buf_space * bpAUX_SGTABLE;

/* RAID5 specific data structure definitions ? RAID5 status */
#define blREADBUF				bit( 0 )
#define blREADRECOVERY			bit( 1 )
#define blWRITERECOVERY			bit( 2 )
#define blReadPartOfBlock		bit( 3 )
#define blReadPartOfParity		bit( 4 )
#define blReadTailPartOfBlock	bit( 5 )
#define blFirstXorRead			bit( 6 )
#define blSecondXorRead			bit( 7 )

/* RMW Status */
#define blRMW			bit( 0 )
/*#define blONEDRIVEBLOCK bit( 1 )*/
/*#define blRMWLINE2		bit( 2 )*/
/*#define blSPECIALSECOND bit( 3 )*/
/*#define blSPECIALFINAL	bit( 4 )*/
/*#define blPARITYONLY	bit( 5 )*/
/*#define blSPECIALRMW	bit( 6 )*/
/*#define blNORMALRMW		bit( 7 )*/

typedef struct	xormap
{
	bpSECT	p_sect[MAX_STREAM_PER_ARRAY];
	SECT	sect;
/*	U8		sectBuffer[STRUCT_SECT_SIZE * (MAX_STREAM_PER_ARRAY-1)];*/
}
XORMAP, buf_space * bpXORMAP;

#define STRUCT_XORMAP_SIZE	( OFFSET_OF(XORMAP, sect) + STRUCT_SECT_SIZE*MAX_STREAM_PER_ARRAY )

typedef struct	xormap1
{
	bpSECT		p_sect[MAX_STREAM_PER_ARRAY];
	MapSgTable	recoversg;
	SECT		sect;
/*	U8			sectBuffer[STRUCT_SECT_SIZE * (MAX_STREAM_PER_ARRAY-1)];*/
}
XORMAP1, buf_space * bpXORMAP1;

#define STRUCT_XORMAP1_SIZE	( OFFSET_OF(XORMAP1, sect) + STRUCT_SECT_SIZE*MAX_STREAM_PER_ARRAY )

struct readmap
{
	bpXORMAP	p_xormap;
	bpXORMAP1	p_xormap1;
#if !defined(ENGINE_ROUND_CRITICAL_R5_ACCESS)
	bpXORMAP1	p_xormap2;
#endif
	bpREQ		p_rq;

	/* access related information */
	/*LBA			wLBAStart;*/					/* range crossed disk LBA range */
	LBA			wLBAEnd;

	U8			bLogicalWidth;
	U8			bXorline;
	U16			hStripeBlockSize;

	U8			bStartStream;				/* the stream that the start LBA located */
	U8			bReadPartOfBlock;
	U8			bReadTailPartOfBlock;
	U8			bRecoverStream;

	U16			hSizeOfPartBlock;
	U16			hTailSizeOfPartBlock;

	U8			xornum[MAX_STREAM_PER_ARRAY*2];

	U8			bRAID5Status;
	U8			bRMWStatus;
	U8			blFirstNotDone;
	U8			Conv_AryID;

	U16			hFirstLastSize;
	U8			bMaxXorNum;
	U8			bReserved;

	U16			hRecoverCount1;
	U16			hRecoverCount2;

	MapSgTable	paritysg;
	MapSgTable	readsg;

	XORMAP		xormap;
/*
	XORMAP1		xormap1;
#if !defined(ENGINE_ROUND_CRITICAL_R5_ACCESS)
	XORMAP1		xormap2;
#endif
*/
};

#define GetXorMapOfReadMap( p_readmap )	(&(p_readmap)->xormap)

#define GetXorMap1OfReadMap( p_readmap )	\
	((bpXORMAP1)(((bpU8)GetXorMapOfReadMap(p_readmap)) + STRUCT_XORMAP_SIZE))


#if !defined(ENGINE_ROUND_CRITICAL_R5_ACCESS)

#define GetXorMap2OfReadMap( p_readmap )	\
	((bpXORMAP1)(((bpU8)GetXorMap1OfReadMap(p_readmap)) + STRUCT_XORMAP1_SIZE))

#define STRUCT_READMAP_SIZE ( OFFSET_OF(READMAP,xormap) + STRUCT_XORMAP_SIZE + STRUCT_XORMAP1_SIZE * 2 )

#else

#define STRUCT_READMAP_SIZE ( OFFSET_OF(READMAP,xormap) + STRUCT_XORMAP_SIZE + STRUCT_XORMAP1_SIZE )

#endif


typedef struct	xorbuf
{
	DECLARE_ENG_Q_HEADER
	U8	buf[1/*STRUCT_READMAP_SIZE*/];
}
XORBUF, buf_space * bpXORBUF;

#define STRUCT_XORBUF_SIZE	(OFFSET_OF(XORBUF,buf) + STRUCT_READMAP_SIZE)

/* End of RAID5 specific data structure definitions */

#ifndef MOONFANG
#define INTERNAL_ARYSTATUS_NORMAL	0x00
#define INTERNAL_ARYSTATUS_CRITICAL 0x01
#define INTERNAL_ARYSTATUS_OFFLINE	0x02
#define INTERNAL_ARYSTATUS_GONE		0x03	/* no stream for this array, totally gone,
											 * reserved for future */
#else
/* The number must be increased by the severity */
#define INTERNAL_ARYSTATUS_NORMAL	0x00
#define INTERNAL_ARYSTATUS_DEGRADE	0x01	/* It's between normal and degrade. */
#define INTERNAL_ARYSTATUS_CRITICAL 0x02
#define INTERNAL_ARYSTATUS_OFFLINE	0x03
#define INTERNAL_ARYSTATUS_GONE		0x04	/* no stream for this array, totally gone,
											 * reserved for future */
#endif

U8 CheckArrayStatusByExamingStreams( bpRaidEngine This, bpARY p_ary );
#if defined(MOONFANG)
U8 CheckArrayStatusByExamingStreamsEx( bpRaidEngine This, bpARY p_ary, dev_bmp_t *bmp );
U8 CheckSubArrayStatus(bpRaidEngine This, bpARY p_ary, dev_bmp_t *bmp, U8 bStream);
#endif
U8 GetDiskIDFromStreamID( bpRaidEngine This, bpARY p_ary, U8 StreamID );
U8 GetStreamIDFromBlockID( bpRaidEngine This, bpARY p_ary, U8 BlockID );
U8 FindAvailableArrayID( bpRaidEngine This, U8 bHint, BoOL bMigration );
#if defined(ENABLE_BACKUP)
U8 FindTempArrayIDBySerial(
	bpRaidEngine	This,
	bpBUF			bSerial,
	U8				SerialSize);
U8 FindAvailableTempArrayID( bpRaidEngine This, U8 bHint);
BoOL IsBackupSourceExisted(bpARY p_ary);
U8 FindBackupDisk(bpARY	p_ary);
#endif
bpSTRM p_smMirrorStream( bpARY p_ary, U8 bStrm );

U8 FindArrayIDBySerial(
	bpRaidEngine	This,
	bpBUF			bSerial,
	U8				SerialSize,
	BoOL			bMigration );

/*U8 FindBlockIDBySerial( bpRaidEngine This, bpBUF bSerial, U8 SerialSize );*/
U8 FindAvailableBlockID( bpRaidEngine This, U8 bHint );
U8 FindBlockIDFromDiskID( bpRaidEngine This, U8 diskID, U8 bStartBlockID );
U8 FindExactBlockIDFromDiskID( bpRaidEngine This, U8 diskID, LBA AnyLBAInTheBlock, OPTIONAL OUT LBA* ExtentLBA );

U8 FindAssociatedArrayIDFromBlockID( bpRaidEngine This, U8 bBlockID, BoOL bMigration );
U8 GetDiskIDFromBlockID( bpRaidEngine This, U8 bBlockID );
U8 GetBlockCountOnDisk( bpRaidEngine This, U8 DiskID);

BoOL FindArrayIDAndStreamFromBlockID(
		bpRaidEngine	This,
		U8				BlockID,
		bpU8			ArrayID,
		bpU8 StreamID	OPTIONAL,
		BoOL			bMigration );

#if MAX_DISK_SUPPORTED > 32
#error "CreateDiskBitmapForArray needs to be modified due to MAX_DISK_SUPPORTED > 32!"
#endif

U32 CreateDiskBitmapForArray(bpRaidEngine This, U8 ArrayID);

/*U8 GetMDDFormatForBlockID( bpRaidEngine This, U8 BlockID );*/
U8 FindMissingStreamInArray( bpRaidEngine This, U8 bArrayID, U8 bStartStream );
U8 bFindProblemStream( bpRaidEngine This, bpARY p_ary );
U8 bFindProblemStreamEx( bpRaidEngine This, bpARY p_ary, U8 bStartStream );
void SetArrayStatus( bpRaidEngine This, U8 bAryID, BoOL bLogEvent );
U32 AddBlockToRedundantArray( bpRaidEngine This, U8 bArrayID, U8 bBlockID, U8* bStream );
U8 IsBlockStandaloneMigratingTarget( bpRaidEngine This, U8 bBlockID );

BoOL CheckBlockSizeForArray( bpRaidEngine This, bpBLOCK p_blk, bpARY p_ary );
U8 EngSetBlockDown( bpRaidEngine This, U8 DiskID, U8 BlockID, BoOL bKeepBlkFlag );
void EngSetDiskDown(
		bpRaidEngine	This,
		U8				DiskID,
		BoOL			bDiskGone,
		BoOL			bToUpdateMDD );
void EngDiskArrived( bpRaidEngine This, U8 DiskID );

LBA GetMinimumBlockSizeForArray( bpARY p_ary );

U8 FindMostFitSizeBlock(
	bpRaidEngine	This,
	LBA				RequiredSize,
	BoOL			bSpare,
	U8				bDedicatedArray,
	U32				ExceptDiskMap );

U8 FindMostFitBlockForArray( bpRaidEngine This, U8 bArrayID, bpARY p_ary, LBA* requiredSize );
#if defined(ENABLE_BACKUP)
void UnmountBackupArrayBlocks( bpRaidEngine This, bpARY p_ary, U8 bBlockID );
#endif
void UnmountArrayBlocks( bpRaidEngine This, bpARY p_ary, BoOL blMarkAsMddOverridden );
void UnmountArraySpares( bpRaidEngine This, U8 bArrayID, BoOL blMarkAsMddOverridden );
void EngineOnSystemFullyStarted( bpRaidEngine This );

void EngReportStarted( bpRaidEngine This );
U8	CountAvailableStrmInArray( bpARY p_ary );

void SetMappingMode( bpRaidEngine This, bpARY p_ary );
void GetDiskCHS( bpRaidEngine This, U8 DiskID, U32 *pCyl, U8 *pHead, U8 *pSector );

typedef enum 
{
	ArrayArrival,
	ArrayRemoval,
	ArrayMounted,
	ArrayStatusChanged,
	ArrayBlockArrival,
	ArrayBlockRemoval
} EngANType;

void EngNotifyArrayChanged(
	IN bpRaidEngine	This,
	IN U8			ArrayID,
	IN EngANType	Type,
	IN ... );

BoOL engSplitBlock(
	bpRaidEngine	This,
	U8				BlockID,
	LBA				wOffsetLBA,
	U8				*newBlockID );

U8 GetBlockIDWidthDiskIDAndExtentIndex(
	bpRaidEngine	This,
	U8				DiskID,
	U8				ExtentIndex );

void 
engDeleteDiskExtents(
	bpRaidEngine	This,
	U8				DiskID );

void 
MarkDiskExtentsDown(
	bpRaidEngine	This,
	U8				DiskID );

U16
RoundToStripeBlockSize(
	bpARY	p_ary,
	U32		wLBAOffset,			/* disk LBA offset */
	U16		hCount );

BoOL
bWillStreamOfflineResultArrayOffline(
	bpRaidEngine	This,
	bpARY			p_ary,
	U8				StreamID );

BoOL
bWillThisDiskDownResultAnyArrayOffline(
	bpRaidEngine This,
	U8 DiskID );

void
MarkArrayExtentsAsMDDUpdate(
	bpRaidEngine	This,
	U8				bArrayID,
	BoOL			bIncludeSpare,
	U8				bControl );

void MarkExtentMDDUpdate( 
	bpRaidEngine	This,
	U8				bExtentID,
	U8				bSaveMddControl );

void ClearExtentMDDUpdate(
	bpRaidEngine	This,
	U8				bExtentID,
	U8				bSaveMddControl,
	BoOL			bForceClearAll );

#if !defined(NO_BGA)
BoOL
ToStopBGA(
	bpRaidEngine	This,
	U8				bArrayID,
	bpREQ			p_rq,
	U8				Action );
#endif

void EngMarkArrayConfigured(
	bpRaidEngine	This,
	U8				bArrayID );

void EngClearArrayConfigured(
	bpRaidEngine	This,
	U8				bArrayID );

void EngWaitRequest(
	bpREQ			p_rq,
	bpREQ			p_rqPended );

void EngPushbackWaitingList(
	bpRaidEngine	This,
	bpREQ			p_rq );

U8 TranslateBGARate2Count(
	bpRaidEngine	This,
	U16			bBGA );

#if defined(MOONFANG)
void EngInitStreamMask(
	bpRaidEngine This,
	bpARY p_ary );
#endif

#define ENG_ARY_DATA_WIDTH(p_ary)	\
	(IS_RAID45((p_ary)->blRAID) ? ((p_ary)->bWidth-1) : (p_ary)->bLogicalWidth)

#define ENG_ARY_STRIPE_DATA_SIZE(p_ary)	\
	(IS_RAID45((p_ary)->blRAID) ? ((p_ary)->hStripeSize-(p_ary)->hStripeBlockSize) : (p_ary)->hStripeSize)


#ifdef MOONFANG
#define ENG_ARY_SUB_ARRAY_DATA_WIDTH(p_ary)	\
	( (IS_RAID45((p_ary)->blRAID) ? ((p_ary)->bWidth-1) : (p_ary)->bLogicalWidth) \
	/ (((p_ary)->SubArrayCount) ? ((p_ary)->SubArrayCount) : 1) )

#define ENG_ARY_SUB_STRIPE_DATA_SIZE(p_ary)	\
	(	(IS_RAID45((p_ary)->blRAID) ? ((p_ary)->hStripeSize-(p_ary)->hStripeBlockSize) : (p_ary)->hStripeSize)	\
	/ (((p_ary)->SubArrayCount) ? ((p_ary)->SubArrayCount) : 1) )
#endif

#endif /* ENGINE_INTERNAL_H */
