#if !defined( ENGINE_REQUEST_H )
#define ENGINE_REQUEST_H

#include "eng_type.h"
#include "eng_con.h"

#if !defined(RESOURCE_FREE_POOL_COUNT)
/*
 * Please don't change definition of RESOURCE_FREE_POOL_COUNT here, RESOURCE_FREE_POOL_COUNT
 * is also defined in eng_res.h. why I defined RESOURCE_FREE_POOL_COUNT is to avoid including 
 * unnecessary header files, which is a problem for Linux partial open source.
 */
#define RESOURCE_FREE_POOL_COUNT	8
#endif

#define ENG_INT_SGTABLE_ITEMCOUNT	Max( ((MAX_STREAM_PER_ARRAY+1)*2), 16 )

/* req.bType */
#define blENGINE_ACCESS				bit( 0 )	/* engine internal request */
#define blSEGMENTED_REQUEST			bit( 1 )	/* set if a request is segmented */
#define blCONV_EXP_REQ				bit( 2 )
#define blSG_CHANGED				bit( 3 )	/* set if a SG table is modified */
#define blHAS_LOCK					bit( 4 )
#define blTRANS_FIX_REQ				bit( 5 )
#define blREQ_LBA_VALID				bit( 6 )
/* blREQ_COUNTED is used to count external requests, see InitRequest */
#define blREQ_COUNTED				bit( 7 )
#define blREQ_MIGRATION_TARGET		bit( 8 )

/*above flags will be kept in ReleaseAllResources */
#define REQ_PERSISTENT_FLAG_MASK	0x01ff

/* following flags will be cleaned in ReleaseAllResources */
#define blREQ_INITIALIZED			bit( 12 )
#define blINTERNAL_SG				bit( 13 )	/* internal SG table is used */
#define blINTERNAL_SENSE			bit( 14 )	/* internal sense info is used */

#define IS_MIGRATION_REQ( p_rq )	( (p_rq)->bType & blCONV_EXP_REQ )
#define IS_SEGMENTED_REQ( p_rq )	( (p_rq)->bType & blSEGMENTED_REQUEST )
#define REQ_HAS_LOCK( p_rq )		( (p_rq)->bType & blHAS_LOCK )
#define REQ_SUCCEED( p_rq ) \
		( ((p_rq)->bAdapterStatus == asNO_ERROR) && ((p_rq)->bTargetStatus == ssGOOD) )
#define IS_REQ_COUNTED( p_rq )		( (p_rq)->bType & blREQ_COUNTED )

#define BGAFLAG_EMERGEMENT			bit(0)

#define	ENG_REQ_DATA_IN				bit(0)
#define	ENG_REQ_DATA_OUT			bit(1)

/*
 -----------------------------------------------------------------------------------------
    notorious REQ definition. The size of REQ is growing... it's more or less a headache,
    however since now we grab memory for REQ from OS request, it should not be a serious
    problem. Of course, engine still needs some internal requests but the number is
    limited.
 -----------------------------------------------------------------------------------------
 */
struct req
{
	DECLARE_ENG_Q_HEADER
	/* above fields are for Engine internal usage, upper layers shall not touch */

	spVOID				pOriginalRequest;	/* original OS request */
	spBUF				p_buf;				/* data buffer pointer */

	spBUF				p_sns;				/* sense information buffer pointer */

	bpSGTABLE			pSgTable;			/* sg table pointer */

	//U8					bCDB[16];
	U8					bCDB[20];			/* 16~19 for 2 Tera Byte Issue*/
	U8					bCdbSize;
	U8					bDataDir;			/* ENG_REQ_DATA_xxx */
	U8					bTimeoutValue;

	U32					wSize;				/* buffer size */

	U8					bAdapterStatus;
	U8					bTargetStatus;
	U8					bTargetID;			/* TargetId */
	U8					bLUN;				/* LUN */

	/* Following fields are for Engine internal usage, upper layers shall not touch */

	LBA					wLBA;				/* for OS access request, it holds Error LBA
											 * on return with error. */

	U16					hCount;				/* sector count */
	U16					bType;

	U8					bSenseSize;
	U8					bRetry;				/* request retry count */
	U8					bDwell;				/* request retry count on the same LBA */
	U8					bID;				/* ArrayID */

#if defined(ENABLE_TRANSACTION_LOG)
	U16					nTransactionID;
	U16					nTransactionFlags;
#endif	/* ENABLE_TRANSACTION_LOG */

	bpREQ				p_rqPending;		/* used for engine internal request chaining */
	bpSEQ				p_sqResumePoint;	/* used for engine internal request chaining */

#if !defined(_X8616B)
	/* no lock for BIOS/DOSAPP... */
	bpREQ				p_Lockrq;			/* locked REQ list */
#endif	/* _X8616B */
	bpREQ				p_Waitrq;

	bpQ					p_head[RESOURCE_FREE_POOL_COUNT];	/* resource queues */

	bpSEQ				p_sq;					/* sequence head */

#if VDEBUG
	bpSEQ				p_sqBase;				/* debugging usage */
#endif

	bpAccessMap			pAccessMap;				/* access map, for all RAID levels */
	bpMAP				p_map;
	bpREADMAP			p_readmap;				/* RAID4/5 only */

	/* error handling fields */
	RequestOnTaskError	OnTaskError;			/* reuest error handler */
	union
	{
		LBA				ErrorDiskLBA;			/* disk LBA where error hit */
		LBA				ErrorBlockLBA;			/* block LBA where error hit */
	} errlba;

	U16					hErrorCount;
	U16					hSectorBySecorCount;	/* see XXXSectorBySectorProcessingXXX in
												 * eng_errorhandling.h */
	U8					bErrorCmd;
	U8					bTaskStatus;			/* see dtask.bStatus (tsXXX)in
												 * eng_errorhandling.h */
	U8					bEHFlags;				/* see bEH_F_xxx in eng_errorhandling.h */
	U8					bInFixingStream;

	U8					bErrExtCnt;
	U8					bErrExtDiskId;

	/* request-dependent parameters. This can be used by concrete request handler. */
	union
	{
		struct
		{
			PTR_SIZE	param[8];
		} generic;

		struct
		{
			bpVOID				pContextBuffer; /* context buffer, refer its uage to the
												 * vmdd interface spec. */
			bpSGTABLE			pSgTable;
			U32					ContextBufferSize;
		} config;
	#if !defined(NO_BGA) || defined(ENABLE_TRANSACTION_LOG)
		/*
		 * used by BGAs, depends on different BGA, CurrentLBA/EndLBA may array LBA or
		 * disk LBA. It is array LBA for all RAID level foreground initialization and
		 * migration, RAID1 synchronization. It is disk LBA for all supported RAID
		 * level background initliazation, rebuild and RAID4/5 synchronization.
		 */
		struct
		{
			LBA		CurrentLBA;
			LBA		EndLBA;
			bpQ		p_auxBuf;					/* all BGAs require auxiliary buffer... */
			bpREQ	p_auxReq;					/* used by quick/foregrnd init and
												 * migration */
			U16		hCount;
			U16		Percentage;					/* BGA handlers will update this
												 * automatically */
			U16		Ratio;
			U16		BGAFlag;
			dev_bmp_t RebuildTargetBmp;
			U8		auxSgTable[
			SGTABLE_SIZE( ENG_INT_SGTABLE_ITEMCOUNT )
			];
		} bga;
	#endif

	#if 1 //!defined(DISABLE_ERROR_SIMULATION)
		struct
		{
			U8	auxSgTable[
			SGTABLE_SIZE( 1 )
			];	/* some debugging requests needs buffer to execute
				 * (PDBG_READ/WRITE_BLOCKS, etc.) */
		} dbg;
	#endif
		struct
		{
			/*
			 * backup SG table to be used when engine needs to modify original SG
			 * table.
			 */
			SGTABLE sgTableBackup;
		} readwrite;

		struct 
		{
			bpEVENT p_ev;
		} patrol;
		struct 
		{
			bpEVENT p_ev;
		} smart;
		struct
		{
			bpQ p_rec;
		} errfix;
	} param;

	/* 
	 * Following fields are for upper layers, upper layer shall touch fields 
	 * belongs to itself only. 
	 */
#if !defined(DISABLE_PBM)
	struct
	{
		bpVOID				Provider;	/* filled by upper layers */
		RequestCompletion	Completion; /* filled by upper layers */
		spVOID				ptr;		/* PBM used, pointer to original REQ */
#if defined(USE_MML_CC)
		U32					flag;		/* PBM internal flag control */
#endif
		bpREQ				prev;
		bpREQ				next;
	} CC; 
#endif	/* DISABLE_PBM */

	struct
	{
		bpVOID				Provider;	/* filled by upper layers */
		RequestCompletion	Completion; /* filled by upper layers */
		/* Add Engine specific data here */
	} Engine;

#if defined( _WINAPP )
	struct
	{
		bpVOID				Provider;
		RequestCompletion	Completion;
	} WinApp;
#endif

#if defined(OSD_CHECK_REQ_TIMEOUT)
	#define OSD_REQ_F_QUEUED		0x01
	#define OSD_REQ_F_IN_ENGINE		0x02
	#define OSD_REQ_F_CC_GENERATED	0x04
	#define OSD_REQ_F_BYPASSED		0x08
	struct	
	{
		bpREQ			prev;
		bpREQ			next;
		U16				flags;
		U8				tickCount;
		U8				cc_tickCount;
	} Osd;
#endif

#if defined(MOONFANG)
	dev_bmp_t			dev_mask;
	void 				*MF_SG_PTR;
#endif
};

void	engSetCompletionHelper( bpREQ p_rq, bpVOID provider, RequestCompletion Completion );

#if defined(_X8616B) && defined(USE_EXPORT)

void	engSetCompletionHelper( bpREQ p_rq, bpVOID provider, RequestCompletion Completion );

#define engSetCompletion( p_rq, provider, Completion ) {	\
	GEN_PCALLBACK pCompletion;	\
	GEN_MK_CALLBACKF( pCompletion, Completion);	\
	engSetCompletionHelper( p_rq, provider, (RequestCompletion) pCompletion );	\
}

#else

#define engSetCompletion( p_rq, provider, Completion )  \
	engSetCompletionHelper( p_rq, provider, (RequestCompletion) Completion );	\

#endif

#define IsWriteReq(p_rq)	scsiIsWrite((p_rq)->bCDB[0])
#define IsReadReq(p_rq)		scsiIsRead((p_rq)->bCDB[0])
#define IsVerifyReq(p_rq)		scsiIsVerify((p_rq)->bCDB[0])	//20060321


#endif /* ENGINE_REQUEST_H */
