#if !defined( ENGINE_SCSI3_H )
#define ENGINE_SCSI3_H

#include "gen_typs.h"

/* scsi3 opcode and CDB bits */
#define svucSCSI_COMMAND		0x60

#define scsiINQUIRY				0x12
#define scsiMODE_SELECT_6		0x15
#define scsiMODE_SELECT_10		0x55
#define scsiMODE_SENSE_6		0x1A
#define scsiMODE_SENSE_10		0x5A
#define scsiREAD_6				0x08
#define scsiREAD_10				0x28
#define scsiREAD_CAPACITY		0x25
#define scsiREQUEST_SENSE		0x03
#define scsiREZERO_UNIT			0x01
#define scsiSEEK_6				0x0B
#define scsiSEEK_10				0x2B
#define scsiSTART_STOP_UNIT		0x1B
#define scsiTEST_UNIT_READY		0x00
#define scsiVERIFY				0x2F
#define scsiWRITE_6				0x0A
#define scsiWRITE_10			0x2A
#define scsiWRITE_VERIFY		0x2E
#define scsiSYNCHRONIZE_CACHE	0x35
#define scsiREAD_16				0x88			/* to be supported */
#define scsiWRITE_16			0x8A			/* to be supported */
#define scsiVERIFY_16			0x8F			/* to be supported */
#define scsiREAD_CAPACITY_16	0x9E			/* to be supported */

#define scsiRESERVE				0x16
#define scsiRELEASE				0x17

#define scsiREPORT_LUNS			0xA0
#if defined(SUPPORT_ASYNCHRONOUS_NOTIFICATION)		// it is to return OSs query with SUCCESS result, do not submit the task to the device if it supports AN. by tammy
#define scsiAtapiEventnoti		0x4A
#define scsiIsAtapiEventnoti( cmd )	((cmd)==scsiAtapiEventnoti)
#endif

#define scsiIsRead( cmd )			(((cmd)==scsiREAD_10) || ((cmd)==scsiREAD_16))
#define scsiIsWrite( cmd )			(((cmd)==scsiWRITE_10) || ((cmd)==scsiWRITE_16))
#define scsiIsVerify( cmd )			(((cmd)==scsiVERIFY) || ((cmd)==scsiVERIFY_16))
#define scsiIsReadWrite( cmd )		(scsiIsRead(cmd) || scsiIsWrite(cmd))
#define scsiIs16BAccessCmd( cmd )	(((cmd)==scsiREAD_16) || ((cmd)==scsiWRITE_16) || ((cmd)==scsiVERIFY_16))

#if defined(ENGINE_SUPPORT_LARGE_LU)
#define scsiREAD_DEFAULT		scsiREAD_16
#define scsiWRITE_DEFAULT		scsiWRITE_16
#else	/*ENGINE_SUPPORT_LARGE_LU*/
#define scsiREAD_DEFAULT		scsiREAD_10
#define scsiWRITE_DEFAULT		scsiWRITE_10
#endif	/*ENGINE_SUPPORT_LARGE_LU*/

/* Promise unique opcodes */
#define svucARRAY_COMMAND	0xE0

#if !defined(BIOS)
/* array command sub opcodes */
#define acmdARRAY_CONTROL		0x00
#define acmdGET_ARRAY_CONFIG	0x01
#define acmdSET_ARRAY_CONFIG	0x02
#define acmdBGA_START			0x03
#define acmdBGA_CONTROL			0x04
#if defined(ENABLE_BACKUP)
#define acmdDELETE_BACKUP		0x05
#endif
#define acmdGET_ARRAY_STATUS	0x06
#define acmdDELETE_ARRAY		0x07
#define acmdBGA_FIRST			0x08
#define acmdREBUILD				0x08
#define acmdINITIALIZATION		0x09
#define acmdSYNCHRONIZATION		0x0a
#define acmdMIGRATION			0x0b
#if defined(ENABLE_BACKUP)
#define acmdBACKUP				0x0c
#define acmdFASTCOPY			0x0d
#endif
#define acmdBGA_LAST			0x0e

#define IsBGARequest( SubOpCode ) \
		( ((SubOpCode) >= acmdBGA_FIRST) && ((SubOpCode) < acmdBGA_LAST) )
#define svucARRAY_COMMAND_MAX	0x0c

#else	/* !defined(BIOS) */

#define acmdDELETE_ARRAY		0x00

#define svucARRAY_COMMAND_MAX	0x01

#endif	/* !defined(BIOS) */

/* helper definitions for SCSI INQUIRY */
#define blINQUIRY_EVPD				bit( 0 )
#define bDISK_UNIT_INFO_PAGE		0xC0
#define DISK_UNIT_INFO_PAGE_SIZE	0xF0
#define STANDARD_INQUIRY_PAGE_SIZE	0x30
#define INQUIRY_PRODUCT_NAME_OFFSET 0x10

/* page codes for MODE_SENSE/SELECT */
#define bCURRENT_PARAMETER			0x00
#define bCHANGEABLE_PARAMETER		0x01
#define bDEFAULT_PARAMETER			0x02
#define bSAVED_PARAMETER			0x03
#define bARRAY_MAPPING_MODE_PAGE	0x20
#define bCONTROLLER_SPECIFIC_PAGE	0x21
#define bMAINTENANCE_CONTROL_PAGE	0x22
#define bALL_PAGES					0x3F

/* Promise controller command opcodes */
#define svucCONTROLLER_COMMAND	0xE1

/* controller command sub opcodes */
#if !defined(BIOS)
#define ccmdSTOP_ALL_BGA			0x00		/* refer to ccmdStopAllBGA */
#define ccmdCREATE_ARRAY			0x01
#define ccmdGET_GLOBAL_CONFIG		0x02
#define ccmdSET_GLOBAL_CONFIG		0x03
#define ccmdPATROL_CONTROL			0x04
#define ccmdSAVE_DISK_MDD			0x05
#define ccmdMEDIA_PATROL			0x06
#define ccmdBLOCK_CONTROL			0x07
#define ccmdENGINE_BIST				0x08
#define ccmdINIT_CONFIG				0x09		/* ccmdINIT_CONFIG is reserved for engine
												 * internal usage ONLY! */
#define ccmdTRANS_FIX				0x0a
#define ccmdATA_PASSTHRU			0x0b
#define ccmdGET_INFO				0x0c		/* misc infor needed by application */
#define ccmdEVENT					0x0d
#define ccmdPOWER_STATE_CHANGING	0x0e		/* CDB[2] = CCMD_POWER_WORKING/CCMD_POWER_SAVING */
#define ccmdSMART_CHECK				0x0f
#define ccmdGET_MAX_ARRAY_SIZE		0x10
#define	ccmdRESUME_ALL_BGA			0x11
#define	ccmdDISABLE_ALL_DISK_CACHE	0x12
#define	ccmdRESTORE_ALL_DISK_CACHE	0x13
#define ccmdDISABLE_ALL_ARRAY_CACHE	0x14
#define ccmdRESTORE_ALL_ARRAY_CACHE	0x15

#define ccmdERROR_FIXING			0x16

#define svucCONTROLLER_COMMAND_MAX	0x17

#else	/* !defined(BIOS) */

#define ccmdCREATE_ARRAY			0x00
#define ccmdSAVE_DISK_MDD			0x01
#define ccmdBLOCK_CONTROL			0x02
#define ccmdINIT_CONFIG				0x03

#define svucCONTROLLER_COMMAND_MAX	0x04

#endif	/* !defined(BIOS) */

/* CDB[2] of ccmdPOWER_STATE_CHANGING */
#define CCMD_POWER_WORKING			0x00	/* system is going to S0 */
#define	CCMD_POWER_SAVING			0x01	/* system is going to power saving mode (S1~S5) */

/* Promise auxiliary command opcodes */
#define svucDEBUGGING_COMMAND		0xE2
#define svucPASSTHROUGH_API_COMMAND 0xE3

/* scsi status or bTargetStatus in REQ Structure */
#define ssGOOD					0x00			/* no error */
#define ssCHECK_CONDITION		0x02			/* check sense data for status */
#define ssCONDITION_MET			0x04			/* should not happen */
#define ssBUSY					0x08			/* RAID Engine is busy, retry after some
												 * time */
#define ssINTERMEDIATE_GOOD		0x10			/* should not happen */
#define ssINTERMEDIATE_MET		0x14			/* should not happen */
#define ssRESERVATION_CONFLICT	0x18			/* should not happen */
#define ssCOMMAND_TERMINATED	0x22			/* should not happen */
#define ssQUEUE_FULL			0x28			/* RAID Engine is busy, retry after some
												 * time */
#define ssLOCK					0x80

/* scsi sense key, ASC, and ASCQ */
#define skNO_SENSE						0x00	/* no additional status */
#define skRECOVERED_ERROR				0x01	/* cmd completed using recovery */
#define skNOT_READY						0x02	/* array is busy, retry after some time */
#define skMEDIUM_ERROR					0x03	/* auto-reassign/recovery failed */
#define skHARDWARE_ERROR				0x04	/* disk hardware error */
#define skILLEGAL_REQUEST				0x05	/* invalid bits in CDB */
#define skUNIT_ATTENTION				0x06	/* special event just happened */
#define skDATA_PROTECT					0x07	/* should not happen */
#define skBLANK_CHECK					0x08	/* should not happen */
#define skCOPY_ABORTED					0x0A	/* should not happen */
#define skABORTED_COMMAND				0x0B	/* should not happen */
#define skEQUAL							0x0C	/* should not happen */
#define skVOLUME_OVERFLOW				0x0D	/* should not happen */
#define skMISCOMPARE					0x0E	/* should not happen */

#define saNO_ASC						0x00
#define saLUN_NOT_READY					0x04
#define saECC_ERROR						0x10
#define saID_ADDR_MARK_NOT_FOUND		0x12
#define saINVALID_OPCODE				0x20
#define saLBA_OUT_OF_RANGE				0x21
#define saINVALID_FEILD_IN_CDB			0x24
#define saLOGICAL_UNIT_NOT_SUPPORTED	0x25
#define saINVALID_FIELD_IN_PARAMETER	0x26
#define saINTERNAL_TARGET_FAILURE		0x44

#define sqNO_ASCQ						0x00
#define sqINTERVENTION_REQUIRED			0x03
#define sqMAINTENANCE_IN_PROGRESS		0x80

/*
 -----------------------------------------------------------------------------------------
    data payload for array commands
 -----------------------------------------------------------------------------------------
 */
#define ARRAY_NAME_LENGTH	64

typedef struct	_LOGICAL_DEVICE_SETTING_PAGE0
{
	U8	CacheMode;						/* used by PBM, engine will ignore it */
	U8	DiskWriteCache;					/* see above LOGICAL_DEVICE */
	U8	DiskReadLookahead;				/* see above LOGICAL_DEVICE */
	U8	Lookahead;
	U8	Consolidation;
	U8	RealCacheMode;
	U8	bReserved[2];
	U8	Name[ARRAY_NAME_LENGTH];

	/*
	 * currently background maintenance rate are defined as global setting. This may
	 * be changed in the future. If so, I will add some fields here to override global
	 * settings.
	 */
}
LOGICAL_DEVICE_SETTING_PAGE0, buf_space * bpLOGICAL_DEVICE_SETTING_PAGE0;

/* BGA (Maintenance) Actions */
#define MT_ACT_START		0x00
#define MT_ACT_RESTART		0x01
#define MT_ACT_PAUSE		0x02
#define MT_ACT_RESUME		0x03
#define MT_ACT_ABORT		0x04
#define MT_ACT_COMPLETE		0x05		/* engine internal usage */
#define MT_ACT_AUTOPAUSE	0x06		/* engine internal usage */
#define MT_ACT_ERROR_ABORT	0x07		/* engine internal usage */

#define MT_ACT_STOPPING( Action )			 \
		(									 \
			((Action) == MT_ACT_PAUSE)		 \
		||	((Action) == MT_ACT_ABORT)		 \
		||	((Action) == MT_ACT_AUTOPAUSE)	 \
		||	((Action) == MT_ACT_ERROR_ABORT) \
		||	((Action) == MT_ACT_COMPLETE)	 \
		)
#define MT_ACT_RUNNING( Action )		 \
		(								 \
			((Action) == MT_ACT_START)	 \
		||	((Action) == MT_ACT_RESTART) \
		||	((Action) == MT_ACT_RESUME)	 \
		)

typedef struct	_CDB_STOP_ALL_BGA
{
	U8	OpCode;						/* FIXED svucCONTROLLER_COMMAND */
	U8	SubOpCode;					/* FIXED ccmdSTOP_ALL_BGA */
	U8	Action;						/* MT_ACT_xxx */
}
PACKED			CDB_STOP_ALL_BGA, buf_space * bpCDB_STOP_ALL_BGA;

/* Initialization Control Flags */
#define INIT_CF_QUICK		0x00
#define INIT_CF_FOREGRND	0x01
#define INIT_CF_BACKGRND	0x02

/* data payload for array commands ends */
#if defined(_X8616B)
#define LD_MAX_EXTENTS	MAX_DISK_SUPPORTED
#else
#define LD_MAX_EXTENTS	32
#endif

#define LD_PARAM_FORMAT_EXTENTS			0x00
#if !defined(_X8616B)
#define LD_PARAM_FORMAT_DISKIDS			0x01
#define LD_PARAM_FORMAT_DISK_STRUCTURES	0x02
#endif
#define LD_PARAM_FORMAT_LOGICAL_DEVICE	0x03

typedef struct 	_LD_FMT_EXTENTS
{
	U8	NumberOfExtents;
	U8	bReserved[7];
	U32	wReserved[2];
	U8	ExtentID[LD_MAX_EXTENTS];
} LD_FMT_EXTENTS, buf_space * bpLD_FMT_EXTENTS;

typedef struct 	_LD_FMT_DISKIDS
{
	U8	NumberDisks;
	U8	bReserved[7];					/* 8 byte alignment */
	LBA	ExtentSize;						/* extent size in sectors */
	U32	wReserved[2];
	U8	DiskID[LD_MAX_EXTENTS];
} LD_FMT_DISKIDS, buf_space * bpLD_FMT_DISKIDS;

typedef struct 	_LD_FMT_DISK_STRUCTURE
{
	U8	NumberOfExtents;
	U8	bReserved[7];

	struct
	{
		U8	DiskID;
		U8	bReserved[7];
		LBA	StartLBA;					/* extent start LBA in the disk */
		LBA	ExtentSize;					/* extent size, usually size of all extents must be same */
	} Extents[LD_MAX_EXTENTS];
} LD_FMT_DISK_STRUCTURE, buf_space * bpLD_FMT_DISK_STRUCTURE;

/*
 * currently _LD_FMT_LD_PARAM definition is the same as _LD_FMT_DISKIDS. If you changed
 * the definition of the any one, you may need to change some code in eng_crld.cpp 
 * correspondingly. */
typedef struct _LD_FMT_LD_PARAM
{
	U8	NumberDisks;
	U8	bReserved[7];					/* 8 byte alignment */
	LBA	LDSize;							/* logical device size in sectors */
	U32	wReserved[2];
	U8	DiskID[LD_MAX_EXTENTS];
} LD_FMT_LD_PARAM, buf_space * bpLD_FMT_LD_PARAM;

typedef struct	_CREATE_LD_PARAM
{
	U8	Format;							/* LD_PARAM_FORMAT_xxx */
	/* common parameters */
	U8	Flags;							/* see below CREATE_LD_F_xxx */
	U8	RaidLevel;
	U8	RoundingScheme;					/* see below RAID_ROUNDING_xxx */

	U8	MDDFormat;
	U8	DriverCacheMode;				/* W/B, W/T or bypass */
	U8	DriverLookahead;
	U8	EnableDiskWriteCache;			/* 1 for enable, 0 for disable */

	U32 StripeBlockSize;				/* in sectors */

	/* MOONFANG Extension, valid when RaidLevel==blRAID_MF */
	U8	MfParityCount;
	U8	MfSymmetric;
	I16	MfShift;

	U8	SubArrayCount;
	U8	bReserved[3];

	U32	wReserved[2];

	U8	Name[ARRAY_NAME_LENGTH];

	U32	wReserved2[1];

	/* format specific parameters */
	union
	{
		LD_FMT_EXTENTS			FormatExtents;
#if !defined(_X8616B)
		LD_FMT_DISKIDS			FormatDiskIDs;
		LD_FMT_DISK_STRUCTURE	FormatDiskStructures;
#endif
		LD_FMT_LD_PARAM			FormatLD;
	} param;
} CREATE_LD_PARAM, buf_space * bpCREATE_LD_PARAM;

/* Create Array Flags */
#define CREATE_LD_F_QUICK_INIT		0x01	/* init flags are exclusive */
#define CREATE_LD_F_FOREGRND_INIT	0x02
#define CREATE_LD_F_BACKGRND_INIT	0x04

/* Rounding Scheme */
#define RAID_ROUNDING_NONE		0x00
#define RAID_ROUNDING_GB		0x01
#define RAID_ROUNDING_10GB		0x02	/* not supported yet */
#define RAID_ROUNDING_CUSTOM	0x03	/* not supported yet */

/* TODO: default rounding scheme needs change in the future */
#define RAID_ROUNDING_DEFAULT	( RAID_ROUNDING_NONE )

typedef struct _LD_MIGRATION_PARAM
{
	U8	Format;							/* LD_PARAM_FORMAT_xxx */
	/* common fields */
	U8	RaidLevel;						/* target array RAID level */
	U8	SubArrayCount;					/* ignored if RaidLevel != blRAID50 */
	U8	bReserved;
	U32 StripeBlockSize;				/* target array stripe block size */

	/* RAID_MF specific parameters, ignored if RaidLevel != blRAID_MF */
	U8	MfParityCount;
	U8	MfSymmetric;
	I16	MfShift;

	U32	wReserved[3];

	union
	{
		LD_FMT_EXTENTS			FormatExtents;
		LD_FMT_DISKIDS			FormatDiskIDs;
		LD_FMT_DISK_STRUCTURE	FormatDiskStructures;
	} param;
}
LD_MIGRATION_PARAM, buf_space * bpLD_MIGRATION_PARAM;

typedef struct _GLOBAL_SETTINGS_PAGE0		/* Page Code 0x01 */
{
	/* To avoid BIOS memory usage, put settings BIOS irrelated in the end */
	U32 JbodDiskIDBitmap;
#if !defined(_X8616B) && !defined(SIMULATE_BIOS_ENV)
	U32 RebuildPolicy;
	U32 SyncPolicy;
	U32 InitPolicy;
	U32 MigrationPolicy;
	U32 MediaPatrolPolicy;

	/*
	 * usually all maintenance activities share the same policy, the setting is
	 * designed independently for later change
	 */
	U8	RebuildRate;						/* rebuild rate */
	U8	SyncRate;							/* synchronization rate */
	U8	InitRate;							/* initialization rate */
	U8	MediaPatrolRate;					/* media patrol rate */

	U8	MigrationRate;						/* migration rate */
	U8	SmartCheckRate;
	U8	EnclosurePollInterval;
	U8	SmartCheckEnable;

	U8	AutoRebuildEnable;
	U8	bReserved[3];

	U32	wReserved[3];
	/* ... subject to change ... */
#endif
}
GLOBAL_SETTINGS_PAGE0, buf_space *bpGLOBAL_SETTINGS_PAGE0;

/*
 * If hotspare is enabled and when array rebuild is required, engine will first
 * try to find a block from dedicated spare pool first and then the global spare
 * pool.
 */
#define REBUILD_POLICY_HOTSPARE_ENABLED bit( 0 )

/*
 * if auto rebuild is enabled, and when a new block is available, engine will try
 * to start rebuild automatically.
 */
#define REBUILD_POLICY_AUTO_ENABLED bit( 1 )

/*
 * svucARRAY_COMMAND/acmdSAVE_MDD ? svucCONTROLLER_COMMAND/ccmdSAVE_DISK_MDD ?
 * CDB_SAVE_DISK_MDD.Flags
 */
#define SAVEMDD_F_NONE			0
#define SAVEMDD_F_PREREAD		bit( 0 )	/* this flag is reserved for internal usage
											 * ONLY! */

/* CDB_SAVE_DISK_MDD.Control */
#define SAVEMDD_C_NORMAL			0		/* whatever */
#define SAVEMDD_C_CLEAR_DISK_MDD	1		/* clear disk MDD */
#define SAVEMDD_C_SAVE_BSL			2		/* to update BSL only */
#if defined(ENABLE_BBM)
#define SAVEMDD_C_SAVE_BBM			3		/* BBM update required */
#endif
#if defined(ENABLE_BACKUP)
#define SAVEMDD_C_SAVE_BACKUP		4
#endif
#define SAVEMDD_C_CREATE_ARRAY		5

#if defined( _COMPILER_MS_C_COMPATIBLE )
#pragma pack( 1 )
#endif /* COMPILER_MS_C_COMPATIBLE */

typedef struct	_CDB_SAVE_DISK_MDD
{
	U8	OpCode;						/* FIXED: svucCONTROLLER_COMMAND */
	U8	SubOpCode;					/* FIXED: ccmdSAVE_DISK_MDD */
	U8	Flags;						/* IN: see above SAVEMDD_F_xxx */
	U8	Control;					/* IN: see above SAVEMDD_C_xxx */
	U8	BlockID;					/* VOLATILE: current Block ID in processing, engine
									 * internal use */
	U8	bFailCase;					/* Engine internal use. */
	U8	DiskID;						/* Engine internal use. */
	U8	bMDD;						/* Engine internal use. */
} PACKED			CDB_SAVE_DISK_MDD, buf_space * bpCDB_SAVE_DISK_MDD;

#define MAX_REBUILD_TARGETS			6

typedef struct	_CDB_START_BGA
{
	U8	OpCode;						/* FIXED svucARRAY_COMMAND */
	U8	SubOpCode;					/* FIXED acmdBGA_START */
	U8	Category;					/* BGA_CAT_xxx */
	union
	{
		U8	TargetBlock;			/* for BGA_CAT_REBUILD */
		U8	SyncKind;				/* SYNC_KIND_xxx, for BGA_CAT_SYNC */
		U8	InitKind;				/* INIT_KIND_xxx, for BGA_CAT_INIT */
		/* for migration, data payload is required */
	} PACKED param;
	U8	RebuildTargetCount;			/* Usually it is 1 but more for RAID_MF */
	U8	RebuildTargets[MAX_REBUILD_TARGETS-1];
}
PACKED	CDB_START_BGA, buf_space * bpCDB_START_BGA;

#define BGA_CAT_REBUILD			0
#define BGA_CAT_INIT			1
#define BGA_CAT_SYNC			2
#define BGA_CAT_MIGRATION		3
#define BGA_CAT_REDUILD_DISKS	4  /* Reserved for Shasta?Aurora */
#if defined(ENABLE_BACKUP)
#define BGA_CAT_BACKUP			5
#define BGA_CAT_FASTCOPY		6
#endif

#define SYNC_KIND_NORMAL		0	/* normal sync (common sense) */
#define SYNC_KIND_CHECKONLY		1	/* consistency checking only, without fixing */

#define INIT_KIND_QUICK			0	/* quick initialization */
#define INIT_KIND_FULL_FOREGRND 1	/* full foreground initialization */
#define INIT_KIND_FULL_BACKGRND 2	/* full background initialization */

typedef struct	_CDB_BGA_CONTROL
{
	U8	OpCode;						/* FIXED svucARRAY_COMMAND */
	U8	SubOpCode;					/* FIXED acmdBGA_CONTROL */
	U8	Action;						/* MT_ACT_xxx */
}
PACKED			CDB_BGA_CONTROL, buf_space * bpCDB_BGA_CONTROL;

typedef struct	_CDB_BLOCK_CONTROL
{
	U8	OpCode;						/* FIXED svucCONTROLLER_COMMAND */
	U8	SubOpCode;					/* FIXED ccmdBLOCK_CONTROL */
	U8	Action;						/* BC_ACT_xxx */
	U8	Flags;						/* BC_F_xxx */
	union
	{
		U8	BlockID;
		U8	DiskID;					/* for BC_ACT_WIPEOUT_DISK_MDD */
	} PACKED id;
	union
	{
		struct
		{
			U8	DedicatedArrayID;	/* 0xff if to make a global spare block */
		} PACKED make_spare_block;
		struct
		{
			U8	ArrayID;
		} PACKED add_block;				/* for BC_ACT_ADD_TO_ARRAY */
		struct
		{
			U8	Param1;
			U8	Param2;
		} PACKED generic;
	} PACKED param;
}
PACKED	CDB_BLOCK_CONTROL, buf_space * bpCDB_BLOCK_CONTROL;

typedef struct _SPLIT_BLOCK_PARAM
{
	LBA	wOffsetLBA;
} SPLIT_BLOCK_PARAM, buf_space * bpSPLIT_BLOCK_PARAM;


/* CDB_BLOCK_CONTROL.Flags */
#define BC_F_BRUTAL_FORCE	0x01	/* valid with BC_ACT_WIPEOUT_DISK_MDD */

/*
 * make the specified block to be a spare block. the given block should be a
 * free/spare/un-recognized disk.
 */
#define BC_ACT_MAKE_SPARE_BLOCK 0x00

/*
 * make the specified block to be a free block. the given block should be a
 * free/spare/un-recognized disk.
 */
#define BC_ACT_MAKE_FREE_BLOCK	0x01

/*
 * wipe block MDD will remove the block from the disk. The given block should be
 * free/spare/un-recognized disk.
 */
#define BC_ACT_WIPEOUT_BLOCK_MDD	0x02

/* wipe out disk MDD will remove all blocks existed on the disk */
#define BC_ACT_WIPEOUT_DISK_MDD 0x03

/*
 * add a free/spare block into a critical redundant array. If a array dedicated
 * spare block is to be added, it must be the spare of the given array.
 */
#define BC_ACT_ADD_TO_ARRAY 0x04

/*
 * Following BC_ACT_xxx are reserved. It will be used when we are going to support
 * multiple blocks per disk. MAKE_BLOCK_PARAM is used for data payload.
 */
#define BC_ACT_SPLIT_BLOCK	0x10

#define BC_ACT_DELETE_BLOCK	0x11

/*
 -----------------------------------------------------------------------------------------
    To remove a block, BC_ACT_WIPEOUT_BLOCK_MDD will be extended and used
 -----------------------------------------------------------------------------------------
 */
typedef struct	_CDB_INIT_CONFIG
{
	U8	OpCode;		/* FIXED svucCONTROLLER_COMMAND */
	U8	SubOpCode;	/* FIXED ccmdINIT_CONFIG */
	U8	Flags;		/* IN see below INIT_CONFIG_xxx, internal used */
	U8	StartDiskID;
	U8	EndDiskID;
	U8	MDDFormat;	/* engine internal usage. current processing MDD format */
	U8	DiskID;		/* engine internal usage. current disk ID in processing */
	U8	bFailCase;	/* engine internal usage. */
}
PACKED			CDB_INIT_CONFIG, buf_space * bpCDB_INIT_CONFIG;

#define INIT_CONFIG_FIRST_READ	bit( 0 )

typedef struct	_CDB_PATROL_CONTROL
{
	U8	OpCode;		/* FIXED svucCONTROLLER_COMMAND */
	U8	SubOpCode;	/* FIXED ccmdPATROL_CONTROL */
	U8	Action;		/* IN MT_ACT_xxx */
	U8	DiskID;		/* IN */
}
PACKED			CDB_PATROL_CONTROL, buf_space * bpCDB_PATROL_CONTROL;

typedef struct	_CDB_ARRAY_CONTROL
{
	U8	OpCode;		/* FIXED svucARRAY_COMMAND */
	U8	SubOpCode;	/* FIXED acmdARRAY_CONTROL */
	U8	Action;		/* AC_ACT_xxx */
	U8	Flags;		/* AC_F_xxx */
}
PACKED	CDB_ARRAY_CONTROL, buf_space * bpCDB_ARRAY_CONTROL;

/*
 -----------------------------------------------------------------------------------------
    AC_F_xxx to be added
 -----------------------------------------------------------------------------------------
 */
typedef struct	_CDB_GLOBAL_INFO
{
	U8	OpCode;		/* svucCONTROLLER_COMMAND */
	U8	SubOpCode;	/* ccmdGET/SET_GLOBAL_CONFIG */
	U8	PageCode;
	U8	Control;
}
PACKED CDB_GLOBAL_INFO, buf_space * bpCDB_GLOBAL_INFO;

#if defined( _COMPILER_MS_C_COMPATIBLE )
#pragma pack()
#endif /* COMPILER_MS_C_COMPATIBLE */

/*
 * During array plug-in, RAID engine won't automatically activate the array until
 * it is fully functional. AC_ACT_ACTIVATE_ARRAY provides a choice to force engine
 * to activate the array.
 */
#define AC_ACT_ACTIVATE_ARRAY	0x00

/*
 * AC_ACT_UNMOUNT_ARRAY will clear all internal data structures of the given
 * array. Usually this can be used before unplug an array. Since this operation is
 * dangerous, AP should force OS to flush system cache data and remove the device
 * before unmount an array.
 */
#define AC_ACT_UNMOUNT_ARRAY	0x01

/* page codes for acmdGET/SET_GLOBAL_CONFIG */
#define GLOBAL_INFO_PAGE_CONTROLLER_INFO	0x00	/* read only: controller info */
#define GLOBAL_INFO_PAGE_SETTTING_PAGE		0x01	/* read/write: controller setting */
#define GLOBAL_INFO_PAGE_PHYSICAL_DRIVES	0x02	/* read only: physical drives */
#define GLOBAL_INFO_PAGE_BLOCKS				0x03	/* read only: logical blocks */
#define GLOBAL_INFO_PAGE_ARRAYS				0x04	/* read only: arrays */
#define GLOBAL_INFO_PAGE_CAPABILITIES		0x05	/* read only: CONTROLLER_CAPABILITIES */
#define GLOBAL_INFO_PAGE_PD_SETTING			0x06	/* read/write: physic disk setting */
#define GLOBAL_INFO_PAGE_ARRAYS_EX			0x07	/* read only: arrays */
#define GLOBAL_INFO_CONTROL_ALL				0xff

#define SMART_STATUS_NO_SMART			0
#define SMART_STATUS_OK					1
#define SMART_STATUS_ERROR				2
#define SMART_STATUS_NOT_SUPPORTED		3

typedef struct _PHYSICAL_DRIVE_INFO
{
	U32 Flags;				/* same as CAM_DISK.Flags */

	U8	ChannelID;
	U8	TargetID;			/* can be used as port address defined in SATA II spec for
							 * port multiplier */
	U8	AssociatedEnclosureID;
	U8	CarrierIDInEnclosure;

	U8	Features;
	U8	PIOMode;
	U8	MDMAMode;
	U8	UDMAMode;

	U32	Features2;			// prepare Physical HDD inf for i2api by tammy

	U64 MaximumLBA;			/* This is definitely necessary */

	U32 Cylinders;			/* # of cyl */
	U8	Heads;				/* # of hd per cyl */
	U8	Sectors;			/* # of sctr per hd */
	U8	SmartStatus;		/* SMART_STATUS_xxx */
	U8	bReserved[1];

	U8	Model[24];
	U8	SerialNo[24];
	U8	FWVersion[16];

	U8	DiskWriteCache;		/* 1 for disk write back, 0 for write through */
	U8	bReserved1[3];
	U32	wReserved[3];
}
PHYSICAL_DRIVE_INFO, buf_space * bpPHYSICAL_DRIVE_INFO;

typedef struct _LOGICAL_BLOCK_INFO
{
	U16 Flags;			/* same as BLOCK.blFlag */
	U16 hReserved;

	U8	bStream;		/* valid if the block is assigned to an array */
	U8	bDiskID;
	U8	bArrayID;		/* valid if the block is assigned to an array */
	U8	bExtentIndex;	/* the extent index on the disk, 0~MAX_BLOCK_PER_DISK-1 */

	LBA StartLBA;
	LBA Size;
#if defined(ENABLE_BACKUP)
	U32	bDate;
	U32	wReserved[3];
#else
	U32  wReserved[4];
#endif
}
LOGICAL_BLOCK_INFO, buf_space * bpLOGICAL_BLOCK_INFO;

typedef struct _LOGICAL_DEVICE_INFO
{
	U16 Flags;				/* same as ARRAY.blFlag */
	U16 hStripeBlockSize;	/* stripe block size */
	U8	RaidLevel;			/* ARRAY.blRAID */
	U8	bWidth;
	U8	CacheMode;			/* see LUN.CacheMode */
	U8	DiskWriteCache;		/* see LUN.DiskWriteCache */
	U8	DiskReadLookahead;	/* see LUN.DiskReadLookahead */
	U8	Lookahead;
	U8	Consolidation;
	U8	TargetRaidLevel;	/* If it's under conversion. */
	U8	RealCacheMode;		/* AP should check this field when CacheMode=LD_CM_AUTO*/
//	U8	bReserved[3];
	U8	PathId;
	U8	TargetId;
	U8	Lun;

	LBA wSize;				/* array size */

	U8	Name[ARRAY_NAME_LENGTH];

	U8	BlockIDs[LD_MAX_EXTENTS];
}
LOGICAL_DEVICE_INFO, buf_space * bpLOGICAL_DEVICE_INFO;

typedef struct _LOGICAL_DEVICE_INFO_EX
{
	LOGICAL_DEVICE_INFO	Basic;
	U8	MfParityCount;
	U8	MfSymmetric;
	I16	MfShift;
	U8	SubArrayCount;
#if defined(ENABLE_BACKUP)
	U16  ExtendFlags;				/* same as Array.blExtendFlag */
	U8	bReserved[1];
#else
	U8	bReserved[3];
#endif
	U32	wReserved1[2];
	U32	SectorSize;					/* 2T Issue*/
	U32	wReserved2;
}
LOGICAL_DEVICE_INFO_EX, buf_space * bpLOGICAL_DEVICE_INFO_EX;

typedef struct _CONTROLLER_CAPABILITIES
{
	U32	Reserved1;
	U8	MaximumDiskSupported;			/* MAX_DISK_SUPPORTED */
	U8	MaximumExtentPerDisk;			/* MAX_BLOCK_PER_DISK */
	U8	MaximumLogicalDriveSupported;	/* MAX_ARRAY_SUPPORTED */
	U8	MaximumExtentPerLogicalDrive;	/* MAX_STREAM_PER_ARRAY */
	U8	MaximumPortSupported;			/* MAX_PORT_SUPPORT */
	U8	bReserved[3];
	U32	Reserved2[13];
} 
CONTROLLER_CAPABILITIES, buf_space * bpCONTROLLER_CAPABILITIES;

typedef struct _PHYSICAL_DRIVE_SETTING
{
	U32 Flags;	/* CAM_DISK.Flags */

	U8	DiskWriteCacheEnabled;
	U8	DiskReadLookaheadEnabled;
	U8	DiskNCQEnabled;
	U8	DiskTCQEnabled;
	/*
	// smart checking is now a global setting.
	U8	SmartCheckEnabled;
	*/
	U32	Features;	/* CAM_DEV_F_xxx */
	U32 wReserved[5];
} 
PHYSICAL_DRIVE_SETTING, buf_space * bpPHYSICAL_DRIVE_SETTING;

#define LD_STATUS_OPTIMAL	0
#define LD_STATUS_DEGRADE	1	/* Still have redundant data but lost at least one stream. */
#define LD_STATUS_OFFLINE	2
#define LD_STATUS_INVALID	3
#define LD_STATUS_CRITICAL	4	/* Don't have any redundant data. */

#define LD_STATE_NORMAL			0
#define LD_STATE_CONFIGURING	1

#define LD_BGA_NONE					0
#define LD_BGA_REBUILD				1
#define LD_BGA_FOREGRND_INIT		2
#define LD_BGA_BACKGRND_INIT		3
#define LD_BGA_CONSISTENCY_CHECK	4
#define LD_BGA_SYNC					5
#define LD_BGA_MIGRATION			6
#if defined(ENABLE_BACKUP)
#define LD_BGA_BACKUP				7
#endif
#define LD_BGA_STATE_NONE			0
#define LD_BGA_STATE_RUNNING		1
#define LD_BGA_STATE_ABORTED		2
#define LD_BGA_STATE_PAUSED			3
#define LD_BGA_STATE_AUTOPAUSED		4

typedef struct _LOGICAL_DEVICE_STATUS
{
	U8	Status;		/* LD_STATUS_xxx */
	U8	State;		/* LD_STATE_xxx */
	U8	Bga;		/* LD_BGA_xxx */
	U8	BgaState;	/* LD_BGA_STATE_xxx */
	U16	BgaPercentage;	/* xx% */
	U8	Reserved[2];
} 
LOGICAL_DEVICE_STATUS, buf_space * bpLOGICAL_DEVICE_STATUS;

#define PDBG_READ_BLOCKS	0x00	/* CDB_DBG_DISK */
#define PDBG_WRITE_BLOCKS	0x01
#define PDBG_BREAK_ECC		0x02

#define PDBG_INJECT_ERROR	0x10	/* CDB_DBG_ERRINJ in eng_esl.h */
#define PDBG_ATA_PASSTHRU	0x11
#define PDBG_SATA_PASSTHRU	0x12

#define PDBG_READ_NVRAM		0x20	/* CDB_DBG_NVRAM */
#define PDBG_WRITE_NVRAM	0x21

#define PDBG_WRITEBACKCACHE 0x30	/* CDB_DBG_DISK_SIMPLE */
#define PDBG_READLOOKAHEAD	0x31
#define PDBG_FLUSH_DISK		0x32
#define PDBG_IDENTIFY		0x33
#define PDBG_RESVSECTOR_LBA 0x34
#define PDBG_BSL_DUMP		0x35
#define PDBG_BBM_DUMP		0x36
#define PDBG_WR_DIMM		0x37
#define PDBG_CMP_DIMM		0x38
#define PDBG_RW_RESV_DIMM	0x39

#define PDBG_LBA_MAP		0x40	/* CDB_DBG_LBA_MAP */

#define PDBG_CC_obsolete	0x50
#define PDBG_CC_DEBUGGING	0x51
#define PDBG_CC_DUMP_OS_REQ_BUF		0	/* CDB[1]==PDBG_CC_DEBUGGING */
#define PDBG_CC_FLUSH				1	/* CDB[3] = Array ID */
#define PDBG_CC_CLEAN_IDLE			2	/* CDB[3] = Array ID */
#define PDBG_CC_DISABLE_BYPASS		3	/* CDB[3] = Array ID */
#define PDBG_CC_ENABLE_BYPASS		4	/* CDB[3] = Array ID */

#define PDBG_I2C_READ		0x60
#define PDBG_I2C_WRITE		0x61

/* CDB_DBG_ERRINJ.injectOpCode CDB[1]==PDBG_INJECT_ERROR */
#define PDBG_INJECT_OP_DUMP		0x00
#define PDBG_INJECT_OP_INJECT	0x01
#define PDBG_INJECT_OP_PURGE	0x02
#define PDBG_INJECT_OP_CLEAR	0x03

/* CDB_DBG_ERRINJ.flags */
#define ES_FLAGS_WRITE_RECOVERABLE	0x01	/* error can be recovered on writing */
#define ES_FLAGS_ERROR_ON_READ		0x02	/* error on reading */
#define ES_FLAGS_ERROR_ON_WRITE		0x04	/* error on writing */
#define ES_FLAGS_ERROR_UNPLUG		0x08	/* disk unplug */
#define ES_FLAGS_ERROR_PLUGIN		0x10	/* disk plugin */
#define ES_FLAGS_ERROR_ON_VERIFY	0x20	/* error on verify */
#define ES_FLAGS_ERROR_SMART_ERROR	0x40	/* smart error */

/* special flags (actually not flags) */
#define ES_FLAGS_ERROR_SET_DOWN U8_MAX

#if defined( _COMPILER_MS_C_COMPATIBLE )
#pragma pack( 1 )
#endif /* COMPILER_MS_C_COMPATIBLE */

typedef struct	_CDB_DBG_ERRINJ
{
	U8	opCode;			/* svucDEBUGGING_COMMAND (0xE2) */
	U8	subOpCode;		/* PDBG_INJECT_ERROR */
	U8	diskID;
	U8	injectOpCode;	/* PDBG_INJECT_OP_xxx */
	U32 startLBA;
	U16 count;
	U8	status;
	U8	flags;			/* ES_FLAGS_xxxx */
}
PACKED CDB_DBG_ERRINJ, buf_space * bpCDB_DBG_ERRINJ;

/*
 -----------------------------------------------------------------------------------------
    Debugging request CDB definitions vary with different sub opcodes. Here is the
    generic debugging request CDB.
 -----------------------------------------------------------------------------------------
 */
typedef struct	_CDB_DBG
{
	U8	opCode;		/* svucDEBUGGING_COMMAND(0xE2) */
	U8	subOpCode;	/* PDBG_xxx */
	U8	bReserved[14];
}
PACKED CDB_DBG, buf_space * bpCDB_DBG;

typedef struct _CDB_DBG_DISK_SIMPLE
{
	U8	opCode;		/* svucDEBUGGING_COMMAND(0xE2) */
	U8	subOpCode;	/* PDBG_xxx (0x30~0x3f) */
	U8	diskID;
	U8	action;
}
PACKED CDB_DBG_DISK_SIMPLE, buf_space * bpCDB_DBG_DISK_SIMPLE;

typedef struct _CDB_DBG_DISK
{
	U8	opCode;		/* svucDEBUGGING_COMMAND(0xE2) */
	U8	subOpCode;	/* PDBG_xxx (0x00~0x0f) */
	U8	diskID;
	U8	bReserved;
	LBA	lba;		/* FIXME: align it! */
	U16 count;		/* block count */
	U16 hReserved;
}
PACKED CDB_DBG_DISK, buf_space * bpCDB_DBG_DISK;

typedef struct _CDB_DBG_NVRAM
{
	U8	opCode;		/* svucDEBUGGING_COMMAND(0xE2) */
	U8	subOpCode;	/* PDBG_xxx (0x20~0x2f) */
	U16 length;		/* byte length */
	U32 offset;		/* NVRAM offset */
}
PACKED CDB_DBG_NVRAM, buf_space * bpCDB_DBG_NVRAM;

/*=============================================================================*/
/* svucDEBUGGING_COMMAND/PDBG_LBA_MAP */
#define PDBG_LBA_MAP_ACT_D2A	0x00	/* map disk lba to array lba */
#define PDBG_LBA_MAP_ACT_A2D	0x01	/* map array lba to disk lba */
#define PDBG_LBA_MAP_ACT_A2A	0x02	/* map array lba to array lba (migration only) */

typedef struct _DBG_LBA_MAP_PARAM_D2A
{
	LBA	DiskLBA;				/* IN */
	U8	DiskID;					/* IN */
	U8	MigrationTarget;		/* IN */
	U8	Reserved1[2];			/* IN */

	U8	IsMapSucceed;			/* OUT */
	U8	MappedArrayID;			/* OUT */
	U8	IsMappedToParity;		/* OUT */
	U8	IsMigrationTarget;		/* OUT */
	LBA	MappedArrayLBA;			/* OUT */
} 
PACKED DBG_LBA_MAP_PARAM_D2A, buf_space * bpDBG_LBA_MAP_PARAM_D2A;

typedef struct _DBG_LBA_MAP_PARAM_A2D
{
	LBA	ArrayLBA;				/* IN */
	U8	ArrayID;				/* IN */
	U8	IsMigrationTarget;		/* IN, valid and checked only if the array in migration */
	U8	Reserved1[2];			/* IN */

	U8	IsMapSucceed;			/* OUT */
	U8	MappedDiskID;			/* OUT */
	U8	Reserved2[2];			/* OUT */
	LBA	MappedDiskLBA;			/* OUT */
} 
PACKED DBG_LBA_MAP_PARAM_A2D, buf_space * bpDBG_LBA_MAP_PARAM_A2D;

typedef struct _DBG_LBA_MAP_PARAM_A2A
{
	LBA	ArrayLBA;				/* IN */
	U8	ArrayID;				/* IN */
	U8	Reserved1[3];			/* IN */

	U8	IsMapSucceed;			/* OUT */
	U8	Reserved2[3];			/* OUT */
	LBA	MappedArrayLBA;			/* OUT */
} 
PACKED DBG_LBA_MAP_PARAM_A2A, buf_space * bpDBG_LBA_MAP_PARAM_A2A;

typedef struct _CDB_DBG_LBA_MAP
{
	U8	opCode;		/* svucDEBUGGING_COMMAND(0xE2) */
	U8	subOpCode;	/* PDBG_LBA_MAP (0x20~0x2f) */
	U8	action;		/* PDBG_LBA_MAP_ACT_xxx */
}
PACKED CDB_DBG_LBA_MAP, buf_space * bpCDB_DBG_LBA_MAP;
/*=============================================================================*/

typedef struct	_CDB_ATA_PTI
{
	U8	OpCode;			/* FIXED svucCONTROLLER_COMMAND */
	U8	SubOpCode;		/* FIXED ccmdATA_PASSTHRU */
	U8	bCommand;		/* refer to ATA spec. */
	U8	bFeatures;		
	U8	bSectorCount;
	U8	bSectorNumber;
	U8	bCylinderLow;
	U8	bCylinderHigh;
	U8	DiskID;
	U8  blFlag;
	U8	bReserved[3];
}
PACKED CDB_ATA_PTI, buf_space * bpCDB_ATA_PTI;

#define CDB_GET_INFO_CMD_PATROL	0x00
#define CDB_GET_INFO_CMD_ERRORTABLE	0x01

typedef struct _DISK_PATROL_INFO
{
	LBA	wWatermark;		/* patrol watermark */
	U16	hFlags;			/* see above BGA_STATE_xxx */
	U16	hCount;			/* loop count */
	U16	hErrorCount;	/* error detected during media patrol */
	U16 hPercentage;	/* Runtime percentage, internal usage */
} DISK_PATROL_INFO, buf_space * bpDISK_PATROL_INFO;

typedef struct _CDB_GET_INFO
{
	U8	OpCode;			/* FIXED svucCONTROLLER_COMMAND */
	U8	SubOpCode;		/* FIXED ccmdGET_INFO */
	U8	Command;		/* CDB_GET_INFO_CMD_xxx */
	union
	{
		U8 DiskID;
	} PACKED patrol;
} PACKED CDB_GET_INFO, buf_space * bpCDB_GET_INFO;

/*=====================================================================================*/
/*
 * Data struture and definition for Engine_GetEvent
 */
#define MAX_EVENTLOG_PARAMS		6
#define ERRORLOG_NUM			21 /* It was 19 if defined 20
                                   */

/* logevent.WarningLevel */
#define EVENT_LEVEL_ERROR		0x00
#define EVENT_LEVEL_WARNING		0x01
#define EVENT_LEVEL_INFORMATION 0x02

struct logevent
{
	U32 id;
	U32 TimeStamp;
	U32 Flags;						/* internal usage, AP shall not touch it */
	U32 NumOfParams;
	U32 WarningLevel;
	union
	{
		U32 params[MAX_EVENTLOG_PARAMS ];
	} ParamList;
	U8	Name[ARRAY_NAME_LENGTH];
};

#define CDB_EVENT_CMD_GETCOUNT		0x1
#define CDB_EVENT_CMD_GETCONTENT	0x2
typedef struct	_CDB_EVENT
{
	U8	OpCode;			/* FIXED svucCONTROLLER_COMMAND */
	U8	SubOpCode;		/* FIXED ccmdEvent */
	U8	Command;		/* Refer to CDB_EVENT_CMD_XXXX */
	U8	bEventCount;		
}
PACKED CDB_EVENT, buf_space * bpCDB_EVENT;

typedef struct _CDB_SMART_CHECK
{
	U8	OpCode;			/* FIXED svucCONTROLLER_COMMAND */
	U8	SubOpCode;		/* FIXED ccmdSMART_CHECK */
	U8	Control;
	U8	DiskID;
	U32	DiskIDMap;
}
PACKED CDB_SMART_CHECK, buf_space * bpCDB_SMART_CHECK;

typedef struct	_CDB_DBG_I2C
{
	U8	opCode;			/* svucDEBUGGING_COMMAND (0xE2) */
	U8	subOpCode;		/* PDBG_I2C_XXX */
	U8	bus;			/* bus number */
	U8	length;			/* data length */
	U16	address;		/* slave address. Currently we use 7bit address but later we may support 10 bit.*/
	U8	reserved[2];
}
PACKED CDB_DBG_I2C, buf_space * bpCDB_DBG_I2C;

#if defined(NEW_ERROR_HANDLING)
typedef struct _CDB_ERROR_FIXING
{
	U8	opCode;			/* FIXED svucCONTROLLER_COMMAND */
	U8	subOpCode;		/* FIXED ccmdERROR_FIXING */
	U8	diskID;			/* Error Disk */
	U8	errorCmd;		/* Error Command */
	U8	reserved[4];	
	LBA	lba;
}
PACKED CDB_ERROR_FIXING, buf_space * bpCDB_ERROR_FIXING;
#endif /* NEW_ERROR_HANDLING */

#if defined( _COMPILER_MS_C_COMPATIBLE )
#pragma pack( )
#endif /* COMPILER_MS_C_COMPATIBLE */

/*------------------------------------------------------------------*/
#define COMPOSE_SCSI_CDB_LBA( bCDB, lba ) do {				\
	if( scsiIs16BAccessCmd((bCDB)[0]) )	{					\
		REVERSE_BYTES_U32( &(bCDB)[2], &(lba).u.High );		\
		REVERSE_BYTES_U32( &(bCDB)[6], &(lba).u.Low );		\
	}														\
	else {													\
		REVERSE_BYTES_U32( &(bCDB)[16], &(lba).u.High );		\
		REVERSE_BYTES_U32( &(bCDB)[2], &(lba).u.Low );		\
	}														\
} while(0)

#define COMPOSE_SCSI_CDB_COUNT( bCDB, hCount ) do {			\
	if( scsiIs16BAccessCmd((bCDB)[0]) )	{					\
		(bCDB)[10] = (bCDB)[11] = 0;						\
		REVERSE_BYTES_U16( &(bCDB)[12], &(hCount) );		\
	}														\
	else {													\
		REVERSE_BYTES_U16( &(bCDB)[7], &(hCount) );			\
	}														\
} while(0)

#define COMPOSE_SCSI_CDB( bCDB, lba, hCount ) do {			\
	if( scsiIs16BAccessCmd((bCDB)[0]) )	{					\
		REVERSE_BYTES_U32( &(bCDB)[2], &(lba).u.High );		\
		REVERSE_BYTES_U32( &(bCDB)[6], &(lba).u.Low );		\
		(bCDB)[10] = (bCDB)[11] = 0;						\
		REVERSE_BYTES_U16( &(bCDB)[12], &(hCount) );		\
	}														\
	else {													\
		REVERSE_BYTES_U32( &(bCDB)[16], &(lba).u.High );		\
		REVERSE_BYTES_U32( &(bCDB)[2], &(lba).u.Low );		\
		REVERSE_BYTES_U16( &(bCDB)[7], &(hCount) );			\
	}														\
} while(0)

/*------------------------------------------------------------------*/

#define GET_LBA_FROM_SCSI_CDB( bCDB, lba ) do {				\
	if( scsiIs16BAccessCmd((bCDB)[0]) )	{					\
		REVERSE_BYTES_U32( &(lba).u.High, &(bCDB)[2] );		\
		REVERSE_BYTES_U32( &(lba).u.Low,  &(bCDB)[6] );		\
	}														\
	else {													\
		REVERSE_BYTES_U32( &(lba).u.High, &(bCDB)[16] );		\
		REVERSE_BYTES_U32( &(lba).u.Low, &(bCDB)[2] );		\
	}														\
} while(0)

#define GET_COUNT_FROM_SCSI_CDB( bCDB, hCount ) do {		\
	if( scsiIs16BAccessCmd((bCDB)[0]) )	{					\
		REVERSE_BYTES_U16( &(hCount), &(bCDB)[12] );		\
	}														\
	else {													\
		REVERSE_BYTES_U16( &(hCount), &(bCDB)[7] );			\
	}														\
} while(0)

#define GET_ACCESS_FROM_SCSI_CDB( bCDB, lba, hCount ) do {	\
	if( scsiIs16BAccessCmd((bCDB)[0]) )	{					\
		REVERSE_BYTES_U32( &(lba).u.High, &(bCDB)[2] );		\
		REVERSE_BYTES_U32( &(lba).u.Low,  &(bCDB)[6] );		\
		REVERSE_BYTES_U16( &(hCount), &(bCDB)[12] );		\
	}														\
	else {													\
		REVERSE_BYTES_U32( &(lba).u.High, &(bCDB)[16] );		\
		REVERSE_BYTES_U32( &(lba).u.Low, &(bCDB)[2] );		\
		REVERSE_BYTES_U16( &(hCount), &(bCDB)[7] );			\
	}														\
} while(0)

/*------------------------------------------------------------------*/

#endif /* ENGINE_SCSI3_H */
