#ifndef NAPA_CONFIG_H
#define NAPA_CONFIG_H

#if defined(_WINAPP)
	#include "cfg_wapp.h"
#elif defined(_WINDRIVER)
	#include "cfg_wdrv.h"
#elif defined(_LINUXDRIVER)
	#include "cfg_linux.h"
#elif defined(_UWDRIVER)
	#include "cfg_uw.h"
#elif defined(BIOS)
	#include "cfg_bios.h"
#elif defined(_SHASTAFW)
	#include "cfg_shasta.h"
#elif defined(__CC_ARM)
	#include "cfg_arm.h"
#else
	#error "What are you going to build?"
#endif

#if !defined(MAX_PORT_SUPPORT)
#define MAX_PORT_SUPPORT	4
#endif

/** Napa abstract channel conception, now channel is the same as disk. low layer CAM
    will help to hide channel from upper layers */
#if !defined(MAX_DISK_SUPPORTED)
	#if defined(CAM_SUPPORT_MASTERSLAVE)
		#define MAX_DISK_SUPPORTED		(MAX_PORT_SUPPORT*2)
	#else	/*CAM_SUPPORT_MASTERSLAVE*/
		#define MAX_DISK_SUPPORTED		MAX_PORT_SUPPORT
	#endif	/*CAM_SUPPORT_MASTERSLAVE*/
#endif /*MAX_DISK_SUPPORTED*/

#if !defined(_X8616B)
	#define MAX_CONCURRENT_IO_DEFAULT	32
#else
	#define MAX_CONCURRENT_IO_DEFAULT	1
#endif

#if defined(_X8616B) || defined(BIOS)
#define MAX_CONCURRENT_IO	MAX_CONCURRENT_IO_DEFAULT
#else
extern unsigned long MAX_CONCURRENT_IO;
#endif

#define MIN_CONCURRENT_IO		1

#if defined(JBOD_MODE_ONLY)
	#undef ENABLE_DM
	#define NO_RAID5
	#define NO_BGA
	#define DISABLE_ERROR_SIMULATION
	#define DISABLE_DEBUGGING_INTERFACE
	#undef ENABLE_TRANSACTION_LOG
	#define DISABLE_SMART_CHECKING
	#undef MAX_BLOCK_PER_DISK
	#define MAX_BLOCK_PER_DISK	1
	#undef V2MDD_INTEGRATED
	#undef DDF_INTEGRATED
	#define DISABLE_PBM
#endif

#if !defined(ENABLE_DM) || defined(USE_DMDIMM_ONLY)
/* media patrol must be disabled when BSL is not enabled. */
#undef XH_PATROL
#endif

#define UNREF_PARAM(x) x

#if defined(_X8616B) || defined(SIMULATE_BIOS_ENV)
	#if defined(ENGINE_TIMER_SUPPORT)
		#undef ENGINE_TIMER_SUPPORT
	#endif
#else
	#define ENGINE_TIMER_SUPPORT
#endif

#if (MAX_DISK_SUPPORTED > 4) && !defined(_X8616B)
#define ENGINE_SUPPORT_RMW
#endif

/* BIOS doesn't support error event log now. */
#if !defined(_X8616B) && !defined(BIOS)
#define ENGINE_LOG_ERROR_EVENT
#define LOG_ERR_EVENT_IN_NVRAM
#endif

#if defined(NVRAM_NOT_AVAILABLE)
#undef ENABLE_TRANSACTION_LOG
#undef LOG_ERR_EVENT_IN_NVRAM
#endif

#if defined(XOR_HOST_MEM_CAPABLE) && defined(DMA_HOST_TO_DISK_SUPPORTED)
	#define CC_BYPASS_SUPPORT
	#if defined(_X8616B)
		/* dont undefine PBM_DO_HDMA to save memory during Windows hibernation */
		#undef PBM_DO_HDMA
	#endif	/*_X8616B*/
#endif	/*defined(XOR_HOST_MEM_CAPABLE) && defined(DMA_HOST_TO_DISK_SUPPORTED)*/

#if defined(_WINDRIVER) && (MAX_DISK_SUPPORTED > 4)
/* Component extension size is growing bigger when supported disk is becoming bigger... */
#define USE_STATIC_DATA_FOR_COMPONENT_EXTENSIONS
#endif

#if !defined(MAX_BLOCK_PER_DISK)
	#define MAX_BLOCK_PER_DISK	2
#endif	/*MAX_BLOCK_PER_DISK*/

#if defined(ATI)		//change define for AMD(ATI) by tammy
#if (MAX_PORT_SUPPORT)==4
#define MAX_EXTENTS_SUPPORTED	8
#elif  (MAX_PORT_SUPPORT)==6
#define MAX_EXTENTS_SUPPORTED 10
#else
#define MAX_EXTENTS_SUPPORTED	(MAX_DISK_SUPPORTED*MAX_BLOCK_PER_DISK)
#endif
#else
#define MAX_EXTENTS_SUPPORTED	(MAX_DISK_SUPPORTED*MAX_BLOCK_PER_DISK)
#endif
#define MAX_LD_SUPPORTED		MAX_EXTENTS_SUPPORTED
#define MAX_LU_SUPPORTED		(MAX_LD_SUPPORTED+1)	/* plus one for the console device */

#if MAX_LU_SUPPORTED > 254
#error "so far cannot support more than 254 devices."
#endif

#if !defined(MAX_LU_PER_TARGET)
	#define MAX_LU_PER_TARGET	1
#endif	/* !MAX_LU_PER_TARGET */

#if MAX_LU_PER_TARGET > 1
	#if !defined(ENGINE_SUPPORT_LUN_MAP)
		#define ENGINE_SUPPORT_LUN_MAP
	#endif

	#define MAX_TARGET_SUPPORTED	(MAX_DISK_SUPPORTED+1)//samuel.liao 20060102
	#if MAX_LD_SUPPORTED > MAX_DISK_SUPPORTED * MAX_LU_PER_TARGET //samuel.liao 20051228
		#error	"MAX_LU_SUPPORTED must be less than MAX_PORT_SUPPORT * MAX_LU_PER_TARGET!"
	#endif
#else
	#define MAX_TARGET_SUPPORTED	MAX_LU_SUPPORTED
#endif

#include "napa_oem.h"

#endif /* NAPA_CONFIG_H */
