#ifndef OSD_DBG_H
#define OSD_DBG_H
/***************************************************************************
 *
 * Copyright (c) 1995 Promise Technology, Inc.  All rights reserved.
 * No part of this document may be reproduced or transmitted in any form or
 * by any means, electronic or mechanical, for any purpose, without the
 * express written permission of Promise Technology, Inc.
 *
 ***************************************************************************/

/***************************************************************************
 *
 * Author		:Jing Qian
 *
 * Module		:Debug Utility
 * Revision		:1.0.0.1
 * Checkin Date	:01/01/2004
 *
 * Description	:Use to debug the code in release and debug build
 *
 * History		:
 *
 ***************************************************************************/
#include "gen_typs.h"

#define DRIVERNAME "FASTSX"

/* Debug Interface 
 * 1. OSD_TRACE			Debug Version:		print out some message
 * 2. OSD_DPRINT		Release Version:	print out some message		//TBD: Name is confusing.
 * 3. OSD_BREAKPOINT	Release Version:	int 3 if debugger is attached or crash system if no debugger.
 * 4. OSD_DBGBREAKPOINT Debug Version:		int 3 if debugger is attached or crash system if no debugger.
 * 5. OSD_CrashSystem	Release Version:	Crash system in release version and debug version.
 * 6. OSD_ASSERT		Debug Version:		Assert in debug version
 * 7. OSD_DASSERT		Release Version:	Assert in release version.	//TBD: Name is confusing.
 */

/* For both debug and release version */
// Debugging Interface
#if defined( _WINAPP )
	#include <stdio.h>
	#include <assert.h>
	#include <stdarg.h>

	extern BOOL	g_bLogToFile;
	extern FILE* g_fpLog;
	extern void SetupLogFile();
	extern void LogToFile(const char* fmt, ...);

	#define OSD_DPRINT( _x_ )	do {	\
		printf _x_;						\
		if( g_bLogToFile )				\
		{								\
			LogToFile _x_;				\
		}								\
	} while(0)

	#define OSD_DASSERT(expr)	assert(expr)

#elif defined(_WINDRIVER)

#if defined(__cplusplus)
	EXTERNC {
#endif
		#include <ntddk.h>
#if defined(__cplusplus)
	}
#endif
	#define OSD_DPRINT(_x_)		do { DbgPrint _x_; } while(0)
	#define OSD_DASSERT(expr)	do { if (!(expr)) OSD_BREAKPOINT; } while(0)

#elif defined(_LINUXDRIVER)
#ifdef CONFIG_KERNEL_ASSERTS
/* kgdb stuff */
#define assert(p) KERNEL_ASSERT(#p, p)
#else
#define assert(p) do {	\
	if (!(p)) {	\
		printk(KERN_CRIT "BUG at %s:%d assert(%s)\n",	\
		       __FILE__, __LINE__, #p);			\
		BUG();	\
	}		\
} while (0)
#endif

	extern int wrap_print(const char *fmt,...);
	//#define OSD_DPRINT( _x_ )  wrap_print _x_ 
	//#ifdef NAPA_DEBUG
	#ifdef T3_DEBUG 
	#define OSD_DASSERT(p)
	/* add by sandy 20071224 for T3 debug mode */
	//#define OSD_DASSERT(p) assert(p)
	#define OSD_DPRINT( _x_ )  do { wrap_print _x_; } while(0) 
	#else
	#define OSD_DASSERT(p) 
	#define OSD_DPRINT( _x_ ) 
	#endif
#elif defined(_UWDRIVER)
	#define OSD_DASSERT(expr)
	extern int LINUX_print(const char *fmt,...);
	#define OSD_DPRINT( _x_ )  LINUX_print _x_ 

#elif defined(BIOS)

#ifdef EXE_BIOS
	#include <stdio.h>
	#include <assert.h>
	#include <stdarg.h>

	#define OSD_DPRINT( _x_ )	do {	\
		printf _x_;						\
	} while(0)
	
	#define OSD_DASSERT(expr)	assert(expr)
#else
	#define OSD_DPRINT( _x_ )
	#define OSD_DASSERT(expr)
#endif

#elif defined(__CC_ARM)
	#include <stdio.h>
	#define OSD_DPRINT(_x_)		do { printf _x_; } while(0)
	#define OSD_DASSERT(expr)	do { if (!(expr)) OSD_BREAKPOINT; } while(0)

#elif defined(_SHASTAFW)
		#include "sh_dbg.h"

		#define OSD_DPRINT( _x_ )	do { sh_print _x_; } while(0)

	#define OSD_DASSERT(expr)	do {													\
		if( !(expr) ) {																	\
			OSD_DPRINT( ("Assert failed at %s(%d): %s\n", __FILE__, __LINE__, #expr) );	\
				sh_assert();																\
		}																				\
	} while(0)

	#define OSD_DASSERT2(expr, comment)	do {											\
		if( !(expr) ) {																	\
			OSD_DPRINT( ("Assert failed at %s(%d): %s [%s]\n", __FILE__, __LINE__, #expr, comment) );	\
				sh_assert();																\
		}																				\
	} while(0)
#else
	#error "What are you going to build?"
#endif

#define OSD_PRINT(_x_)		OSD_DPRINT(_x_)

#if !defined(ASSERT)
	#define ASSERT	OSD_ASSERT
#endif

#if defined(_32BPLATFORM)
	#if defined(_OS_LINUX)	
		#define OSD_BREAKPOINT 	
	#elif defined(_OS_WINDOWS)
		#define OSD_BREAKPOINT	{ __asm int 3 }
	#else
		#define OSD_BREAKPOINT 
	#endif
#else
	#define OSD_BREAKPOINT 
#endif

#if !defined(BIOS)
#define dump_ax(x)
#endif

void
OSD_CrashSystem(
	IN bpVOID ComponentExtension,
	IN U32 Parameter2,
	IN U32 Parameter3,
	IN U32 Parameter4
	);

/* For Debug Version Only */
#if VDEBUG
	#if defined(_SHASTAFW)
	#define OSD_ASSERT2	OSD_DASSERT2
	#endif
	#define OSD_ASSERT	OSD_DASSERT
	#define OSD_TRACE( _x_ )							   \
		do												   \
		{												   \
			OSD_DPRINT( ("%s(%d) ", __FILE__, __LINE__) ); \
			OSD_DPRINT( _x_ );							   \
		} while( 0 )
	#define OSD_DBGBREAKPOINT	OSD_BREAKPOINT
#else
	#if defined(_SHASTAFW)
	#define OSD_ASSERT2(expr,comment)
	#endif
	#define OSD_ASSERT(expr)
	#define OSD_TRACE(_x_)
	#define OSD_DBGBREAKPOINT
#endif

#if !defined(_X8616B)
#define BiosDbgPrint(x)
#endif

#define DUMP_XXX(xxx)	OSD_DPRINT( ("%s=%x\n", #xxx, xxx) )
#define DUMP_SSS(xxx)	OSD_DPRINT( ("%s=%s\n", #xxx, xxx) )

/*
 * OSD_CASSERT provides compile time assertion.
 */
#define OSD_CASSERT(expr)	do {		\
	char _osd_assert[(expr) ? 1 : -1];	\
	(void)_osd_assert;					\
} while(0)

#if defined(_WINDRIVER) && defined(_32BPLATFORM)
#define CHECK_POINTER(p)	OSD_ASSERT( ((PTR_SIZE)p)>0x80000000L )
#else
#define CHECK_POINTER(p)
#endif

#endif /* OSD_DBG_H */
