#if !defined(OSD_EXP_H)
#define OSD_EXP_H

#include "gen_typs.h"

#include "osd_dbg.h"

#if defined(BIOS)
#include "hwapi.h"
#elif !defined(_WINAPP)
#include "osd_hwapi.h"
#endif

#if defined(_OS_LINUX)
	extern void Global_OSD_InitializeExports(void);
#else
	EXTERNC void Global_OSD_InitializeExports();
#endif

#if defined(CHECK_DMA_DIMM_AREA)
void OSD_AddDMADimmArea( IN bpVOID pOsdExtension, IN U32 DimmAddr, IN U32 Length );
BoOL OSD_CheckDMADimmArea( IN bpVOID pOsdExtension, IN U32	DimmAddr, IN U32 Length );
#endif

/*********************************************************************
 * 					Component Management Functions 
 *********************************************************************/
#define COMP_TYPE_OSD		0
#define COMP_TYPE_CAM		1
#define COMP_TYPE_DM		2
#define COMP_TYPE_ENGINE	3
#define COMP_TYPE_PBM		4

#define COMP_TYPE_MAXIMUM		5

/* Definition for OSD_ComponentStarted */
#define COMP_START_STATUS_SUCCESS 1
#define COMP_START_STATUS_FAILURE 0

/*
 * Definition for OSD_ComponentState
 */
#define SYS_STATE_INITIALIZING	0x00
#define SYS_STATE_STARTING		0x01
#define SYS_STATE_GOING_SLEEP	0x02
#define SYS_STATE_STARTED		0x03
#define SYS_STATE_FULLY_STARTED	0x04

/*
 * definition for Engine/PBM_OnBroadcasting 
 */
#define BROADCAST_SYSTEM_FULLY_STARTED	0x01
#define BROADCAST_BATTERY_MALFUNCTIONAL	0x02
#define BROADCAST_BATTERY_FUNCATIONAL	0x03

#if defined(USE_EXPORT)
#define OSD_EXP_API(funcName) EXP_OSD_##funcName
#else
#define OSD_EXP_API(funcName) OSD_##funcName
#endif

#define DECLARE_OSD_EXP(funcName) GEN_EXPORT OSD_EXP_API(funcName)

/*********************************************************************
 * 					OSD Exported Component Functions 
 *********************************************************************/
DIMMADDR
DECLARE_OSD_EXP(GetComponentDIMMStartAddress)(
	bpVOID CallerExtension,
	U16 ComponentType
	);


bpVOID 
DECLARE_OSD_EXP(GetComponentExtension)(
	bpVOID CallerExtension,
	U16 ComponentType
	);


void 
DECLARE_OSD_EXP(ComponentGetExtensionSize)(
	OUT bpMEMORY_REQUIREMENT pMemRequirement 
	);

void
DECLARE_OSD_EXP(ComponentGetDMASafeMemoryRequirement)(
	IN U32 VenDevID,
	IN U32 SubVenDevID,
	OUT bpMEMORY_REQUIREMENT pMemRequirement 
	);

void
DECLARE_OSD_EXP(ComponentGetDIMMRequirement)(
	IN U32 VenDevID,
	IN U32 SubVenDevID,
	OUT bpMEMORY_REQUIREMENT pMemRequirement 
	);
	
void
DECLARE_OSD_EXP(ComponentInitialize)(
	IN bpVOID pOSDExtension,
	IN bpMEMORY_AVAILABLE pMemAvail,
	IN U32 DeviceID,
	IN U32 SubDeviceID,
	IN bpVOID ComponentSpecificContext
	);

void	
DECLARE_OSD_EXP(ComponentStart)(
	IN bpVOID ComponentExtension,
	IN U32 DIMMOffset,
	IN U32 DIMMSize
	);

void
DECLARE_OSD_EXP(ComponentStarted)(
	bpVOID SenderComponentExtension,
	U32 StartStatus
	);

void
DECLARE_OSD_EXP(ComponentStop)(
	IN bpVOID ComponentExtension
	);

void 
DECLARE_OSD_EXP(ComponentRestart)(
	IN bpVOID ComponentExtension
	);

/* Not used yet. */
void
DECLARE_OSD_EXP(ComponentBist)(
	IN U8 controllerStatus
	);

U32
DECLARE_OSD_EXP(ComponentState)(
   IN bpVOID ComponentExtension
   );

/*********************************************************************
 * 				Logical Device and Physical Device Handler
 *********************************************************************/
void
DECLARE_OSD_EXP(OnPhysicalDiskArrival)(
	bpVOID SenderComponentExtension,
	U32 DiskIDMap
	);

void
DECLARE_OSD_EXP(OnPhysicalDiskRemoval)(
	bpVOID SenderComponentExtension,
	U32 DiskIDMap
	);

void
DECLARE_OSD_EXP(OnLogicalDeviceArrival)(
	bpVOID SenderComponentExtension,
	U8 LogicalDeviceID
	);

void
DECLARE_OSD_EXP(OnLogicalDeviceRemoval)(
	bpVOID SenderComponentExtension,
	U8 LogicalDeviceID
	);

#ifdef MOONFANG
void
DECLARE_OSD_EXP(OnLogicalDeviceChanged)(
	bpVOID SenderComponentExtension,
	U8 LogicalDeviceID
	);
#endif

/*********************************************************************
 * 				Timer Interface
 *********************************************************************/
#define OSD_TIMER_F_PERIODICAL 1	// Not used yet

typedef void (GEN_CALLBACK *GEN_TIMER_CALLBACK) (
	IN bpVOID ComponentExtension,	/* request component extension */
	IN bpVOID Timer,				/* same as return value of OSD_RequestTimer */
	IN bpVOID Context				/* the context provided by requester */
);

/* Timer Data Structure */
struct _GEN_TIMER_ENTRY;
typedef struct _GEN_TIMER_ENTRY GEN_TIMER_ENTRY, buf_space* bpGEN_TIMER_ENTRY;

struct _GEN_TIMER_ENTRY
{
	bpGEN_TIMER_ENTRY Next; /* internal link pointer */
	bpVOID ComponentExtension;
	bpVOID Context;
	GEN_TIMER_CALLBACK Callback;
	U32 ControlFlags; /* see OSD_TIMER_F_xxx above */
	U32 TimerInterval; /* number of microsecond, for periodical Timer only */
	U32 Ticks;	/* voliatile, internal used, in microsecond*/
	U32 Reserved;
#if	defined(_OS_WINDOWS) || defined(_FIRMWARE_VER)
#if VDEBUG
	LARGE_INTEGER whenRequestTimer;
#endif /*VDEBUG*/
#endif
};

bpVOID 
DECLARE_OSD_EXP(RequestTimer)(
	IN bpVOID ComponentExtension,
	IN U32 TimerValue,
	IN GEN_TIMER_CALLBACK TimerRoutine,
	IN bpVOID Context,
	IN U32 ControlFlags
	);

void
DECLARE_OSD_EXP(CancelTimer)(
	IN bpVOID ComponentExtension,
	IN bpVOID Timer
	);

/* Time Module interface is using microsecond as a unit. */
#define SECOND_IN_TIMER 1000000

/*********************************************************************
 * 				Other Functions
 *********************************************************************/
BoOL 
DECLARE_OSD_EXP(IsLdrDump)(
	bpVOID callerExtension	/* callerExtension is ignored. */
	);

#if !defined(BIOS)
void 
DECLARE_OSD_EXP(Delay)(
	IN U32 DelayTime
	);
#endif

#if defined(_WINAPP)
void
DECLARE_OSD_EXP(Terminate)(
	IN bpVOID ComponentExtension
	);
#endif

/* System Time */
U32 
DECLARE_OSD_EXP(GetTickCountSinceMidnight)(void);

/* returns seconds from Jan 1, 1970 */
U32 
DECLARE_OSD_EXP(GetCurrentTime)(void);

BoOL
DECLARE_OSD_EXP(EventNotification)( 
	IN bpVOID ComponentExtension,
	IN bpVOID pEngEvent
	);

/* For CAM Initialize */
#if defined(SIMULATED_DIMM)
#define	BASE_ADDRESS_NUM			7
#define BASE_ADDRESS_SIMULATED_DIMM	6
#else	/*SIMULATED_DIMM*/
#define	BASE_ADDRESS_NUM			6
#endif	/*SIMULATED_DIMM*/

typedef struct _CAM_CONTROLLER_CONFIG_INFO
{
	LPVOID BaseAddresses
#if defined(_FIRMWARE_VER)
		[MAX_MARVELL_CHIP_SUPPORT]
#endif
	[BASE_ADDRESS_NUM]; /* mapped addresses */
	/* other necessary information will be added gradually */
#if defined(SIMULATED_DIMM) || defined(_SHASTAFW)
	U32	SimulatedDimmSize;
#endif

#if defined(NAPA_MARVELL)
	U8	Revision;
	U8	bReserved1[3];
#endif

#if defined(_FIRMWARE_VER)
	U32	Vector;
	U8  ChipCnt;
	U8	bReserved2[3];
	
#endif
} CAM_CONTROLLER_CONFIG_INFO, GEN_FAR* PCAM_CONTROLLER_CONFIG_INFO;

#define OSD_ZeroMemory gen_bzero

/*=================================================================================*/
/* Engine Export Function Table Definition */
#if defined(USE_EXPORT)

#define DEFINE_OSD_EXP_FUNC(funcName)	(GEN_EXPORT* OSD_EXP_API(funcName))

typedef struct
{
	bpVOID 
	DEFINE_OSD_EXP_FUNC(GetComponentExtension)(
		bpVOID CallerExtension,
		U16 ComponentType
		);


	void 
	DEFINE_OSD_EXP_FUNC(ComponentGetExtensionSize)(
		OUT bpMEMORY_REQUIREMENT pMemRequirement 
		);

	void
	DEFINE_OSD_EXP_FUNC(ComponentGetDMASafeMemoryRequirement)(
		IN U32 VenDevID,
		IN U32 SubVenDevID,
		OUT bpMEMORY_REQUIREMENT pMemRequirement 
		);

	void
	DEFINE_OSD_EXP_FUNC(ComponentGetDIMMRequirement)(
		IN U32 VenDevID,
		IN U32 SubVenDevID,
		OUT bpMEMORY_REQUIREMENT pMemRequirement 
		);
		
	void
	DEFINE_OSD_EXP_FUNC(ComponentInitialize)(
		IN bpVOID pOSDExtension,
		IN bpMEMORY_AVAILABLE pMemAvail,
		IN U32 DeviceID,
		IN U32 SubDeviceID,
		IN bpVOID ComponentSpecificContext
		);

	void	
	DEFINE_OSD_EXP_FUNC(ComponentStart)(
		IN bpVOID ComponentExtension,
		IN U32 DIMMOffset,
		IN U32 DIMMSize
		);

	void
	DEFINE_OSD_EXP_FUNC(ComponentStarted)(
		bpVOID SenderComponentExtension,
		U32 StartStatus
		);

	void
	DEFINE_OSD_EXP_FUNC(ComponentStop)(
		IN bpVOID ComponentExtension
		);

	void 
	DEFINE_OSD_EXP_FUNC(ComponentRestart)(
		IN bpVOID ComponentExtension
		);

	void
	DEFINE_OSD_EXP_FUNC(ComponentBist)(
		IN U8 controllerStatus
		);

	U32
	DEFINE_OSD_EXP_FUNC(ComponentState)(
	IN bpVOID ComponentExtension
	);

	void
	DEFINE_OSD_EXP_FUNC(OnPhysicalDiskArrival)(
		bpVOID SenderComponentExtension,
		U32 DiskIDMap
		);

	void
	DEFINE_OSD_EXP_FUNC(OnPhysicalDiskRemoval)(
		bpVOID SenderComponentExtension,
		U32 DiskIDMap
		);

	void
	DEFINE_OSD_EXP_FUNC(OnLogicalDeviceArrival)(
		bpVOID SenderComponentExtension,
		U8 LogicalDeviceID
		);

	void
	DEFINE_OSD_EXP_FUNC(OnLogicalDeviceRemoval)(
		bpVOID SenderComponentExtension,
		U8 LogicalDeviceID
		);

#ifdef MOONFANG
	void
	DEFINE_OSD_EXP_FUNC(OnLogicalDeviceChanged)(
		bpVOID SenderComponentExtension,
		U8 LogicalDeviceID
		);
#endif

	bpVOID 
	DEFINE_OSD_EXP_FUNC(RequestTimer)(
		IN bpVOID ComponentExtension,
		IN U32 TimerValue,
		IN GEN_TIMER_CALLBACK TimerRoutine,
		IN bpVOID Context,
		IN U32 ControlFlags
		);

	void
	DEFINE_OSD_EXP_FUNC(CancelTimer)(
		IN bpVOID ComponentExtension,
		IN bpVOID Timer
		);

	BoOL 
	DEFINE_OSD_EXP_FUNC(IsLdrDump)(
		bpVOID callerExtension
		);

	#if !defined(BIOS)
	void 
	DEFINE_OSD_EXP_FUNC(Delay)(
		IN U32 DelayTime
		);
	#endif

#if defined(_WINAPP)
	void
	DEFINE_OSD_EXP_FUNC(Terminate)(
		IN bpVOID ComponentExtension
		);
#endif

	U32 
	DEFINE_OSD_EXP_FUNC(GetTickCountSinceMidnight)(void);

	U32 
	DEFINE_OSD_EXP_FUNC(GetCurrentTime)(void);

	BoOL
	DEFINE_OSD_EXP_FUNC(EventNotification)( 
		IN bpVOID ComponentExtension,
		IN bpVOID pEngEvent
		);
} OSD_EXPORT_FUNC_TABLE, buf_space * bpOSD_EXPORT_FUNC_TABLE;

//extern bpOSD_EXPORT_FUNC_TABLE PSEUDOBASE g_pOsdApi;
#if defined(_OS_LINUX)
	extern bpOSD_EXPORT_FUNC_TABLE  g_pOsdApi;
#else
	EXTERNC bpOSD_EXPORT_FUNC_TABLE  g_pOsdApi;
#endif

#define OSD_GetComponentExtension					g_pOsdApi->OSD_EXP_API(GetComponentExtension)
#define OSD_ComponentGetExtensionSize				g_pOsdApi->OSD_EXP_API(ComponentGetExtensionSize)
#define OSD_ComponentGetDMASafeMemoryRequirement	g_pOsdApi->OSD_EXP_API(ComponentGetDMASafeMemoryRequirement)
#define OSD_ComponentGetDIMMRequirement				g_pOsdApi->OSD_EXP_API(ComponentGetDIMMRequirement)
#define OSD_ComponentInitialize						g_pOsdApi->OSD_EXP_API(ComponentInitialize)
#define OSD_ComponentStart							g_pOsdApi->OSD_EXP_API(ComponentStart)
#define OSD_ComponentStarted						g_pOsdApi->OSD_EXP_API(ComponentStarted)
#define OSD_ComponentStop							g_pOsdApi->OSD_EXP_API(ComponentStop)
#define OSD_ComponentRestart						g_pOsdApi->OSD_EXP_API(ComponentRestart)
#define OSD_ComponentBist							g_pOsdApi->OSD_EXP_API(ComponentBist)
#define OSD_ComponentState							g_pOsdApi->OSD_EXP_API(ComponentState)
#define OSD_OnPhysicalDiskArrival					g_pOsdApi->OSD_EXP_API(OnPhysicalDiskArrival)
#define OSD_OnPhysicalDiskRemoval					g_pOsdApi->OSD_EXP_API(OnPhysicalDiskRemoval)
#define OSD_OnLogicalDeviceArrival					g_pOsdApi->OSD_EXP_API(OnLogicalDeviceArrival)
#define OSD_OnLogicalDeviceRemoval					g_pOsdApi->OSD_EXP_API(OnLogicalDeviceRemoval)
#ifdef MOONFANG
#define OSD_OnLogicalDeviceChanged					g_pOsdApi->OSD_EXP_API(OnLogicalDeviceChanged)
#endif
#define OSD_RequestTimer							g_pOsdApi->OSD_EXP_API(RequestTimer)
#define OSD_CancelTimer								g_pOsdApi->OSD_EXP_API(CancelTimer)
#define OSD_IsLdrDump								g_pOsdApi->OSD_EXP_API(IsLdrDump)
#if !defined(_X8616B)
#define OSD_Delay									g_pOsdApi->OSD_EXP_API(Delay)
#endif
#if defined(_WINAPP)
#define OSD_Terminate								g_pOsdApi->OSD_EXP_API(Terminate)
#endif
#define OSD_GetTickCountSinceMidnight				g_pOsdApi->OSD_EXP_API(GetTickCountSinceMidnight)
#define OSD_GetCurrentTime							g_pOsdApi->OSD_EXP_API(GetCurrentTime)
#define OSD_EventNotification						g_pOsdApi->OSD_EXP_API(EventNotification) 

#endif /*USE_EXPORT*/

/** helper macros for timer. */
#if defined(_X8616B) && defined(USE_EXPORT)

#define OSD_RequestTimerNaked(ComponentExtension, TimerValue, TimerRoutine, Context, ControlFlags) {	\
	GEN_PCALLBACK pCallback;	\
	GEN_MK_CALLBACKF( pCallback, TimerRoutine );	\
	OSD_RequestTimer(ComponentExtension, TimerValue, (GEN_TIMER_CALLBACK)pCallback, Context, ControlFlags);	\
}

#define OSD_RequestTimerLValue(LValue, ComponentExtension, TimerValue, TimerRoutine, Context, ControlFlags) {	\
	GEN_PCALLBACK pCallback;	\
	GEN_MK_CALLBACKF( pCallback, TimerRoutine );	\
	LValue = OSD_RequestTimer(ComponentExtension, TimerValue, (GEN_TIMER_CALLBACK)pCallback, Context, ControlFlags);	\
}

#else

#define OSD_RequestTimerLValue(LValue, ComponentExtension, TimerValue, TimerRoutine, Context, ControlFlags)	\
	LValue = OSD_RequestTimer(ComponentExtension, TimerValue, (GEN_TIMER_CALLBACK)TimerRoutine, Context, ControlFlags)

#define OSD_RequestTimerNaked(ComponentExtension, TimerValue, TimerRoutine, Context, ControlFlags)	\
	OSD_RequestTimer(ComponentExtension, TimerValue, (GEN_TIMER_CALLBACK)TimerRoutine, Context, ControlFlags)

#endif

#endif //OSD_EXP_H
