#ifndef OSD_HARDWARE_API_H
#define OSD_HARDWARE_API_H

/************************************************************************
 *					Hardware Access APIs
 ***********************************************************************/

/* added by isaac to remove os related file including in CAM */
#if !defined(OSD_COMP_H)

#if defined(_OS_LINUX)

extern void wrap_memcpyfromio(void* mmiobase, const void* buffer, unsigned long count);
extern void wrap_memcpytoio(void* mmiobase, const void* buffer, unsigned long count);

extern U32 wrap_readl(void * mmiobase);
extern U16 wrap_readw(void * mmiobase);
extern U8 wrap_readb(void * mmiobase);

extern void wrap_writel(void* mmiobase, U32 data);
extern void wrap_writew(void* mmiobase, U16 data);
extern void wrap_writeb(void* mmiobase, U8 data);

extern unsigned char wrap_inb(int addr);
extern unsigned short wrap_inw(int addr);
extern unsigned int wrap_inl(int addr);

extern void wrap_outb( int addr, unsigned char b);
extern void wrap_outw( int addr, unsigned short b);
extern void wrap_outl(int addr, unsigned int b);
//#define LINUX_memcpy(x,y,z); memcpy(x,y,z)



#define ScsiPortReadRegisterBufferUlong(x,y,z) wrap_memcpyfromio(x,y,z*4)
#define ScsiPortReadRegisterBufferUchar(x,y,z) wrap_memcpyfromio(x,y,z)


#define ScsiPortReadPortUlong(x) wrap_inl(x)
#define	ScsiPortReadPortUshort(x) wrap_inw(x)
#define	ScsiPortReadPortUchar(x) wrap_inb(x) 

#define ScsiPortReadPortBufferUlong(x,y,z) {U32 i; for(i=0;i<z;i++){ *(U32 *)y=wrap_inl(x); y++;}}
#define ScsiPortReadPortBufferUshort(x,y,z) {U32 i; for(i=0;i<z;i++){ *(U16 *)y=wrap_inw(x); y++;}}
#define ScsiPortReadPortBufferUchar(x,y,z) {U32 i; for(i=0;i<z;i++){ *(U8 *)y=wrap_inb(x); y++;}}


#define ScsiPortReadRegisterUlong(x) wrap_readl(x)
#define ScsiPortReadRegisterUshort(x) wrap_readw(x)
#define ScsiPortReadRegisterUchar(x)  wrap_readb(x)


#define	ScsiPortWriteRegisterBufferUlong(x,y,z) wrap_memcpytoio(x,y,z*4)
#define	ScsiPortWriteRegisterBufferUchar(x,y,z) wrap_memcpytoio(x,y,z);

#define ScsiPortWritePortUlong(x,y) wrap_outl(x,y)
#define	ScsiPortWritePortUshort(x,y) wrap_outw(x,y)
#define	ScsiPortWritePortUchar(x,y) wrap_outb(x,y)

#define ScsiPortWritePortBufferUlong(x,y,z)  {U32 i; for(i=0;i<z;i++){ wrap_outl(x,y);y++;}}
#define ScsiPortWritePortBufferUshort(x,y,z)  {U32 i; for(i=0;i<z;i++){ wrap_outw(x,y);y++;}}
#define ScsiPortWritePortBufferUchar(x,y,z)  {U32 i; for(i=0;i<z;i++){ wrap_outb(x,y);y++;}}


#define ScsiPortWriteRegisterUlong(x,y)   wrap_writel(x,y)
#define ScsiPortWriteRegisterUshort(x,y)  wrap_writew(x,y)
#define ScsiPortWriteRegisterUchar(x,y)  wrap_writeb(x,y)

//extern void memcpy(unsigned char *x, unsigned char  *y, unsigned long z);
//extern void memset(unsigned char *x,unsigned long y,unsigned long z);
#elif defined(_SHASTAFW)

#include "shasta_hwapi.h"

#elif defined(_WINDRIVER)

#if defined(__cplusplus)
EXTERNC { 
#endif
#ifndef USE_STORPORT
	#include <ntddk.h>
	#include <srb.h>	//TBD
#else
	#include <storport.h>
#endif
#if defined(__cplusplus)
}
#endif
#include <basetsd.h>

#else

	#error "Please defined HWAPI for your platform!"

#endif	/* _OS_WINDOWS */

#endif	/* OSD_COMP_H */
/* end */

/* For Scsi Miniport Driver. */
/* Port IO */
#define OSD_IoReadU8(BaseAddr)\
	ScsiPortReadPortUchar((PU8)(BaseAddr))

#define OSD_IoReadU16(BaseAddr)\
	ScsiPortReadPortUshort((PU16)(BaseAddr))

#define OSD_IoReadU32(BaseAddr)\
	ScsiPortReadPortUlong((PU32)(BaseAddr))

#define OSD_IoReadBufferU8(BaseAddr, Buffer, Count)\
	ScsiPortReadPortBufferUchar((PU8)(BaseAddr), (PU8)(Buffer), (U32)(Count))

#define OSD_IoReadBufferU16(BaseAddr, Buffer, Count)\
	ScsiPortReadPortBufferUshort((PU16)(BaseAddr), (PU16)(Buffer), (U32)(Count))

#define OSD_IoReadBufferU32(BaseAddr, Buffer, Count)\
	ScsiPortReadPortBufferUlong((PU32)(BaseAddr), (PU32)(Buffer), (U32)(Count))

#define OSD_IoWriteU8(BaseAddr, Value)\
	ScsiPortWritePortUchar((PU8)(BaseAddr), (U8)(Value))

#define OSD_IoWriteU16(BaseAddr, Value)\
	ScsiPortWritePortUshort((PU16)(BaseAddr), (U16)(Value))

#define OSD_IoWriteU32(BaseAddr, Value)\
	ScsiPortWritePortUlong((PU32)(BaseAddr), (U32)(Value))

#define OSD_IoWriteBufferU8(BaseAddr, Buffer, Count)\
	ScsiPortWritePortBufferUchar((PU8)(BaseAddr), (PU8)(Buffer), (U32)(Count))

#define OSD_IoWriteBufferU16(BaseAddr, Buffer, Count)\
	ScsiPortWritePortBufferUshort((PU16)(BaseAddr), (PU16)(Buffer), (U32)(Count))

#define OSD_IoWriteBufferU32(BaseAddr, Buffer, Count)\
	ScsiPortWritePortBufferUlong((PU32)(BaseAddr), (PU32)(Buffer), (U32)(Count))

/* Memory Mapped IO */
#define OSD_MmReadU8(BaseAddr)\
	ScsiPortReadRegisterUchar((PU8)(BaseAddr))

#define OSD_MmReadU16(BaseAddr)\
	ScsiPortReadRegisterUshort((PU16)(BaseAddr))

#define OSD_MmReadU32(BaseAddr)\
	ScsiPortReadRegisterUlong((PU32)(BaseAddr))

#define OSD_MmReadBufferU8(BaseAddr, Buffer, Count)\
	ScsiPortReadRegisterBufferUchar((PU8)(BaseAddr), (PU8)(Buffer), (U32)(Count))

#define OSD_MmReadBufferU16(BaseAddr, Buffer, Count)\
	ScsiPortReadRegisterBufferUshort((PU16)(BaseAddr), (PU16)(Buffer), (U32)(Count))

#define OSD_MmReadBufferU32(BaseAddr, Buffer, Count)\
	ScsiPortReadRegisterBufferUlong((PU32)(BaseAddr), (PU32)(Buffer), (U32)(Count))

#define OSD_MmWriteU8(BaseAddr, Value)\
	ScsiPortWriteRegisterUchar((PU8)(BaseAddr), (U8)(Value))

#define OSD_MmWriteU16(BaseAddr, Value)\
	ScsiPortWriteRegisterUshort((PU16)(BaseAddr), (U16)(Value))

#define OSD_MmWriteU32(BaseAddr, Value)\
	ScsiPortWriteRegisterUlong((PU32)(BaseAddr), (U32)(Value))

#define OSD_MmWriteBufferU8(BaseAddr, Buffer, Count)\
	ScsiPortWriteRegisterBufferUchar((PU8)(BaseAddr), (PU8)(Buffer), (U32)(Count))

#define OSD_MmWriteBufferU16(BaseAddr, Buffer, Count)\
	ScsiPortWriteRegisterBufferUshort((PU16)(BaseAddr), (PU16)(Buffer), (U32)(Count))

#define OSD_MmWriteBufferU32(BaseAddr, Buffer, Count)\
	ScsiPortWriteRegisterBufferUlong((PU32)(BaseAddr), (PU32)(Buffer), (U32)(Count))

#ifdef _MMMAP_	
	/*  Memory Mapped IO */
	#define OSD_HwReadU8			OSD_MmReadU8
	#define OSD_HwReadU16			OSD_MmReadU16
	#define OSD_HwReadU32			OSD_MmReadU32
	#define OSD_HwReadBufferU8		OSD_MmReadBufferU8
	#define OSD_HwReadBufferU16		OSD_MmReadBufferU16
	#define OSD_HwReadBufferU32		OSD_MmReadBufferU32
	#define OSD_HwWriteU8			OSD_MmWriteU8
	#define OSD_HwWriteU16			OSD_MmWriteU16
	#define OSD_HwWriteU32			OSD_MmWriteU32
	#define OSD_HwWriteBufferU8		OSD_MmWriteBufferU8
	#define OSD_HwWriteBufferU16	OSD_MmWriteBufferU16
	#define OSD_HwWriteBufferU32	OSD_MmWriteBufferU32
#else
	/*  Port IO */
	#define OSD_HwReadU8			OSD_IoReadU8
	#define OSD_HwReadU16			OSD_IoReadU16
	#define OSD_HwReadU32			OSD_IoReadU32
	#define OSD_HwReadBufferU8		OSD_IoReadBufferU8
	#define OSD_HwReadBufferU16		OSD_IoReadBufferU16
	#define OSD_HwReadBufferU32		OSD_IoReadBufferU32
	#define OSD_HwWriteU8			OSD_IoWriteU8
	#define OSD_HwWriteU16			OSD_IoWriteU16
	#define OSD_HwWriteU32			OSD_IoWriteU32
	#define OSD_HwWriteBufferU8		OSD_IoWriteBufferU8
	#define OSD_HwWriteBufferU16	OSD_IoWriteBufferU16
	#define OSD_HwWriteBufferU32	OSD_IoWriteBufferU32
#endif

#endif /* OSD_HARDWARE_API_H */
