#if !defined( OSD_SCSI_H )
#define OSD_SCSI_H

#include "gen_typs.h"

#define ocmdEnclosure	0x0
#define ocmdBattery		0x1
#define ocmdGetAdapter	0x5
#define ocmdFlash		0x6
#define ocmdGetIOP		0x7
#define ocmdMemory		0x8

/* For ocmdEnclosure: Enclosure Control */
#if defined(SHASTA_CONTROLLER)
	#define CAM_MAX_ENCLOSURE					0x10
#else
	#if defined(NAPA_ASIC) || defined(_WINAPP)
		#define CAM_MAX_ENCLOSURE				0x08	/* NAPA is 8 */
	#else
		#define CAM_MAX_ENCLOSURE				0x04	/* SX4 is 4 */
	#endif
#endif

#define CAM_MAX_SENSOR_PER_ENCLOSURE	0x06	/* SS4100 has 6 sensors */
#define BASE_ADDRESS_MAX_NUM	6

#define DIMM_TYPE_NORMAL	0
#define DIMM_TYPE_ECC		1
#define DIMM_TYPE_DDR		2
#define DIMM_TYPE_SIMULATED	3 

/* 
 * To avoid inconsistent enum type size on different platforms, we decide to cancel all
 * enum definitions in driver and application shared definitions.
 */
#define ENCLOSURE_NONE		0
#define ENCLOSURE_SS4100	1
#define ENCLOSURE_SS1100	2
#define ENCLOSURE_NEC		3
#define ENCLOSURE_FSC		4
#define ENCLOSURE_HPQ		5
#define ENCLOSURE_SAFTE		6

#define SENSOR_NONE			0
#define SENSOR_FAN			1
#define SENSOR_VOLTAGE33	2
#define SENSOR_VOLTAGE5		3
#define SENSOR_VOLTAGE12	4
#define SENSOR_TEMPERATURE	5

typedef struct _APIFRAME_ENCLOSURE
{
	U8 EnclosureIndex;
	U8 EnclosureType;
	U8 NumberOfFan;
	U8 NumberOfTemperature;

	U8 NumberOfVoltage;
	U8 NumberOfCarrier;
	U8 Reserved[2];
} APIFRAME_ENCLOSURE, * PAPIFRAME_ENCLOSURE;

typedef struct _APIFRAME_ENCLOSURE_INFORMATION {
	APIFRAME_ENCLOSURE Enclosure[CAM_MAX_ENCLOSURE];
} APIFRAME_ENCLOSURE_INFORMATION, *PAPIFRAME_ENCLOSURE_INFORMATION;

typedef struct _APIFRAME_ENCLOSURE_MAPDISKID {
	U8 EnclosureID;
	U8 CarrierID;
	U8 Reserved[6];
} APIFRAME_ENCLOSURE_MAPDISKID, * PAPIFRAME_ENCLOSURE_MAPDISKID;

/* For ocmdBattery: Battery Information */
typedef struct _APIFRAME_BATTERY_INFORMATION {
	U8 BatteryExist;
	U8 BatteryState; 
	U8 Reserved[6];
	U32 Temperature;
	U32 Voltage;
} APIFRAME_BATTERY_INFORMATION, *PAPIFRAME_BATTERY_INFORMATION;

#define BATTERY_STATE_CHARGING                0x00
#define BATTERY_STATE_DISCHARGING             0x01
#define BATTERY_STATE_FULLY_CHARGED           0x02
#define BATTERY_STATE_MALFUNCTION             0x03
#define BATTERY_STATE_RECONDITION             0x04

/****************************************************************
 * Flash Control: can support following functions				*
 * 1. Upload Image												*
 * 2. Download Image											*
 * 2. Get Flash Type, Size										*
 ****************************************************************/
/* SDRAM [126M,127M) is used for Flash. [127M,128M) is used by Redboot. */
#define FLASH_IMAGE_PAGE_SIZE	1024*64
#define FLASH_IMAGE_PAGE_COUNT	10
#define FLASH_IMAGE_MAX_LENGTH	(FLASH_IMAGE_PAGE_SIZE*FLASH_IMAGE_PAGE_COUNT)

#define FLASH_IMAGE_TYPE_BIOS		0
#define FLASH_IMAGE_TYPE_FIRMWARE	1
#define FLASH_IMAGE_TYPE_REDBOOT	2
#define FLASH_IMAGE_TYPE_FCONFIG	3
#define FLASH_IMAGE_TYPE_FIS		4
#define FLASH_IMAGE_TYPE_TRACE		5
#define FLASH_IMAGE_TYPE_MAX		6

#define FLASH_CONTROL_SHOWINFORMATION	0
#define FLASH_CONTROL_UPLOADIMAGE		1
#define FLASH_CONTROL_DOWNLOADIMAGE		2
#define FLASH_CONTROL_SETUPUPLOAD		3

#define FLASH_FLAG_LAST_PAGE		bit(0)

/* Image Header has 64 bytes. Late on, we may need add checksum. */
typedef struct _APIFRAME_FLASH_IMAGE_HEADER
{
	U32 ImageSize;
	U32 Reserved[15];
}APIFRAME_FLASH_IMAGE_HEADER, *PAPIFRAME_FLASH_IMAGE_HEADER;

/* Only the first page has the imageheader.
 * For other pages, ImageHeader is used as the ImageBuffer as well. 
 * So it will contain the image data. */
typedef struct _APIFRAME_FLASH_PAGE
{
	APIFRAME_FLASH_IMAGE_HEADER ImageHeader;
	U8 Image[FLASH_IMAGE_PAGE_SIZE];
}APIFRAME_FLASH_PAGE, *PAPIFRAME_FLASH_PAGE;

typedef struct	_CDB_FLASH
{
	U8	OpCode;						/* FIXED: svucPASSTHROUGH_API_COMMAND */
	U8	SubOpCode;					/* FIXED: ocmdFlash */
	U8	FlashCommand;				/* see FLASH_CONTROL_XXX */
	U8	ImageType;					/* see FLASH_IMAGE_TYPE_XXX*/
	U32 PageSize;					/* Size of this page */
	U8  PageNumber;					/* Page Number */
	U8  PageFlag;					/* Page Flag: see FLASH_FLAG_XXX */
	U8	Reserved[6];
} PACKED CDB_FLASH, buf_space * bpCDB_FLASH;


/****************************************************************
 * Memory Control: Dump the memory								*
 ****************************************************************/
#define MEMORY_MAX_LENGTH	1024*1024*128
#define MEMORY_PAGE_SIZE	FLASH_IMAGE_PAGE_SIZE

#define MEMORY_CONTROL_DUMP		1

typedef struct _APIFRAME_MEMORY_PAGE
{
	U8 Buffer[MEMORY_PAGE_SIZE];
}APIFRAME_MEMORY_PAGE, *PAPIFRAME_MEMORY_PAGE;

typedef struct	_CDB_MEMORY
{
	U8	OpCode;						/* FIXED: svucPASSTHROUGH_API_COMMAND */
	U8	SubOpCode;					/* FIXED: ocmdMemory */
	U8	Command;					/* see DIMM_CONTROL_XXX */
	U8  PageNumber;					/* Page Number */
	U8	Reserved[12];
} PACKED CDB_MEMORY, buf_space * bpCDB_MEMORY;

/****************************************************************
 *						Trace Buffer							*
 ****************************************************************/
#ifdef STORE_TRACE
#define TRACE_FLAG_VALID	bit(0)		/* The trace buffer is valide. */

#define TRACE_MAX_ENTRY		500			/* How many trace entries it can hold. */
#define TRACE_ENTRY_LENGTH	256			/* For each trace(string), how long this trace can be. */

#define TRACE_IMAGE_SIZE	0x40000		/* Totally we can use up to 256K */
typedef struct _TRACE_BUFFER
{
	/* Trace Buffer Header: 64 bytes */
	U32 head;			/* Which entry: Head is the one has the latest trace. */
	U32 count;			/* How many valid entries we have? */
	U32 flag;			/* Is this trace valid? */
	U32 reserved1;
	U32 timestamp;		/* The time we flash the trace. */
	U32 reserved2[11];

	/* Trace Buffer */
	U8 trace[TRACE_MAX_ENTRY][TRACE_ENTRY_LENGTH];
} TRACE_BUFFER, * PTRACE_BUFFER;
#endif /* STORE_TRACE */

#endif /* OSD_SCSI_H */
