/***************************************************************************
 *
 * Copyright (c) 1995 Promise Technology, Inc.  All rights reserved.
 * No part of this document may be reproduced or transmitted in any form or
 * by any means, electronic or mechanical, for any purpose, without the
 * express written permission of Promise Technology, Inc.
 *
 ***************************************************************************/

/***************************************************************************
 *
 * Author		:Jing Qian
 *
 * Module		:OSD Component Interface
 * Revision		:1.0.0.1
 * Checkin Date	:01/01/2004
 *
 * Description	:OSD Component Interface Implementation.
 *
 * History		:
 *
 ***************************************************************************/
 
#include "osd_inc.h"
void init_data_pool(bpVOID OSDExtension);

bpVOID DECLARE_OSD_EXP(GetComponentExtension)(
	LPVOID CallerExtension,
	U16 ComponentType
	)	
{
	bpOSD_COMP_EXTENSION pOSDExtension;
	
	pOSDExtension=(bpOSD_COMP_EXTENSION)OSDI_GetOSDExtension(CallerExtension);

	OSD_ASSERT(ComponentType<COMP_TYPE_MAXIMUM);
	return pOSDExtension->ComponentResource[ComponentType].ComponentExtension;
}

void DECLARE_OSD_EXP(ComponentGetExtensionSize)(
	OUT bpMEMORY_REQUIREMENT pMemRequirement 
	)
{
	ULONG constantSize = FIELD_OFFSET(OSD_COMP_EXTENSION, TimerPoolBuffer);
	constantSize += ( (MAX_DATA_BUFFER_SIZE) + SPT_SENSE_LENGTH)*(MAX_DATA_BUFFER_COUNT); 
	//U8 linux_GetTimerEntryNumber(BoOL IsHibernation); 
	pMemRequirement->MaxRequirement = constantSize ;
	pMemRequirement->MinRequirement = constantSize ;
	OSD_TRACE( ("OSD component: MaxReqMem=0x%x, MinReqMem=0x%x\n", pMemRequirement->MaxRequirement, pMemRequirement->MinRequirement) ); 
}

void
DECLARE_OSD_EXP(ComponentGetDMASafeMemoryRequirement)(
	IN U32 DeviceID,
	IN U32 SubDeviceID,
	OUT bpMEMORY_REQUIREMENT pMemRequirement 
	)
{
	pMemRequirement->MaxRequirement = 0;	
	pMemRequirement->MinRequirement = 0;
}

void
DECLARE_OSD_EXP(ComponentGetDIMMRequirement)(
	IN U32 DeviceID,
	IN U32 SubDeviceID,
	OUT bpMEMORY_REQUIREMENT pMemRequirement 
	)
{
	pMemRequirement->MaxRequirement = RESERVED_DIMM_SIZE;	
	pMemRequirement->MinRequirement = RESERVED_DIMM_SIZE;
}

void
DECLARE_OSD_EXP(ComponentInitialize)(
	IN LPVOID OsdExt,
	IN bpMEMORY_AVAILABLE pMemAvail,
	IN U32 DeviceID,
	IN U32 SubDeviceID,
	IN LPVOID ComponentSpecificContext
)
{
	bpOSD_COMP_EXTENSION pOSDExtension = (bpOSD_COMP_EXTENSION) OsdExt;
	bpOSD_COMP_EXTENSION p = pOSDExtension;
	
	
	//init_waitqueue_head(&(pOSDExtension->WaitQ) );
	init_waitqueue_head(&(pOSDExtension->Init_Waitq) );
	RES_LOCK_INIT
	spin_lock_init(&(pOSDExtension->datapool_lock) ); 
	spin_lock_init(&(pOSDExtension->timer_lock) );
	
	init_req_pool(pOSDExtension);

#if defined(OSD_CHECK_REQ_TIMEOUT)
	GenList_Initialize( &pOSDExtension->CurrentReqList, OFFSET_OF(REQ, Osd.prev) );
#endif
	
	init_timer_queue( (bpVOID)pOSDExtension);

	init_data_pool( (bpVOID)pOSDExtension);
	
}

void
DECLARE_OSD_EXP(ComponentStop)(
	IN bpVOID OSDExt
	)
{
	bpOSD_COMP_EXTENSION pOSDExtension = (bpOSD_COMP_EXTENSION)OSDExt;

	/* Reset the OSD Data Structure. */
	OSDI_ClearComponentsReadyFlag(pOSDExtension);

	/* Stop the Timer */
//	linux_StopOSTimer(pOSDExtension);
//	linux_InitializeTimerQueue(
//		&pOSDExtension->TimerModule, 
//		&pOSDExtension->TimerPoolBuffer, 
//		0);
}

void
DECLARE_OSD_EXP(ComponentStart)(
	IN bpVOID ComponentExtension,
	IN U32 DIMMOffset,
	IN U32 DIMMSize
	)
{
	/*
	 * Timer Module has been initialized already in OSD_ComponentInitialize and is working now.
	 */

	OSD_ComponentStarted(ComponentExtension, COMP_START_STATUS_SUCCESS);
}

void 
DECLARE_OSD_EXP(ComponentRestart)(
	IN bpVOID ComponentExtension
	)
{
#if 0
	/* Component should check whether it's ready for Timer or not based on its status. 
	 * I have to call it here because CAM use Timer to Start Component.
	 */
	OSDI_RestartTimer(&pOSDExtension->TimerModule, pOSDExtension->DeviceExtension);
#endif /* 0 */
	
	/* Nothing else needs done. */
	OSD_ComponentStarted(ComponentExtension, COMP_START_STATUS_SUCCESS);
}


void
DECLARE_OSD_EXP(OnPhysicalDiskArrival)(
	LPVOID SenderComponentExtension,
	U32 DiskIDMap
	)
{
	LPVOID pEngineExtension = 
		OSD_GetComponentExtension(SenderComponentExtension, COMP_TYPE_ENGINE);

	Engine_OnPhysicalDiskArrival(pEngineExtension, DiskIDMap);
}

void
DECLARE_OSD_EXP(OnPhysicalDiskRemoval)(
	LPVOID SenderComponentExtension,
	U32 DiskIDMap
	)
{
	LPVOID pEngineExtension = 
		OSD_GetComponentExtension(SenderComponentExtension, COMP_TYPE_ENGINE);

	Engine_OnPhysicalDiskRemoval(pEngineExtension, DiskIDMap);
}

void
OSDI_DriverOnArrayChanged(
	bpOSD_COMP_EXTENSION pOSDExtension
	)
{
	//ScsiPortNotification(BusChangeDetected,pDeviceExtension,0);

/* removed scsi_scan host, it will result schedule while atomic when call it in interrupt context
    
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0)
	scsi_scan_host(pOSDExtension->host);
#endif

*/
	
}

void
DECLARE_OSD_EXP(OnLogicalDeviceArrival)(
	LPVOID SenderComponentExtension,
	U8 LogicalDeviceID
	)
{
	U16 ComponentType = OSDI_GetComponentType(SenderComponentExtension);

	if ( ComponentType==COMP_TYPE_ENGINE )
	{
#if !defined(DISABLE_PBM)
		bpVOID pPBMExtension = 
			OSD_GetComponentExtension(SenderComponentExtension, COMP_TYPE_PBM);
		if( pPBMExtension )
		{
			PBM_OnLogicalDeviceArrival(pPBMExtension, LogicalDeviceID);
			return;
		}
#endif		
	}	

	{
		/* All components are ready for the OS request to the new logical disk. */
		bpVOID pOSDExtension = 
			OSD_GetComponentExtension(SenderComponentExtension, COMP_TYPE_OSD);
		OSDI_DriverOnArrayChanged((bpOSD_COMP_EXTENSION)pOSDExtension);
	}		

}


void
DECLARE_OSD_EXP(OnLogicalDeviceRemoval)(
	LPVOID SenderComponentExtension,
	U8 LogicalDeviceID
	)
{
	U16 ComponentType=OSDI_GetComponentType(SenderComponentExtension);

	if ( ComponentType==COMP_TYPE_ENGINE )
	{
		bpVOID pPBMExtension = 
			OSD_GetComponentExtension(SenderComponentExtension, COMP_TYPE_PBM);
		bpVOID pOSDExtension = 
			OSD_GetComponentExtension(SenderComponentExtension, COMP_TYPE_OSD);

		OSDI_DriverOnArrayChanged((bpOSD_COMP_EXTENSION)pOSDExtension);

#if !defined(DISABLE_PBM)
		if( pPBMExtension )
			PBM_OnLogicalDeviceRemoval(pPBMExtension, LogicalDeviceID);
#endif
		return;
	}	

	if ( ComponentType==COMP_TYPE_PBM )
	{
		/* All components are ready to remove the logical disk. */
		return;
	}		
}



/* Other functions */

#if VDEBUG 
extern int gFlag;
#endif

void Timer_callback(unsigned long  dev)
{
	Timer_Queue_t * ptimer=(Timer_Queue_t *)dev;
	bpOSD_COMP_EXTENSION pOSDExtension = (bpOSD_COMP_EXTENSION)
		OSD_GetComponentExtension(ptimer->pdata, COMP_TYPE_OSD);
#if VDEBUG	
	LPVOID	pPBMExtension = OSD_GetComponentExtension(pOSDExtension, COMP_TYPE_PBM);	
#endif
	bpOSD_COMP_EXTENSION p = pOSDExtension;
	
	GEN_TIMER_CALLBACK fun;
	unsigned long smpflag;
	
	fun=(GEN_TIMER_CALLBACK)(ptimer->callback);

	//OSD_TRACE( ("Before call timer routine \n") );	
	
	DRIVER_LOCK(p->host,smpflag);
#if VDEBUG		
	if( (gFlag) && (ptimer->pdata == pPBMExtension) )
		OSD_DPRINT( ("Oh, My God, we will call PBM timer handler in queue command routine \n") );	
		
#endif
	fun(ptimer->pdata,ptimer,ptimer->pContext);

	
	DRIVER_UNLOCK(p->host,smpflag);
	//OSD_TRACE( ("After call timer routine") );
	
	release_timer_queue( (bpVOID)pOSDExtension, ptimer);	
	return;
}

bpVOID 
DECLARE_OSD_EXP(RequestTimer)(
	IN bpVOID ComponentExtension,
	IN U32 TimerValue,
	IN GEN_TIMER_CALLBACK TimerRoutine,
	IN bpVOID Context,
	IN U32 ControlFlags
	)
{
	bpOSD_COMP_EXTENSION pOSDExtension = (bpOSD_COMP_EXTENSION)
		OSD_GetComponentExtension(ComponentExtension, COMP_TYPE_OSD);
	struct Timer_Queue * ptimer;
	
	ptimer=alloc_timer_queue( (bpVOID)pOSDExtension);
	ptimer->callback=TimerRoutine;
	ptimer->pdata=ComponentExtension;
	ptimer->pContext=Context;
	ptimer->ControlFlags=ControlFlags;
   	init_timer(ptimer->list);
	
   	//ptimer->list->expires = linux_get_jiffies()+TimerValue/(SECOND_IN_TIMER/HZ);	
	
	//wrap_print("linux_get_jiffies = %d\n", linux_get_jiffies());
        //wrap_print("TimerValue/(SECOND_IN_TIMER/HZ) = %d\n", TimerValue/(SECOND_IN_TIMER/HZ));
        //wrap_print("Hz = %d\n", HZ);

        ptimer->list->expires = linux_get_jiffies()+TimerValue/(SECOND_IN_TIMER/HZ);

        //wrap_print("ptimer->list->expires = %d\n", ptimer->list->expires);		

	/*linux_timer_list will cause 64bit pointer cast issue, I will fixed it later */
   	ptimer->list->data = (unsigned long)ptimer ;
   	ptimer->list->function = Timer_callback;
   	add_timer(ptimer->list);
	return ptimer;	
}


void
DECLARE_OSD_EXP(CancelTimer)(
	IN bpVOID ComponentExtension,
	IN bpVOID TimerEntry
	)
{
	bpOSD_COMP_EXTENSION pOSDExtension = (bpOSD_COMP_EXTENSION)
		OSD_GetComponentExtension(ComponentExtension, COMP_TYPE_OSD);
	
	release_timer_queue( (bpVOID)pOSDExtension, (struct Timer_Queue *)TimerEntry);

}


void 
DECLARE_OSD_EXP(Delay)(
	IN U32 DelayTime
	)
{
	linux_delay(DelayTime);

}

U16 DECLARE_OSD_EXP(GetDaysSince1970)()
{
	struct timeval tv;
	U32 SecondsInDay = 86400; /* 24*60*60  */
	U16 day;

	do_gettimeofday(&tv);
	day = (U16)(cpu_to_le32(tv.tv_sec) / SecondsInDay);

	return day; 
}

/* It's in milliseconds. 
 * Since it's U32, I cannot return microsecond at all.
 */
U32 DECLARE_OSD_EXP(GetTickCountSinceMidnight)()
{
	struct timeval tv;
	U32 MilliSecondsInDay = 86400000; /* 24*60*60*1000 */
	U32 ms;

	do_gettimeofday(&tv);
	ms = cpu_to_le32(tv.tv_sec);
	ms += cpu_to_le32(tv.tv_usec) / 1000;
  	
	ms %= MilliSecondsInDay;
	return ms;
}

/* Seconds elapsed since midnight, Jan 1, 1970 */
U32
DECLARE_OSD_EXP(GetCurrentTime)()
{
	struct timeval tv;
	U32 sec;

	do_gettimeofday(&tv);

	sec = cpu_to_le32(tv.tv_sec);

	return sec;
}

BoOL DECLARE_OSD_EXP(IsLdrDump)(
	bpVOID callerExtension	/* callerExtension is ignored. */
	)
{
	return 0;
}
/*
void
OSD_ZeroMemory(
    IN unsigned char * Buffer,
    IN U32 Count
    )
{
    U32 i;

    for (i = 0; i < Count; i++) {
        Buffer[i] = 0;
    }
}
*/

void
OSD_CrashSystem(
	IN LPVOID ComponentExtension,
	IN U32 Parameter2,
	IN U32 Parameter3,
	IN U32 Parameter4
	)
{
}

BoOL DECLARE_OSD_EXP(EventNotification)(  
	IN LPVOID ComponentExtension,
	IN LPVOID pEngEvent /* struct logevent * */
	)
{
	bpOSD_COMP_EXTENSION pOSDExtension = (bpOSD_COMP_EXTENSION)
			OSD_GetComponentExtension(ComponentExtension, COMP_TYPE_OSD);

#ifdef _WMI_
	OSD_WMI_FireEvent(
		pOSDExtension,
		pEngEvent
	);

#endif /* _WMI_ */

	if(pOSDExtension->async_queue)
		kill_fasync(&pOSDExtension->async_queue, SIGIO, POLL_OUT);
	
	/* Always return false, so Engine will not remove the event. 
	 * Engine will only remove the event when API gets the event from the SCSI PASSTHROUGH command.
	 */
	return FALSE;
}

U32
DECLARE_OSD_EXP(ComponentState)(
   IN bpVOID ComponentExtension
   )
{
	bpOSD_COMP_EXTENSION pOSDExtension = (bpOSD_COMP_EXTENSION)
		OSD_GetComponentExtension(ComponentExtension, COMP_TYPE_OSD);

	return pOSDExtension->State;
}

void
DECLARE_OSD_EXP(ComponentBist)(
	IN U8 controllerStatus
	)
{
}

#if defined(CHECK_DMA_DIMM_AREA)

void 
OSD_AddDMADimmArea( 
	bpVOID pOsdExtension, 
	U32 DimmAddr, 
	U32 Length )
{
	bpOSD_COMP_EXTENSION This = (bpOSD_COMP_EXTENSION) pOsdExtension;

	if( This->DMADimmCount>=MAX_DMADIMM_AREA )
	{
		OSD_TRACE( ("Maximum DMA DIMM area %d exceeded!\n", MAX_DMADIMM_AREA) );
	}
	else
	{
		OSD_TRACE( ("DMA DIMM area %x(%x) added.\n", DimmAddr, Length) );

		This->DMADimmArea[This->DMADimmCount].DimmAddr = DimmAddr;
		This->DMADimmArea[This->DMADimmCount].Length = Length;
		This->DMADimmCount++;
	}
}

BoOL OSD_CheckDMADimmArea(
	bpVOID pOsdExtension, 
	IN U32	DimmAddr,
	IN U32	Length )
{
	bpOSD_COMP_EXTENSION This = (bpOSD_COMP_EXTENSION) pOsdExtension;
	U32 i;

	if( This->State<SYS_STATE_STARTED )
		return FALSE;

	for( i=0; i<This->DMADimmCount; i++ )
	{
		if( This->DMADimmArea[i].DimmAddr < DimmAddr+Length
			&& This->DMADimmArea[i].DimmAddr + This->DMADimmArea[i].Length > DimmAddr )
		{
			return TRUE;
		}
	}

	return FALSE;
}


#endif

//#ifdef AHCI
//#ifdef T3
// dummy func. , unneeded in linux driver : see cam/ahci_pack.c and remove it later
PHYADDR
OSD_GetPhysicalAddress(
	IN LPVOID ComponentExtension,
	IN LPVOID VirtualAddress,
	IN LPVOID TranslateContext,
	OUT U32* length )
{
	//bpOSD_COMP_EXTENSION pOSDExtension = (bpOSD_COMP_EXTENSION)
        //OSD_GetComponentExtension(ComponentExtension, COMP_TYPE_OSD);
	//bpVOID pDeviceExtension = pOSDExtension->DeviceExtension;
	PHYADDR phyAddr;
        //printk
	//SCSI_PHYSICAL_ADDRESS scsiPhyAddr = ScsiPortGetPhysicalAddress(
//						pDeviceExtension,
	//					(PSCSI_REQUEST_BLOCK)TranslateContext, /* Either SRB or NULL */
        //				VirtualAddress,
        //				length );
        OSD_DPRINT(("ERROR : GetPhysicalAddress Called!\n"));
        phyAddr.u.Low = 0;//scsiPhyAddr.LowPart;
        phyAddr.u.High = 0;//scsiPhyAddr.HighPart;

	return phyAddr;
}
//#endif /* AHCI */

unsigned long *wrap_P2V(void *paddr);

bpVOID
OSD_GetSGAddress(bpREQ p_rq){
#if LINUX_VERSION_CODE<KERNEL_VERSION(2,6,14)
	return p_rq->p_buf;
#else
	return wrap_P2V(SG_STARTADDRESS(p_rq->pSgTable->table[0].StartAddress));
#endif

}
