/***************************************************************************
 *
 * Copyright (c) 1995 Promise Technology, Inc.  All rights reserved.
 * No part of this document may be reproduced or transmitted in any form or
 * by any means, electronic or mechanical, for any purpose, without the
 * express written permission of Promise Technology, Inc.
 *
 ***************************************************************************/

/***************************************************************************
 *
 * Author		:Jing Qian
 *
 * Module		:OSD Component Management
 * Revision		:1.0.0.1
 * Checkin Date	:01/01/2004
 *
 * Description	:Assign, Manage the Component/Component Extension.
 *
 * History		:
 *
 ***************************************************************************/
#include  "osd_inc.h"
extern BoOL gIsHibernation;

struct _ComponentInterface
{
	const char* CompName;	/* component name, debugging convinience */
	void (*COMP_GET_EXT_REQUIREMENT)(bpMEMORY_REQUIREMENT);
	void (*COMP_GET_DMA_REQUIREMENT)(U32, U32, bpMEMORY_REQUIREMENT);
	void (*COMP_GET_DIMM_REQUIREMENT)(U32, U32, bpMEMORY_REQUIREMENT);
	void (*COMP_INITIALIZE)(LPVOID,bpMEMORY_AVAILABLE,U32,U32,LPVOID);
	void (*COMP_START)(bpVOID,U32,U32);
	void (*COMP_STOP)(bpVOID);
	void (*COMP_RESTART)(bpVOID);
};

struct _ComponentInterface IComp[COMP_TYPE_MAXIMUM] = {
#if !defined(USE_EXPORT)
	{ /* comp_osd */
		"OSD",
		OSD_ComponentGetExtensionSize, 
		OSD_ComponentGetDMASafeMemoryRequirement, 
		OSD_ComponentGetDIMMRequirement, 
		OSD_ComponentInitialize, 
		OSD_ComponentStart,
		OSD_ComponentStop,
		OSD_ComponentRestart,
	},
	{ /* comp_cam */
		"CAM",
		CAM_ComponentGetExtensionSize, 
		CAM_ComponentGetDMASafeMemoryRequirement, 
		CAM_ComponentGetDIMMRequirement, 
		CAM_ComponentInitialize, 
		CAM_ComponentStart,
		CAM_ComponentStop,
		CAM_ComponentRestart
	},
	{ /* comp_dm */
		"DM",
		NULL, 
		NULL, 
		NULL, 
		NULL, 
		NULL, 
		NULL, 
		NULL
	},
	{ /* comp_engine */
		"Engine",
		Engine_ComponentGetExtensionSize, 
		Engine_ComponentGetDMASafeMemoryRequirement, 
		Engine_ComponentGetDIMMRequirement, 
		Engine_ComponentInitialize, 
		Engine_ComponentStart,
		Engine_ComponentStop,
		Engine_ComponentRestart
	},
#if !defined(DISABLE_PBM)
	{ /* comp_pbm */
		"PBM",
		PBM_ComponentGetExtensionSize, 
		PBM_ComponentGetDMASafeMemoryRequirement, 
		PBM_ComponentGetDIMMRequirement, 
		PBM_ComponentInitialize, 
		PBM_ComponentStart,
		PBM_ComponentStop,
		PBM_ComponentRestart
	}
#endif
#else
	/* to satifisy some compilers. */
	{	0, },
#endif
};

/***************************************************************************
 *						Component Management							   *
 ***************************************************************************/

#if defined(USE_EXPORT)
void OSDI_InitializeComponentInterfaceTable()
{
	IComp[COMP_TYPE_OSD].CompName = "OSD";
	IComp[COMP_TYPE_OSD].COMP_GET_EXT_REQUIREMENT = OSD_ComponentGetExtensionSize;
	IComp[COMP_TYPE_OSD].COMP_GET_DMA_REQUIREMENT = OSD_ComponentGetDMASafeMemoryRequirement;
	IComp[COMP_TYPE_OSD].COMP_GET_DIMM_REQUIREMENT = OSD_ComponentGetDIMMRequirement;
	IComp[COMP_TYPE_OSD].COMP_INITIALIZE = OSD_ComponentInitialize;
	IComp[COMP_TYPE_OSD].COMP_START = OSD_ComponentStart;
	IComp[COMP_TYPE_OSD].COMP_STOP = OSD_ComponentStop;
	IComp[COMP_TYPE_OSD].COMP_RESTART = OSD_ComponentRestart;

	IComp[COMP_TYPE_ENGINE].CompName = "Engine";
	IComp[COMP_TYPE_ENGINE].COMP_GET_EXT_REQUIREMENT = Engine_ComponentGetExtensionSize;
	IComp[COMP_TYPE_ENGINE].COMP_GET_DMA_REQUIREMENT = Engine_ComponentGetDMASafeMemoryRequirement;
	IComp[COMP_TYPE_ENGINE].COMP_GET_DIMM_REQUIREMENT = Engine_ComponentGetDIMMRequirement;
	IComp[COMP_TYPE_ENGINE].COMP_INITIALIZE = Engine_ComponentInitialize;
	IComp[COMP_TYPE_ENGINE].COMP_START = Engine_ComponentStart;
	IComp[COMP_TYPE_ENGINE].COMP_STOP = Engine_ComponentStop;
	IComp[COMP_TYPE_ENGINE].COMP_RESTART = Engine_ComponentRestart;

	IComp[COMP_TYPE_CAM].CompName = "CAM";
	IComp[COMP_TYPE_CAM].COMP_GET_EXT_REQUIREMENT = CAM_ComponentGetExtensionSize;
	IComp[COMP_TYPE_CAM].COMP_GET_DMA_REQUIREMENT = CAM_ComponentGetDMASafeMemoryRequirement;
	IComp[COMP_TYPE_CAM].COMP_GET_DIMM_REQUIREMENT = CAM_ComponentGetDIMMRequirement;
	IComp[COMP_TYPE_CAM].COMP_INITIALIZE = CAM_ComponentInitialize;
	IComp[COMP_TYPE_CAM].COMP_START = CAM_ComponentStart;
	IComp[COMP_TYPE_CAM].COMP_STOP = CAM_ComponentStop;
	IComp[COMP_TYPE_CAM].COMP_RESTART = CAM_ComponentRestart;

	IComp[COMP_TYPE_PBM].CompName = "PBM";

#if !defined(DISABLE_PBM)
#if !defined(USE_MML_CC)
	if( OSD_IsLdrDump(0) )
	{
		#if !defined(SIMULATED_DIMM)
		/* disable PBM if DIMM is not available to save memory requirement */
		IComp[COMP_TYPE_PBM].COMP_GET_EXT_REQUIREMENT = PBM_MINI_EXP_API(ComponentGetExtensionSize);
/*
		IComp[COMP_TYPE_PBM].COMP_GET_DMA_REQUIREMENT = PBM_MINI_EXP_API(ComponentGetDMASafeMemoryRequirement);
*/
		IComp[COMP_TYPE_PBM].COMP_GET_DIMM_REQUIREMENT = PBM_MINI_EXP_API(ComponentGetDIMMRequirement);
		IComp[COMP_TYPE_PBM].COMP_INITIALIZE = PBM_MINI_EXP_API(ComponentInitialize);
		IComp[COMP_TYPE_PBM].COMP_START = PBM_MINI_EXP_API(ComponentStart);
		#endif /*SIMULATED_DIMM*/
	}
	else
#endif		
	{
		IComp[COMP_TYPE_PBM].COMP_GET_EXT_REQUIREMENT = PBM_ComponentGetExtensionSize;
		IComp[COMP_TYPE_PBM].COMP_GET_DMA_REQUIREMENT = PBM_ComponentGetDMASafeMemoryRequirement;
		IComp[COMP_TYPE_PBM].COMP_GET_DIMM_REQUIREMENT = PBM_ComponentGetDIMMRequirement;
		IComp[COMP_TYPE_PBM].COMP_INITIALIZE = PBM_ComponentInitialize;
		IComp[COMP_TYPE_PBM].COMP_START = PBM_ComponentStart;
		IComp[COMP_TYPE_PBM].COMP_STOP = PBM_ComponentStop;
		IComp[COMP_TYPE_PBM].COMP_RESTART = PBM_ComponentRestart;
	}
#endif
}
#endif


unsigned long
OSDI_GetComponentExtensionRequirement(
	BoOL isHibernation
)
{
	MEMORY_REQUIREMENT totalRequirement = { 0, 0 };
	MEMORY_REQUIREMENT compRequirement;
	U16 CompIndex; 

	for( CompIndex=0; CompIndex<COMP_TYPE_MAXIMUM; CompIndex++ )
	//for( CompIndex=COMP_TYPE_MAXIMUM-1; CompIndex>=0; CompIndex-- )
	{
		OSD_TRACE( ("CompIndex =%d will get extension requirement\n", CompIndex) ); 
		if( IComp[CompIndex].COMP_GET_EXT_REQUIREMENT )
		{
			IComp[CompIndex].COMP_GET_EXT_REQUIREMENT(&compRequirement);
			totalRequirement.MinRequirement += compRequirement.MinRequirement;
			totalRequirement.MaxRequirement += compRequirement.MaxRequirement;
		}
	}
	totalRequirement.MinRequirement += COMP_TYPE_MAXIMUM * COMPONENT_HEADER_SIZE;
	totalRequirement.MaxRequirement += COMP_TYPE_MAXIMUM * COMPONENT_HEADER_SIZE;

	OSD_DPRINT( ("Required extension size: max: %d Min: %d\n", 
		totalRequirement.MaxRequirement, totalRequirement.MinRequirement) );

#if defined(USE_STATIC_DATA_FOR_COMPONENT_EXTENSIONS)
	IComp[COMP_TYPE_OSD].COMP_GET_EXT_REQUIREMENT( &compRequirement );
	compRequirement.MinRequirement += COMPONENT_HEADER_SIZE;

	OSD_DASSERT( OSD_LDRDUMP_BUFFER_SIZE >= 
		(totalRequirement.MinRequirement - compRequirement.MinRequirement) );

	totalRequirement.MinRequirement = compRequirement.MinRequirement;
#endif	/*USE_STATIC_DATA_FOR_COMPONENT_EXTENSIONS*/

	return (isHibernation 
		? totalRequirement.MinRequirement
		: totalRequirement.MaxRequirement);
}

static U32
OSDI_AssignComponent(
	IN bpOSD_COMP_EXTENSION pOSDExtension,
	IN PTR_SIZE	pTemp,
	IN U16	CompIndex )
{
	MEMORY_REQUIREMENT	compRequirement;
	PCOMPONENT			pCompHeader = NULL;

	if( IComp[CompIndex].COMP_GET_EXT_REQUIREMENT )
	{
		IComp[CompIndex].COMP_GET_EXT_REQUIREMENT(&compRequirement);

		pCompHeader = (PCOMPONENT) pTemp;

		pOSDExtension->ComponentResource[CompIndex].ComponentExtension 
			= (LPVOID) (pTemp + COMPONENT_HEADER_SIZE);

		/* Assign the back pointer to OSD Extension as well. */
		pCompHeader->OSDExtension = pOSDExtension;
		pCompHeader->ComponentType = CompIndex;
		pCompHeader->HeaderSize = COMPONENT_HEADER_SIZE;

		if ( gIsHibernation )
			pCompHeader->ExtensionSize = compRequirement.MinRequirement;
		else
			pCompHeader->ExtensionSize = compRequirement.MaxRequirement;
		
		pOSDExtension->ComponentResource[CompIndex].ComponentExtensionSize 
			= pCompHeader->ExtensionSize;

		return (COMPONENT_HEADER_SIZE + pCompHeader->ExtensionSize);
	}
	return 0;
}

BoOL
OSDI_AssignComponentExtension(
	IN bpOSD_COMP_EXTENSION pOSDExtension,
	IN LPVOID DeviceExtension,
	IN unsigned long  ExtensionSize
	)
{
	PTR_SIZE pTemp = (PTR_SIZE) DeviceExtension;
	U16 CompIndex = 0;

#if defined(USE_STATIC_DATA_FOR_COMPONENT_EXTENSIONS)
	if( OSD_IsLdrDump(pOSDExtension) )
	{
		OSDI_AssignComponent( pOSDExtension, pTemp, COMP_TYPE_OSD );
		pTemp = (PTR_SIZE) g_OsdLdrDumpBuffer;
		CompIndex++;
	}
#endif	/*USE_STATIC_DATA_FOR_COMPONENT_EXTENSIONS*/

	/* Assign Component Extension */
	for( /*CompIndex=0*/; CompIndex<COMP_TYPE_MAXIMUM; CompIndex++ )
	{
		pTemp += OSDI_AssignComponent( pOSDExtension, pTemp, CompIndex );
	}

	return TRUE;
}

unsigned long
OSDI_GetUncachedMemoryRequirement(
	IN bpOSD_COMP_EXTENSION pOSDExtension,
	IN BoOL isHibernation
	)
{
	unsigned long DeviceID = pOSDExtension->ControllerInfo.DeviceID;
	unsigned long SubDeviceID = pOSDExtension->ControllerInfo.SubDeviceID;
	U16 CompIndex;

	MEMORY_REQUIREMENT totalRequirement = { 0, 0 };
	MEMORY_REQUIREMENT compRequirement;

	for( CompIndex=0; CompIndex<COMP_TYPE_MAXIMUM; CompIndex++ )
	{
		if( IComp[CompIndex].COMP_GET_DMA_REQUIREMENT )
		{
			IComp[CompIndex].COMP_GET_DMA_REQUIREMENT(
				DeviceID, SubDeviceID, &compRequirement);
			totalRequirement.MinRequirement += compRequirement.MinRequirement;
			totalRequirement.MaxRequirement += compRequirement.MaxRequirement;
		}
	}

	OSD_DPRINT( ("Required DMA safe size: max: %d Min: %d\n", 
		totalRequirement.MaxRequirement, totalRequirement.MinRequirement) );

	return (isHibernation 
		? totalRequirement.MinRequirement
		: totalRequirement.MaxRequirement);
}

BoOL
OSDI_AssignUncachedMemory(
	IN bpOSD_COMP_EXTENSION pOSDExtension,
	IN LPVOID VirtualAddress,
	IN PHYADDR PhysicalAddress,
	IN unsigned long MemorySize
	)
{
	/* Check the Uncached Memory Size */
	unsigned long DeviceID = pOSDExtension->ControllerInfo.DeviceID;
	unsigned long SubDeviceID = pOSDExtension->ControllerInfo.SubDeviceID;

	unsigned long requirement = 
		OSDI_GetUncachedMemoryRequirement(pOSDExtension, gIsHibernation);

	PTR_SIZE pTempV = (PTR_SIZE)VirtualAddress;
	PHYADDR pTempP = PhysicalAddress;
	MEMORY_REQUIREMENT compRequirement;
	U16 CompIndex;

	OSD_ASSERT(MemorySize>=requirement);

	/* Assign Uncached Memory */
	for( CompIndex=0; CompIndex<COMP_TYPE_MAXIMUM; CompIndex++ )
	{
		if( IComp[CompIndex].COMP_GET_DMA_REQUIREMENT )
		{
			IComp[CompIndex].COMP_GET_DMA_REQUIREMENT(DeviceID, SubDeviceID, &compRequirement);
			pOSDExtension->ComponentResource[CompIndex].DMASafeAddress = (LPVOID)pTempV;
			pOSDExtension->ComponentResource[CompIndex].DMASafePhysicalAddress = pTempP;

			if ( gIsHibernation )
			{
				pOSDExtension->ComponentResource[CompIndex].DMASafeSize =
					compRequirement.MinRequirement;
				pTempV += compRequirement.MinRequirement;
				pTempP.u.Low += compRequirement.MinRequirement;
			}
			else
			{
				pOSDExtension->ComponentResource[CompIndex].DMASafeSize =
					compRequirement.MaxRequirement;
				pTempV += compRequirement.MaxRequirement;
				pTempP.u.Low += compRequirement.MaxRequirement;
			}
		}
	}

	return TRUE;
}

unsigned long
OSDI_GetDIMMRequirement(
	IN bpOSD_COMP_EXTENSION pOSDExtension,
	IN BoOL isHibernation
	)
{
	unsigned long DeviceID = pOSDExtension->ControllerInfo.DeviceID;
	unsigned long SubDeviceID = pOSDExtension->ControllerInfo.SubDeviceID;
	U16 CompIndex;

	MEMORY_REQUIREMENT totalRequirement = { 0, 0 };
	MEMORY_REQUIREMENT compRequirement;

	for( CompIndex=0; CompIndex<COMP_TYPE_MAXIMUM; CompIndex++ )
	{
		if( IComp[CompIndex].COMP_GET_DIMM_REQUIREMENT )
		{
			IComp[CompIndex].COMP_GET_DIMM_REQUIREMENT(
				DeviceID, SubDeviceID, &compRequirement);
			totalRequirement.MinRequirement += compRequirement.MinRequirement;
			totalRequirement.MaxRequirement += compRequirement.MaxRequirement;
		}
	}

	return (isHibernation 
		? totalRequirement.MinRequirement
		: totalRequirement.MaxRequirement);
}

BoOL
OSDI_AssignDIMM(
	IN bpOSD_COMP_EXTENSION pOSDExtension,
	IN DIMMADDR DIMMAddress,
	IN unsigned long DIMMSize
	)
{
	/* Check the DIMM Size */
	unsigned long DeviceID = pOSDExtension->ControllerInfo.DeviceID;
	unsigned long SubDeviceID = pOSDExtension->ControllerInfo.SubDeviceID;

	unsigned long requirement = 
		OSDI_GetDIMMRequirement(pOSDExtension, gIsHibernation);

	DIMMADDR pTemp = DIMMAddress;
	MEMORY_REQUIREMENT compRequirement;
	U16 CompIndex;

	OSD_ASSERT(DIMMSize>=requirement);

	/* Assign Uncached Memory */
	for( CompIndex=0; CompIndex<COMP_TYPE_MAXIMUM; CompIndex++ )
	{
		if( IComp[CompIndex].COMP_GET_DIMM_REQUIREMENT )
		{
			IComp[CompIndex].COMP_GET_DIMM_REQUIREMENT(
				DeviceID, SubDeviceID, &compRequirement);
#if !defined(SIMULATED_DIMM)
			pTemp = ALIGN_ROUND( pTemp, 0x10000 );	/* 64K alignment */
#endif
			pOSDExtension->ComponentResource[CompIndex].DIMMAddress = pTemp;

			/* Just assign all the DIMM left to PBM. */
			if ( CompIndex==COMP_TYPE_PBM )
			{
				OSD_ASSERT(CompIndex==COMP_TYPE_MAXIMUM-1);	/* It must be the last one. */
				pOSDExtension->ComponentResource[CompIndex].DIMMSize = 
					DIMMAddress+DIMMSize-pTemp;
				pTemp += pOSDExtension->ComponentResource[CompIndex].DIMMSize; /* Although it's not neccessary. */
			}
			else
			{
				if ( gIsHibernation )
				{
					pOSDExtension->ComponentResource[CompIndex].DIMMSize =
						compRequirement.MinRequirement;
					pTemp += compRequirement.MinRequirement;
				}
				else
				{
					pOSDExtension->ComponentResource[CompIndex].DIMMSize =
						compRequirement.MaxRequirement;
					pTemp += compRequirement.MaxRequirement;
				}
			}
		}
	}

	return TRUE;
}

/* It includes DIMM and DMA Safe memory. 
 * DeviceExtension has been assigned already at the very beginning. 
 */
BoOL
OSDI_AssignComponentResource(
	IN bpOSD_COMP_EXTENSION  OSDComponentExtension)
{
	bpOSD_COMP_EXTENSION pOSDExtension = OSDComponentExtension;
	BoOL result;

	/* Uncached Memory */
	LPVOID VirtualAddress = pOSDExtension->DMASafeAddress;
	PHYADDR PhysicalAddress = pOSDExtension->DMASafePhysicalAddress;
	unsigned long MemorySize = pOSDExtension->DMASafeSize;	

	/* Till now, we still don't how much DIMM we have. */
	unsigned long DIMMSize = OSDI_GetDIMMRequirement(pOSDExtension, FALSE);
	/* Leave 64K. Must leave some space. 
	 * CAM cannot accept DIMM address 0. Otherwise when it'll think out of memory when assigning DIMM for SG Table.
	 */ 
	DIMMADDR DIMMAddress = 0; //TBD: the exact size.

	/* Assign Uncached Memory */
	result = OSDI_AssignUncachedMemory(
				pOSDExtension,
				VirtualAddress,
				PhysicalAddress,
				MemorySize
			);
	OSD_ASSERT(result);

	/* Assign DIMM */
	result = OSDI_AssignDIMM(
				pOSDExtension,
				DIMMAddress,	//TBD: Do we need leave some space
				DIMMSize
			);
	OSD_ASSERT(result);

	return TRUE;
}


/* Initialize the Components. */
BoOL 
OSDI_InitializeComponents(
	IN OUT bpOSD_COMP_EXTENSION pOSDExtension,
 	IN unsigned long DeviceID,
	IN unsigned long SubDeviceID
	)
{
	MEMORY_AVAILABLE memAvailable;
	PCOMP_RESOURCE pCompResource;
	U16 CompIndex;
	U8 i;
	LPVOID pContext = NULL;
	CAM_CONTROLLER_CONFIG_INFO controllerInfo;	/* For CAM only. */
  ENGINE_INIT_CONFIG EngineConfig;

	/* OSD Component is the one which is served first. 
	 * The order ( the index ) of each Compnent is very important.
	 */

	for( CompIndex=0; CompIndex<COMP_TYPE_MAXIMUM; CompIndex++ )
	{
		if( IComp[CompIndex].COMP_INITIALIZE )
		{
			pCompResource = &pOSDExtension->ComponentResource[CompIndex];
			memAvailable.ComponentExtension = pCompResource->ComponentExtension ;
			memAvailable.ComponentExtensionSize = pCompResource->ComponentExtensionSize ;
			memAvailable.DMASafeAddress = pCompResource->DMASafeAddress ;
			memAvailable.DMASafeSize = pCompResource->DMASafeSize ;
			memAvailable.DMASafePhysicalAddress = pCompResource->DMASafePhysicalAddress ;

			switch ( CompIndex ){
				case COMP_TYPE_OSD:
					//pContext = ConfigInfo;
					break;

				case COMP_TYPE_CAM:
                    /* For CAM, this parameter should be a controller information structure. */
					for ( i=0; i<BASE_ADDRESS_NUM; i++ )
						controllerInfo.BaseAddresses[i] = pOSDExtension->BaseAddress[i];
#ifdef SIMULATED_DIMM
					controllerInfo.SimulatedDimmSize = pOSDExtension->SimulatedDimmSize;
#endif

#if defined(NAPA_MARVELL)
					{
						U8   Revision;
						pci_bus_read_config_byte(pOSDExtension->pdev->bus, 
											    pOSDExtension->pdev->bus,
											    PCI_REVISION_ID_ADDRESS,
											    &Revision);
						controllerInfo.Revision = Revision;
					}
#endif
					pContext = &controllerInfo;
					break;

case COMP_TYPE_ENGINE:
					{
						extern U32 g_JbodOnly;
						EngineConfig.JbodOnly = g_JbodOnly ? TRUE : FALSE;
						pContext = &EngineConfig;
					}
					break;

				default:
					pContext = NULL;
					break;
			}
	
			IComp[CompIndex].COMP_INITIALIZE(
				pCompResource->ComponentExtension, 
				&memAvailable, 
				DeviceID, 
				SubDeviceID,
				pContext
				);
		}
	}

	return TRUE;
}

 void 
 OSDI_StartComponents(
	bpOSD_COMP_EXTENSION pOSDExtension,
	U16 BeginComp
	)
{
	U16 CompIndex;
	BoOL isRestart = OSDI_IsRestart(pOSDExtension);

	pOSDExtension->State = SYS_STATE_STARTING;

	for( CompIndex=BeginComp; CompIndex<COMP_TYPE_MAXIMUM; CompIndex++ )
	{
		if( IComp[CompIndex].COMP_START==0 )
		{
			pOSDExtension->CompStartStatus |= (1<<CompIndex);
		}
	}

	/* 
		Important: The order of the Component Index must be the same of the order of Start Sequence 
		So: Index: OSD->CAM->DM->Engine->PBM
	*/
	for( CompIndex=BeginComp; CompIndex<COMP_TYPE_MAXIMUM; CompIndex++ )
	{
		if( IComp[CompIndex].COMP_START )
		{
			if ( pOSDExtension->CompStartStatus&(1<<CompIndex) )
				continue;
			else
			{
				if ( !isRestart)
				{
					IComp[CompIndex].COMP_START(
						pOSDExtension->ComponentResource[CompIndex].ComponentExtension,	/* Extension */	
						pOSDExtension->ComponentResource[CompIndex].DIMMAddress,		/* DIMM Offset */
						pOSDExtension->ComponentResource[CompIndex].DIMMSize			/* DIMM Size */
						);
					return;
				}
				else
				{
					IComp[CompIndex].COMP_RESTART(
						pOSDExtension->ComponentResource[CompIndex].ComponentExtension	/* Extension */	
						);
					return;
				}
			}
		}
		else
		{
			pOSDExtension->CompStartStatus |= (1<<CompIndex);
		}
	}
	return;

}

void
OSDI_RestartComponents(
	bpOSD_COMP_EXTENSION pOSDExtension
	)
{
	OSDI_SetRestartFlag(pOSDExtension);
	OSDI_ClearComponentsReadyFlag(pOSDExtension);

	OSDI_StartComponents(pOSDExtension, COMP_TYPE_OSD);
}

void 
OSDI_StopComponents(
	bpOSD_COMP_EXTENSION pOSDExtension
	)	
{
	U16 CompIndex;

	/*	Stop all the other Components. Begin at the PBM, end with CAM. 
	  COMP_TYPE_CAM==1 COMP_TYPE_DM==2 COMP_TYPE_ENGINE=3 COMP_TYPE_PBM==4
	*/
	for( CompIndex=COMP_TYPE_PBM; CompIndex>=COMP_TYPE_CAM; CompIndex-- )
	{
		if( IComp[CompIndex].COMP_STOP )
		{
			IComp[CompIndex].COMP_STOP(
				OSD_GetComponentExtension(pOSDExtension, CompIndex)
				);

			OSD_TRACE(("Component %s is stopped.\n", IComp[CompIndex].CompName));
		}
	}

	/* Stop OSD Component First. Suppose COMP_STOP functions are synchronized. */
	OSD_ComponentStop(pOSDExtension);
    OSD_TRACE(("OSD Component is stopped.\n"));
}

/* Whenever there is a component started, it'll call the OSD_ComponentStarted function. */
#define COMP_START_STATUS_SUCCESS	1

void OSDI_Start_Engine_Maintenance (
	bpVOID ComponentExtension,
	bpVOID Timer,
	bpVOID Context
	)
{
	bpOSD_COMP_EXTENSION pOSDExtension = (bpOSD_COMP_EXTENSION) ComponentExtension;

	pOSDExtension->State = SYS_STATE_FULLY_STARTED;

	Engine_OnBroadcasting(
		OSD_GetComponentExtension(ComponentExtension, COMP_TYPE_ENGINE),
		BROADCAST_SYSTEM_FULLY_STARTED, 0, 0
	);
}

#if !defined(SIMULATED_DIMM)
VOID
OSDFindValidLastTimeDimmInfo(
	bpOSD_COMP_EXTENSION OsdCompExt
	)
{
	DIMM_HEADER DimmHeader;
	bpVOID		pCamExt = OSD_GetComponentExtension( OsdCompExt, COMP_TYPE_CAM );
	bpVOID		pExt;
	U16			i;

	CAM_DIMMRead( pCamExt, &DimmHeader, 0, sizeof(DIMM_HEADER) );

	if( DimmHeader_CheckValidity(&DimmHeader) )
	{
		OSD_TRACE( ("Valid DIMM header information found!\n") );
		for( i=0; i<COMP_TYPE_MAXIMUM; i++ )
		{
			if( i==COMP_TYPE_CAM )
				continue;
			pExt = OSD_GetComponentExtension(OsdCompExt,i);
			if( pExt )
			{
				OsdCompExt->ComponentResource[i].DIMMAddress = DimmHeader.Arenas[i].Offset;
				OsdCompExt->ComponentResource[i].DIMMSize = DimmHeader.Arenas[i].Length;
			}
		}
	}
}

VOID
OSDSaveDimmInfo(
	bpOSD_COMP_EXTENSION OsdCompExt
	)
{
	DIMM_HEADER DimmHeader;
	U16			i;

	DimmHeader.Signature = DIMM_HEADER_SIGNATURE;

	for( i=0; i<COMP_TYPE_MAXIMUM; i++ )
	{
		bpVOID pExt = OSD_GetComponentExtension(OsdCompExt,i);
		if( pExt )
		{
			DimmHeader.Arenas[i].Offset = OsdCompExt->ComponentResource[i].DIMMAddress;
			DimmHeader.Arenas[i].Length = OsdCompExt->ComponentResource[i].DIMMSize;
		}
	}

	DimmHeader_Checksum( &DimmHeader );

	CAM_DIMMWrite( OSD_GetComponentExtension(OsdCompExt,COMP_TYPE_CAM), 
		&DimmHeader, 
		0, 
		sizeof(DIMM_HEADER) );
}
#endif /* !defined(SIMULATED_DIMM) */

void
DECLARE_OSD_EXP(ComponentStarted)(
	LPVOID SenderComponentExtension,
	U32 StartStatus
	)
{	
	U16 ComponentType = 
		OSDI_GetComponentType(SenderComponentExtension);
	U32 DIMMSize = 0;
	U8 DIMMType = 0;
	DIMMADDR DIMMAddress = 0;

	bpOSD_COMP_EXTENSION pOSDExtension = (bpOSD_COMP_EXTENSION)
		OSD_GetComponentExtension(SenderComponentExtension, COMP_TYPE_OSD);

	OSD_ASSERT(StartStatus==COMP_START_STATUS_SUCCESS);

	OSD_TRACE(("Component %s is started.\n", IComp[ComponentType].CompName));

	/* Mark the corresponding component is started. */
	pOSDExtension->CompStartStatus |= (1<<ComponentType);

	if ( ComponentType==COMP_TYPE_CAM )
	{
		/* Adjust DIMM information for PBM at this time according to available DIMM size. */
		DIMMSize = CAM_DIMMGetSize(SenderComponentExtension, &DIMMType);

		pOSDExtension->ControllerInfo.DIMMSize = DIMMSize;
		pOSDExtension->ControllerInfo.DIMMType = DIMMType;
		OSDI_AssignDIMM(pOSDExtension, DIMMAddress, DIMMSize);

#if !defined(SIMULATED_DIMM)
		if( OSD_IsLdrDump(pOSDExtension) )
		{
			OSDFindValidLastTimeDimmInfo(pOSDExtension);
		}
#endif	/* !defined(SIMULATED_DIMM) */
	}

	if ( !OSDI_ComponentsAreReady(pOSDExtension) )
	{
		/* Start other Component. */
		OSDI_StartComponents(pOSDExtension, ComponentType+1);
	}
	else
	{	
	//	if(pOSDExtension->init_status != NP_INIT_FAILED)
		{
			pOSDExtension->State = SYS_STATE_STARTED;
			OSD_TRACE( ("OSD_ComponentExtension: all component are ready now\n") ); 
			pOSDExtension->init_status = NP_INIT_FINISHED;
			wake_up(&pOSDExtension->Init_Waitq);
		
#if !defined(SIMULATED_DIMM)
			OSDSaveDimmInfo( pOSDExtension );
#endif	/* !defined(SIMULATED_DIMM) */
		/* All the Components are started. Is there anything we need do? */
		// TBD
			#define	DELAY_ENGINE_START_MAINTAINANCE_TIMER 60000000
			OSD_RequestTimer(
				pOSDExtension, DELAY_ENGINE_START_MAINTAINANCE_TIMER,
				OSDI_Start_Engine_Maintenance, NULL, 0);
			}
	}
}
