/**********************************************************************
 *	Component Management: 
 *	Manage all the components including PBM, Engine, DM, CAM and OSD.
 *********************************************************************/
#if !defined(OSD_COMP_H)
#define OSD_COMP_H

#if defined(USE_EXPORT)
void OSDI_InitializeComponentInterfaceTable(void);
#endif

/* Gather Component Resource Requirement. */
unsigned long
OSDI_GetComponentExtensionRequirement(BoOL isHibernation);

unsigned long 
OSDI_GetUncachedMemoryRequirement(
	IN bpOSD_COMP_EXTENSION pOSDExtension,
	IN BoOL IsHibernation
	);



unsigned long
OSDI_GetDIMMRequirement(
	IN bpOSD_COMP_EXTENSION pOSDExtension,
	IN BoOL isHibernation
	);


/* Assign Resource to Components. */
BoOL 
OSDI_AssignComponentExtension(
	IN bpOSD_COMP_EXTENSION pOSDExtension,
	IN LPVOID DeviceExtension,
	IN unsigned long ExtensionSize
);

BoOL
OSDI_AssignUncachedMemory(
	IN bpOSD_COMP_EXTENSION pOSDExtension,
	IN LPVOID VirtualAddress,
	IN PHYADDR PhysicalAddress,
	IN unsigned long MemorySize
);

BoOL
OSDI_AssignDIMM(
	IN bpOSD_COMP_EXTENSION pOSDExtension,
	IN DIMMADDR DIMMAddress,
	IN unsigned long DIMMSize
);
BoOL
OSDI_AssignComponentResource(
	IN bpOSD_COMP_EXTENSION  OSDComponentExtension);

/* Component Operations */
BoOL 
OSDI_InitializeComponents(
	IN OUT bpOSD_COMP_EXTENSION pOSDExtension,
 	IN unsigned long DeviceID,
	IN unsigned long SubDeviceID
	);

void
OSDI_StartComponents(
	bpOSD_COMP_EXTENSION pOSDExtension,
	U16 BeginComp
	);

void
OSDI_RestartComponents(
	bpOSD_COMP_EXTENSION pOSDExtension
	);

void 
OSDI_StopComponents(
	bpOSD_COMP_EXTENSION pOSDExtension
	);

void 
OSDI_Start_Engine_Maintenance (
	bpVOID ComponentExtension,
	bpVOID Timer,
	bpVOID Context
);

/* bit(0):OSD, bit(1):CAM, bit(2):DM, bit(3):Engine, bit(4):PBM */
#define COMPS_ALL_STARTED	(bit(0)|bit(1)|bit(2)|bit(3)|bit(4))
/* If bit(7) is set, it's during restart procedure. */
#define COMPS_RESTART bit(7)	

#define OSDI_ComponentsAreReady(pOSDExtension) ( (pOSDExtension->CompStartStatus&COMPS_ALL_STARTED)==COMPS_ALL_STARTED )
//#define OSDI_IsRestart(pOSDExtension) ( (pOSDExtension->CompStartStatus&COMPS_RESTART)==COMPS_RESTART )
#define OSDI_IsRestart(pOSDExtension) ( (pOSDExtension->CompStartStatus&COMPS_RESTART)==COMPS_RESTART )

#define OSDI_ClearComponentsReadyFlag(pOSDExtension) pOSDExtension->CompStartStatus&=~COMPS_ALL_STARTED;
#define OSDI_SetRestartFlag(pOSDExtension) pOSDExtension->CompStartStatus|=COMPS_RESTART;
#define OSDI_ClearRestartFlag(pOSDExtension) pOSDExtension->CompStartStatus&=~COMPS_RESTART;

#endif

