/*************************************************************************
 *
 *	Here is the definition of OSD internal data structure
 *
 *************************************************************************/
#if !defined(OSDI_H)
#define OSDI_H

/*************************************************************************
 *
 *	Component Management Related Data Structure
 *
 *************************************************************************/
/* Component Type Definition. Don't change the order. */
typedef struct _COMPONENT
{
	/* ... Here is the Component Header... */
	U16		ComponentType;		/* see COMP_xxx definition */
	U16		HeaderSize;			/* size of the header only */
	U32		ExtensionSize;		/* size of the ComponentExtension only */
	LPVOID	OSDExtension;		/* fast back reference pointer */

	/* ... Here is the Component Extension... */
	U8		ComponentExtension[1];
} COMPONENT, FAR* PCOMPONENT;

typedef struct _COMP_RESOURCE
{
	LPVOID		ComponentExtension;			/* point to the Component Extension */

	unsigned long		ComponentExtensionSize;		/* size of the Component Extension */
	unsigned long		DMASafeSize;				/* size of the assigned DMA-safe mem */
	LPVOID		DMASafeAddress;				/* virtual address of the assigned DMA-safe Memory */
	DIMMADDR	DIMMAddress;					/* assigned DIMM address */
	unsigned long		DIMMSize;					/* size of the DIMM */
	PHYADDR		DMASafePhysicalAddress;		/* Physical Address the DMA Safe Memory*/
}COMP_RESOURCE, *PCOMP_RESOURCE;


/*************************************************************************
 *
 *	OSD Component Extension
 *
 *************************************************************************/

typedef struct _CONTROLLER_INFO
{
	unsigned long SystemIoBusNumber;
	unsigned long SlotNumber;
	unsigned long InterruptLevel;
	unsigned long InterruptVector;
	unsigned long DeviceID;
	unsigned long SubDeviceID;
	
	unsigned long DIMMSize;
	unsigned char DIMMType;
	unsigned char Reserved[3];
} CONTROLLER_INFO, * PCONTROLLER_INFO;

typedef struct SGTABLE SGTABLE_T;

#define PASSTHRU_REQ_HEAD_TYPE 	0x00000001
#define PENDING_REQ_HEAD_TYPE 	0x80000000
#define SLEEPING_REQ_HEAD_TYPE  	0x40000000
typedef struct	_REQ_HEAD {
   REQ			req;
   SGTABLE		SgTable;
   void			*cmd;
   void			(*fn)(void *);   
   unsigned long 	ReqType;
   void  		*pSptd; 
   wait_queue_head_t	waitq;	
   struct _REQ_HEAD	*next;
} REQ_HEAD, *PREQ_HEAD; 

typedef struct	_DATA_POOL {
   unsigned char *	Buffer;
   unsigned char *	SenseBuffer;
   struct _DATA_POOL	*next;
} DATA_POOL, *PDATA_POOL; 

#define NP_INIT_UNDERWAY 0
#define NP_INIT_FINISHED 1
#define NP_INIT_FAILED 2

typedef struct _OSD_COMP_EXTENSION 
{ 
	/* Component Management Related Variables */
	COMP_RESOURCE ComponentResource[COMP_TYPE_MAXIMUM];	/* 5 Components */

	/* Device Extension */
	bpVOID	DeviceExtension;

	U32 State;
	/* Controller Related Variabled */
	CONTROLLER_INFO ControllerInfo;

	/* Resource Address */
	LPVOID	BaseAddress[BASE_ADDRESS_NUM];
	LPVOID	DMASafeAddress;			/* The DMA Safe Memory got from OS. */
	U32	DMASafeSize; 			/* The DMA Safe Memory Size. */
	dma_addr_t DMAAddr;
	PHYADDR	DMASafePhysicalAddress; /* The DMA Safe Memory Physical Address. */
#if defined(SIMULATED_DIMM)
	U32		SimulatedDimmSize;
#endif

#if defined(CHECK_DMA_DIMM_AREA)
	#define MAX_DMADIMM_AREA	10
	U32		DMADimmCount;
	struct	
	{
		U32 DimmAddr;
		U32 Length;
	} DMADimmArea[MAX_DMADIMM_AREA];
#endif

#if defined(OSD_CHECK_REQ_TIMEOUT)
	GEN_LIST_HEADER	CurrentReqList;
#endif

	/* Timer Queue */
	/* struct linux_timer_list TimerListPool[MAX_TIMER_QUEUE]; */
	struct timer_list TimerListPool[MAX_TIMER_QUEUE];
	Timer_Queue_t TimerQueue[MAX_TIMER_QUEUE];				/* Timer Management Variable */
	Timer_Queue_t * pTimerCurrent;

	/* OSD Req Head Queue*/
	REQ_HEAD  ReqHeadPool[CAN_QUEUE];
	PREQ_HEAD	pReqCurrent;
	/* add sense buffer for internal synchronize cache req use */
	unsigned char SenseBuffer[SCSI_SENSE_BUFFERSIZE];
	
	//wait_queue_head_t WaitQ;
 	wait_queue_head_t  Init_Waitq;
	
	spinlock_t              datapool_lock; 
	spinlock_t		 timer_lock;
	spinlock_t		 res_lock;
 	unsigned long 		 resflags;

	struct fasync_struct * async_queue;
	
      	/* OS Related Variables */
      	int			 AdapterIndex;
  	struct pci_dev          *pdev;
  	struct Scsi_Host        *host;             /* pointer to scsi host */
  	int                      host_no;          /* SCSI host number */
  	unsigned int     major;            /* control device's major */
  	/* int                   counter;          counter of using control device*/
  	unsigned char		irq;
  	unsigned char		bus_num;
  	unsigned char		devfn;
  	unsigned char		present;
  	
	unsigned char	CompStartStatus;		/* All components have started or not? */
	unsigned char init_status;
	
	/* Used for ioctl, no use now 
	unsigned char DataBuffer[MAX_DATA_BUFFER_SIZE*MAX_DATA_BUFFER_COUNT]; */
	
	DATA_POOL DataPool[MAX_DATA_BUFFER_COUNT];
	PDATA_POOL	pDataPoolCurrent;
	
  	unsigned	char	TimerPoolBuffer[1];
}OSD_COMP_EXTENSION,buf_space *bpOSD_COMP_EXTENSION;


/* Define FIELD_OFFSET for Linux */
#define COMPONENT_EXTENSION_OFFSET FIELD_OFFSET(COMPONENT, ComponentExtension)

#define	COMPONENT_HEADER_SIZE	COMPONENT_EXTENSION_OFFSET

/* Get the Component Header for each Component Extension */
#define OSDI_GetComponentHeader(pComponentExtension)	(LPVOID)((PTR_SIZE)(pComponentExtension)- COMPONENT_EXTENSION_OFFSET)

/* Return the Component Type */
#define	OSDI_GetComponentType(ComponentExtension) ((PCOMPONENT)(OSDI_GetComponentHeader(ComponentExtension)))->ComponentType

/* From the HwDeviceExtension, get the OSD Extension 
 * If Component Extension is got from global variable, just redefine this macro.
 */
#define OSDI_GetOSDExtensionFromDeviceExtension(pDeviceExtension)	(bpOSD_COMP_EXTENSION)((PTR_SIZE)pDeviceExtension + COMPONENT_EXTENSION_OFFSET)

/* From any kind of Component Extension, get the OSD Extension */
#define OSDI_GetOSDExtension(pComponentExtension) ((PCOMPONENT)(OSDI_GetComponentHeader(pComponentExtension)))->OSDExtension

/* DRIVER_LOCK/UNLOCK pair used for synchronization in interrupt handler or timer handler */
/* In 2.4 kernel, we should use block layer lock: io_request_lock for io completions, but */ 					/* in 2.6 kernel we should take host_lock in Scsi_host struture	instead			  */
#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,5,0) 

#define DRIVER_LOCK(host,flags)		spin_lock_irqsave(&io_request_lock, flags)
#define DRIVER_UNLOCK(host,flags)	spin_unlock_irqrestore(&io_request_lock, flags);

#else

#define DRIVER_LOCK(host,flags)		spin_lock_irqsave(host->host_lock, flags)
#define DRIVER_UNLOCK(host,flags)         spin_unlock_irqrestore(host->host_lock, flags)

#endif

#define RES_LOCK_INIT		spin_lock_init(&p->res_lock);
#define RES_LOCK		spin_lock_irqsave(&p->res_lock, p->resflags);
#define RES_UNLOCK		spin_unlock_irqrestore(&p->res_lock, p->resflags);

#if defined(USE_STATIC_DATA_FOR_COMPONENT_EXTENSIONS)
#ifndef NAPA_MARVELL
#define OSD_LDRDUMP_BUFFER_SIZE	(48*1024)
#else
#define OSD_LDRDUMP_BUFFER_SIZE	(80*1024)
#endif
extern U8 g_OsdLdrDumpBuffer[];
#endif	/*USE_STATIC_DATA_FOR_COMPONENT_EXTENSIONS*/

#endif /* OSDI_H */


