/***************************************************************************
*
* Copyright (c) 1995 Promise Technology, Inc.  All rights reserved.
* No part of this document may be reproduced or transmitted in any form or
* by any means, electronic or mechanical, for any purpose, without the
* express written permission of Promise Technology, Inc.
*
***************************************************************************/

/***************************************************************************
*
* Author		:Isaac Huang
*
* Module		:OSD IOCTL HANDLE
* Revision		:1.0.0.1
* Checkin Date	:12/05/2004
*
* Description	: provide ioctl interface for all kinds of ioctl command. 
*				
* History		:
*
***************************************************************************/
#include "osd_inc.h"

/* struct timer_list IOCTL_Timer; */

#if VDEBUG
extern int gFlagInt;
int gIoctlFlag;
#endif

void init_data_pool(bpVOID OSDExtension);
int ft_open(struct inode *inodep, struct file *filep);
int ft_release(struct inode *inodep, struct file *filep);
int ft_ioctl(struct inode *inodep,struct file *filep,unsigned int req,unsigned long arg);
#if defined (_64BPLATFORM) && (LINUX_VERSION_CODE>=KERNEL_VERSION(2,6,14))
int ft_compat_ioctl(struct file *filep,unsigned int req,unsigned long arg);
/*add new ft_ioctl from compat_ioctl, solve kernel version after 2.6.14 remove register_ioctl32_conversion, 
then 32bits applications(ex: webpam) can't work on 64 bits OS, by sandy 20070309*/
#endif

int ft_fasync(int fd, struct file * filep, int mode);

extern unsigned char * t3_hostp[MAX_ADAPTORS];

struct file_operations t3_fops = {
	owner:			THIS_MODULE,
        ioctl:                  ft_ioctl,
#if defined (_64BPLATFORM) && (LINUX_VERSION_CODE>=KERNEL_VERSION(2,6,14))
	compat_ioctl:		ft_compat_ioctl,
#endif
/*add new ft_ioctl from compat_ioctl, solve kernel version after 2.6.14 remove register_ioctl32_conversion,
then 32bits applications(ex: webpam) can't work on 64 bits OS, by sandy 20070309*/

	fasync:			ft_fasync,
	open:			ft_open,
	release:		ft_release
};


void 
init_data_pool(bpVOID OSDExtension)
{
	bpOSD_COMP_EXTENSION p = (bpOSD_COMP_EXTENSION)OSDExtension;
	ULONG constantSize = FIELD_OFFSET(OSD_COMP_EXTENSION, TimerPoolBuffer);
	int i;

	for (i=0;i<MAX_DATA_BUFFER_COUNT;i++)
	{
		p->DataPool[i].Buffer = ( (PU8)p + constantSize + i*( (MAX_DATA_BUFFER_SIZE)+ SPT_SENSE_LENGTH) );
		p->DataPool[i].SenseBuffer = p->DataPool[i].Buffer + (MAX_DATA_BUFFER_SIZE);
		OSD_DPRINT( ("Buffer %d address is %p\n", i, p->DataPool[i].Buffer) );
		if(MAX_DATA_BUFFER_COUNT -1 == i)
			p->DataPool[i].next=NULL;
		else
			p->DataPool[i].next=&(p->DataPool[i+1]);
	}	
	
	p->pDataPoolCurrent = p->DataPool;
	
	return;	
}
PDATA_POOL 
alloc_data_pool(bpVOID OSDExtension)
{
	bpOSD_COMP_EXTENSION p = (bpOSD_COMP_EXTENSION)OSDExtension;
	unsigned long flags;
	PDATA_POOL pDataPool;
	
	spin_lock_irqsave(&p->datapool_lock, flags);
	
	pDataPool = p->pDataPoolCurrent;

	if(NULL == pDataPool){
		OSD_DPRINT( ("alloc_data_pool: data pool is empty\n") );
		spin_unlock_irqrestore(&p->datapool_lock, flags);
		return NULL;
	}	
	
	p->pDataPoolCurrent = pDataPool->next;

	spin_unlock_irqrestore(&p->datapool_lock, flags);
	
	pDataPool->next = NULL;

	return pDataPool;
}

void 
release_data_pool(bpVOID OSDExtension, PDATA_POOL pDataPool)
{
	bpOSD_COMP_EXTENSION p = (bpOSD_COMP_EXTENSION)OSDExtension;
	unsigned long flags;

	spin_lock_irqsave(&p->datapool_lock, flags);
	
	if(NULL == pDataPool){
		OSD_DPRINT( ("release_data_pool: intend to put NULL data buffer to pool \n") );
		spin_unlock_irqrestore(&p->datapool_lock, flags);
		return;
	}	
	
	pDataPool->next = p->pDataPoolCurrent;
	p->pDataPoolCurrent = pDataPool;

	spin_unlock_irqrestore(&p->datapool_lock, flags);

	return;	
}

/* function   :int ft_open(struct inode *inodep, struct file *filep)
 * input        : Standard Parameters
 * output      : 0 if success, otherwise error reason
 * description  : open device
 */
int ft_open(struct inode *inodep, struct file *filep)
{
	/* unsigned long flags; */
	int aptno = MINOR(inodep->i_rdev);
	bpOSD_COMP_EXTENSION p = OSDI_GetOSDExtensionFromDeviceExtension(t3_hostp[aptno]);	
		
	//OSD_DPRINT( ("try to open Adapter %d \n", aptno) );

	if(!t3_hostp[aptno] || p->major <= 0 || p->major != MAJOR(inodep->i_rdev) ){
	   	OSD_TRACE( ("open Adapter %d error\n", aptno) );
		return(-ENODEV);
	}
	/* spin_lock_irqsave(&p->simple_lock, flags);
	if(p->counter != 0) {
		spin_unlock_irqrestore(&p->simple_lock, flags);
		return(-EBUSY);
	}
	p->counter++;
	spin_unlock_irqrestore(&p->simple_lock, flags);*/

	filep->private_data = (void *)p;

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
	MOD_INC_USE_COUNT;
#endif
	return(0);
}

/* function   :int ft_release(struct inode *inodep, struct file *filep)
 * input        : Standard Parameters
 * output      : 0 if success, otherwise error reason
 * description  : release device
 */
int ft_release(struct inode *inodep, struct file *filep)
{
	/* unsigned long flags; */
	int aptno = MINOR(inodep->i_rdev);
	bpOSD_COMP_EXTENSION p = OSDI_GetOSDExtensionFromDeviceExtension(t3_hostp[aptno]);	
	if(!t3_hostp[aptno] || p->major <= 0 || p->major != MAJOR(inodep->i_rdev))
		return(-ENODEV);

	/*spin_lock_irqsave(&p->simple_lock, flags);
	if(p->counter <= 0) {
		spin_unlock_irqrestore(&p->simple_lock, flags);
		return(-EBADF);
	}
	p->counter = 0;

	spin_unlock_irqrestore(&p->simple_lock, flags);*/

	ft_fasync(-1, filep, 0);

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
	MOD_DEC_USE_COUNT;
#endif
	return(0);
}

#if 0
void ioctl_timer_routine(
	IN bpVOID ComponentExtension,
	IN bpVOID Timer,
	IN bpVOID pContext
)	
{
	PREQ_HEAD pReq = (PREQ_HEAD)pContext;
	bpVOID pTimer;
	
	if(!(PENDING_REQ_HEAD_TYPE & pReq->ReqType) ){
		//OSD_DPRINT( ("ioctl_timer_routin: wake up pending req kkk reqtype %x\n",pReq->ReqType) );
		wake_up(&(pReq->waitq) );	
		return;	
	}
	
	pTimer=OSD_RequestTimer(ComponentExtension, 0.2*SECOND_IN_TIMER, ioctl_timer_routine, (bpVOID)pReq, (U32)NULL);

}	
#endif

typedef void ( *OSD_API_Handler ) ( bpOSD_COMP_EXTENSION pOSDExtension, PMY_SCSI_PASS_THROUGH_DIRECT_WITH_BUFFER pSptd);
#define OSD_PASSTHROUGHAPI_MAX 10

//#if !defined(NAPA_MARVELL) && !defined(OCTOPUSII) && !defined(AHCI) && !defined(ATI)
#if defined(SUPPORT_T3)
extern void CAM_GetBIOSVersion(bpVOID, PVERSION_INFO);
#endif

unsigned int
ft_api_get_adapter(
	IN bpOSD_COMP_EXTENSION pOSDExtension,
	IN PMY_SCSI_PASS_THROUGH_DIRECT_WITH_BUFFER pSptd
)
{
	PAPIFRAME_ADAPTER_INFORMATION pAdapter = 
		(PAPIFRAME_ADAPTER_INFORMATION)(pSptd+1);
	LPVOID pCam = OSD_GetComponentExtension(pOSDExtension, COMP_TYPE_CAM);
	VERSION_INFO BIOSVersion;
	int i;
	
	//TBD: optimize.
	pAdapter->SystemIoBusNumber = pOSDExtension->ControllerInfo.SystemIoBusNumber;
	pAdapter->SlotNumber = pOSDExtension->ControllerInfo.SlotNumber;
	pAdapter->InterruptLevel = pOSDExtension->ControllerInfo.InterruptLevel;
	pAdapter->InterruptVector = pOSDExtension->ControllerInfo.InterruptVector;
	pAdapter->VenDevID = pOSDExtension->ControllerInfo.DeviceID;
	pAdapter->SubVenDevID = pOSDExtension->ControllerInfo.SubDeviceID;
	for (i=0; i<BASE_ADDRESS_MAX_NUM; i++ )
		pAdapter->BaseAddress[i] = pOSDExtension->BaseAddress[i];
	pAdapter->ChannelNumber = 4; //TBD;
	pAdapter->ExpansionROMAddress = 0; //TBD;

	/* The version information is duplicated in PIOCTL_DRIVERVERSION_FRAME */
	pAdapter->DriverVersion.VerMajor = VERSION_MAJOR;
	pAdapter->DriverVersion.VerMinor = VERSION_MINOR;
	pAdapter->DriverVersion.VerOEM = VERSION_OEMCODE;
	pAdapter->DriverVersion.VerBuild = VERSION_BUILDNUM;

//#if !defined(NAPA_MARVELL) && !defined(OCTOPUSII) && !defined(AHCI)
#if defined(SUPPORT_T3) 
	CAM_GetBIOSVersion(pCam, &BIOSVersion);
	pAdapter->BIOSVersion.VerMajor = BIOSVersion.VerMajor;
	pAdapter->BIOSVersion.VerMinor = BIOSVersion.VerMinor;
	pAdapter->BIOSVersion.VerOEM = BIOSVersion.VerOEM;
	pAdapter->BIOSVersion.VerBuild = BIOSVersion.VerBuild;
#endif

	pAdapter->DIMMSize = pOSDExtension->ControllerInfo.DIMMSize;
	pAdapter->DIMMType = pOSDExtension->ControllerInfo.DIMMType;
	
	pSptd->SrbStatus = SRB_STATUS_SUCCESS;

	return 0;
}

#ifdef SUPPORT_ENCLOSURE
unsigned int
ft_api_enc_ctrl(
	IN bpOSD_COMP_EXTENSION pOSDExtension,
	IN PMY_SCSI_PASS_THROUGH_DIRECT_WITH_BUFFER pSptd
)
{
	/* Srb->Cdb[0]==svucPASSTHROUGH_API_COMMAND && Srb->Cdb[1]==0 
	 * Srb->Cdb[2] is the Enclosure Sub Command.
	 */
	unsigned int ret=0;
	bpVOID pCam = OSD_GetComponentExtension(pOSDExtension, COMP_TYPE_CAM);

	switch ( pSptd->Cdb[2] )
	{
		case API_ENCLOSURE_SUB_GETENCLOSURE:	/* Enumerate Enclosure */
			{
				PAPIFRAME_ENCLOSURE_INFORMATION pFrame = 
											(PAPIFRAME_ENCLOSURE_INFORMATION)(pSptd+1);
				CAM_EnclosureEnumerate(pCam, pFrame);

				pSptd->SrbStatus = SRB_STATUS_SUCCESS;
			}
			break;

		case API_ENCLOSURE_SUB_GETSENSOR:		/* Get Sensor Information */
			{	
				/* Srb->Cdb[3]: Enclosure Index	 */
				U8 sensorID;
				PAPIFRAME_SENSOR_INFORMATION pFrame = 
											(PAPIFRAME_SENSOR_INFORMATION)(pSptd+1);
				PAPIFRAME_SENSOR pAPI_Sensor=NULL;
				for ( sensorID=0; sensorID<CAM_MAX_SENSOR_PER_ENCLOSURE; sensorID++ )
				{
					pAPI_Sensor = &pFrame->Sensor[sensorID];
					pAPI_Sensor->SensorType = CAM_EnclosureGetSensorInfo(
						pCam, pSptd->Cdb[3], sensorID, &pAPI_Sensor->SensorValue);
				}

				pSptd->SrbStatus = SRB_STATUS_SUCCESS;
			}
			break;

		case API_ENCLOSURE_SUB_LEDLOCATE:	/* Locate Disk by Setting LED */
			/* 
			 * Srb->Cdb[3]: DiskID under Srb->Cdb[2]=API_ENCLOSURE_SUB_LEDLOCATE
			 * Srb->Cdb[4]: 1=Locate, 2=Stop, 3=Get Locate Status under Srb->Cdb[2]=API_ENCLOSURE_SUB_LEDLOCATE
			 */
			{
			
				U32 *pLEDControlStatus = (U32 *)(pSptd+1);
				*pLEDControlStatus = CAM_EnclosureLocateDiskWithDiskID(pCam, pSptd->Cdb[3], pSptd->Cdb[4]);

				pSptd->SrbStatus = SRB_STATUS_SUCCESS;
			}
			break;
		
		case API_ENCLOSURE_SUB_MAPDISKID: /* Map From Physical Disk ID to EnclosureID+CarrierID */
			/*
			 * Srb->Cdb[3]: DiskID
			 */
			{
				PAPIFRAME_ENCLOSURE_MAPDISKID pFrame = 
					(PAPIFRAME_ENCLOSURE_MAPDISKID)(pSptd+1);
				if ( CAM_EnclosureMapDiskID(pCam, pSptd->Cdb[3], &pFrame->EnclosureID, &pFrame->CarrierID) ){
                   			pSptd->SrbStatus = SRB_STATUS_SUCCESS;
				}else{
					pSptd->SrbStatus = SRB_STATUS_ERROR;
					ret =-EFAULT;
				}	
			}
			break;

		default:
			pSptd->SrbStatus = SRB_STATUS_ERROR;
			ret = -EFAULT;
			break;
	}

	return ret;
}
#endif

#ifdef SUPPORT_BATTERY
unsigned int
ft_api_battery_get_info(
	IN bpOSD_COMP_EXTENSION pOSDExtension,
	IN PMY_SCSI_PASS_THROUGH_DIRECT_WITH_BUFFER pSptd
)
{
	bpVOID pCam = OSD_GetComponentExtension(pOSDExtension, COMP_TYPE_CAM);

	PAPIFRAME_BATTERY_INFORMATION pFrame = 
									(PAPIFRAME_BATTERY_INFORMATION)(pSptd+1);
	CAM_BatteryGetInformation(pCam, pFrame);

	pSptd->SrbStatus = SRB_STATUS_SUCCESS;

	return 0;
}

#endif

/* Handle Passthrough API */
unsigned int 
ft_handle_passthu_api(
	IN bpOSD_COMP_EXTENSION pOSDExtension,
	IN PMY_SCSI_PASS_THROUGH_DIRECT_WITH_BUFFER pSptd
	)
{
	unsigned int ret=0;
	
	OSD_ASSERT( pSptd->Cdb[0]==svucPASSTHROUGH_API_COMMAND );
	OSD_ASSERT( pSptd->Cdb[1]<OSD_PASSTHROUGHAPI_MAX );

	/* Handle Passthrough API. */
	switch ( pSptd->Cdb[1] )
	{
#ifdef SUPPORT_ENCLOSURE
		case ocmdEnclosure:
			ret = ft_api_enc_ctrl(pOSDExtension, pSptd);
			break;
#endif
#ifdef SUPPORT_BATTERY
		case ocmdBattery:
            		ret = ft_api_battery_get_info(pOSDExtension, pSptd);
			break;
#endif
		case ocmdGetAdapter:
			ret = ft_api_get_adapter(pOSDExtension, pSptd);
			break;
		default:
			pSptd->SrbStatus = SRB_STATUS_ERROR;
			ret =-EFAULT;
			break;
	}
	
	return ret;	/* OSD Passthrough API always return immediately. */
}

void Ioctl_Return(void *sender, bpREQ request)
{
	REQ_HEAD *pReq = (REQ_HEAD *)(request->pOriginalRequest);

	//if(gFlagInt)
		//OSD_DPRINT( ("Ioctl_Return: call Ioctl_Return in interrupt \n") );	
	
	if(pReq->ReqType & PASSTHRU_REQ_HEAD_TYPE){
		pReq->ReqType &= ~PENDING_REQ_HEAD_TYPE;
		//OSD_DPRINT( ("Ioctl_Return(ioctl): SrbStatus =%x, ScsiStatus = %x, ReqType=%x\n", request->bAdapterStatus, request->bTargetStatus,pReq->ReqType) );
		if(!(pReq->ReqType & SLEEPING_REQ_HEAD_TYPE) )
			wake_up(&(pReq->waitq) );
		return;	
	}

}

/* function   : int ft_ioctl(struct inode *inodep,
		struct file *filep,
		unsigned int req,
		unsigned long arg)
 * input        : Standard Parameters
 * output      : 0 if success, otherwise error reason
 * description  : device IOCTL entry
 */

int ft_ioctl(struct inode *inodep, struct file *filep, unsigned int req, unsigned long arg)
{	

	unsigned int ret = 0;
	int aptno = MINOR(inodep->i_rdev);
	bpOSD_COMP_EXTENSION p = OSDI_GetOSDExtensionFromDeviceExtension(t3_hostp[aptno]);
	PSRB_IO_CONTROL pSrbCtl = (PSRB_IO_CONTROL)arg;

	/* Step1: basic parameter check */
	if(aptno < 0 || aptno >= MAX_ADAPTORS || !t3_hostp[aptno] || p->major <= 0 || p->major != MAJOR(inodep->i_rdev) )
		return(-ENODEV);

	/* if(p->counter <= 0) {
		return(-EIO);
	} */
	
	if( !arg ) {
		return(0x10);
	}

	//OSD_TRACE( ("passthu ioctl number %x, req is %x \n",IOCTL_FASTTRAK_PASSTHRU, req) );
	/* Step2: Signature of the SRB_IO_CONTROL is wrong. Return immediately. */
	if(	(*((U32 *)&(pSrbCtl->Signature))!=FASTTRAK_IOCTL_SIGNATURE1) ||
		(*(((U32 *)&(pSrbCtl->Signature))+1)!=FASTTRAK_IOCTL_SIGNATURE2)
	  )
	 	return(-EIO);

	

	/*Step3: handle cmd */
	switch(req){
		case IOCTL_FASTTRAK_GETVERSION:
		{
			PIOCTL_DRIVERVERSION_FRAME pVersion = (PIOCTL_DRIVERVERSION_FRAME)(pSrbCtl + 1);
			pVersion->Major = VERSION_MAJOR;
			pVersion->Minor = VERSION_MINOR;
			pVersion->Oem = VERSION_OEMCODE;
			pVersion->BuildNo = VERSION_BUILDNUM;
			pVersion->Signature[0] = 'NAPA';
			pVersion->ConsoleID = CONSOLEDEVICE_TARGETID;
			pVersion->HostNumber = p->host_no;
	//		OSD_TRACE( ("ret = %x\n", ret) );
			break;
		}
		case IOCTL_FASTTRAK_PASSTHRU:
		{
			PREQ_HEAD pReq; 
			bpREQ p_rq;
			PMY_SCSI_PASS_THROUGH_DIRECT_WITH_BUFFER pSptd = (PMY_SCSI_PASS_THROUGH_DIRECT_WITH_BUFFER)(pSrbCtl +1);
			bpOSD_COMP_EXTENSION pOSDExtension = p;
			unsigned long flags;
			PDATA_POOL pDataPool;
			//bpVOID Timer;
			unsigned long	dma; //add by sandy
		
			/* Handle Console Request which related to the OSD. */ 
			if ( pSptd->TargetId==CONSOLEDEVICE_TARGETID && pSptd->Lun==0 ){

	//			OSD_TRACE( ("Scsi Passthrough Request: Cdb[0]=0x%x, Cdb[1]=0x%x, Cdb[2]=0x%x Cdb[3]=0x%x.\n", pSptd->Cdb[0], pSptd->Cdb[1], pSptd->Cdb[2], pSptd->Cdb[3]) );

				/* What kind of request OSD can handle. */
				if( svucPASSTHROUGH_API_COMMAND == pSptd->Cdb[0]){ 
					return ft_handle_passthu_api(p, pSptd);
				}

				/* OSD have no idea to handle request, pass it  to Engine */

				
				if( (pReq = get_req_from_pool(pOSDExtension) ) == NULL ){	
					return (-EIO);
				}	
				
				//OSD_DPRINT( ("Get req from pool succeed\n") );				

				if( (pDataPool = alloc_data_pool(pOSDExtension) ) == NULL ){	
					return (-EIO);
				}
				

				//OSD_DPRINT( ("Get data buffer from pool succeed\n") );				
				
				init_waitqueue_head(&(pReq->waitq) );	
				pReq->pSptd = (void *)pSptd;
				pReq->ReqType = PASSTHRU_REQ_HEAD_TYPE + PENDING_REQ_HEAD_TYPE + SLEEPING_REQ_HEAD_TYPE;
				//pReq->waitq = (void *)(&(pOSDExtension->WaitQ) );
					
				p_rq = &(pReq->req);
				
				p_rq->bID = pSptd->TargetId;
				p_rq->bSenseSize = pSptd->SenseInfoLength;
				//p_rq->p_sns = pSptd->SenseBuffer;				
				p_rq->p_sns = (spBUF)(pDataPool->SenseBuffer);				
				p_rq->bLUN = pSptd->Lun;
				p_rq->p_buf = (spBUF)(pDataPool->Buffer);
				//p_rq->p_buf = (spBUF)(pSptd + 1);
				p_rq->wSize = pSptd->DataTransferLength;

				//OSD_DPRINT( ("ft_ioctl: Req = 0x%p, ReqType=0x%x, p_rq->wSize = 0x%x, p_rq->p_buf = 0x%p\n", pReq, pReq->ReqType, p_rq->wSize, p_rq->p_buf ) );
				memcpy(p_rq->bCDB, pSptd->Cdb, Min(16, pSptd->CdbLength) );
				memcpy(p_rq->p_sns, pSptd->SenseBuffer, Min(p_rq->bSenseSize,SPT_SENSE_LENGTH) );

				if( p_rq->bCDB[0]==svucDEBUGGING_COMMAND ){
                                        p_rq->p_buf = (LPVOID)pci_alloc_consistent(NULL, 1024, &dma);
                                        if( !p_rq->p_buf  ){

                                                release_data_pool(pOSDExtension, pDataPool);
                                                put_req_to_pool(pOSDExtension, pReq);
                                                ret = -EFAULT;
                                                break;
                                        }
                                }
                                //for DBG_SECTOR_RW, 20070301, by sandy

				memcpy(p_rq->p_buf, (spBUF)(pSptd +1), Min(p_rq->wSize,MAX_DATA_BUFFER_SIZE) );
				//OSD_DPRINT( ("Data Buffer Address is %p\n",pDataPool->Buffer) );
				//memcpy(pDataPool->Buffer, (spBUF)(pSptd +1), Min(p_rq->wSize,MAX_DATA_BUFFER_SIZE) );
			
				if( p_rq->bCDB[0]==svucDEBUGGING_COMMAND ){		// prepare sg
					
 		 			pReq->SgTable.SGCount = 1;
					#if defined(CONFIG_HIGHMEM64G) || defined(CONFIG_X86_64)
						pReq->SgTable.table[0].StartAddress.Value = dma;
					#else
  						pReq->SgTable.table[0].StartAddress.u.Low = dma;
					#endif
					
					pReq->SgTable.table[0].Count = (U32)p_rq->wSize;
					pReq->SgTable.SizeInByte = (U32)p_rq->wSize;
					pReq->SgTable.table[0].CtrlFlag = SG_CF_EOT|SG_CF_64B|SG_CF_HOST;
					//pReq->SgTable.table[0].CtrlFlag = SG_CF_EOT|SG_CF_64B;
					
					pReq->req.wSize = (U32)p_rq->wSize; //add by sandy 20061025	
  					pReq->req.pSgTable = &(pReq->SgTable);			
				}
				//for DBG_SECTOR_RW, 20070301, by sandy
				
				DRIVER_LOCK(pOSDExtension->host,flags);
#if VDEBUG
				gIoctlFlag = 1;
				if(gFlagInt)
					OSD_DPRINT( ("Oh, My God, we will call IOCTL handler in interrupt routine \n") );	
#endif					  					

				Local_StartIO(pOSDExtension, pReq, Ioctl_Return);
#if VDEBUG
				gIoctlFlag = 0;
#endif				
				DRIVER_UNLOCK(pOSDExtension->host,flags);
				
				pReq->ReqType &= ~SLEEPING_REQ_HEAD_TYPE;

				if(PENDING_REQ_HEAD_TYPE & pReq->ReqType)
					//	interruptible_sleep_on(&(pReq->waitq) );
					wait_event(pReq->waitq, !(pReq->ReqType & PENDING_REQ_HEAD_TYPE) );
				
				//OSD_DPRINT( ("ft_ioctl:  p_rq->wSize = 0x%x, p_rq->p_buf = 0x%p\n", p_rq->wSize, p_rq->p_buf ) );

				memcpy( pSptd->SenseBuffer, p_rq->p_sns, Min(p_rq->bSenseSize,SPT_SENSE_LENGTH) );
				memcpy( (spBUF)(pSptd +1), p_rq->p_buf, Min(p_rq->wSize,MAX_DATA_BUFFER_SIZE) );
				
				if( p_rq->bCDB[0]==svucDEBUGGING_COMMAND ){
					if(p_rq->p_buf )
						pci_free_consistent(NULL, 1024, p_rq->p_buf, dma);
				}
				//for DBG_SECTOR_RW, 20070301, by sandy

				pSptd->SrbStatus = p_rq->bAdapterStatus;
				pSptd->ScsiStatus = p_rq->bTargetStatus;			

				release_data_pool(pOSDExtension, pDataPool);
				put_req_to_pool(pOSDExtension, pReq);	
				
				if( (SRB_NO_ERROR == pSptd->SrbStatus) || (SRB_SEE_SENSE == pSptd->SrbStatus) )
					ret = 0;
				else 
					ret = -EFAULT;


			} /* if console request. */
			break;
		}
		default:
			OSD_DPRINT( ("ft_ioctl: unknown ioctl %x\n", req) );
			ret = -EINVAL;
			break;
	}	

	return ret;	

}

int ft_compat_ioctl(struct file *filep, unsigned int req, unsigned long arg)
/*add new ft_ioctl from compat_ioctl, solve kernel version after 2.6.16 remove register_ioctl32_conversion,
then 32bits applications(ex: webpam) can't work on 64 bits OS, by sandy 20070309*/

{
	struct inode *inodep = filep->f_dentry->d_inode;
	ft_ioctl(inodep, filep, req, arg);
}


int ft_fasync(int fd, struct file * filep, int mode)
{
	bpOSD_COMP_EXTENSION p = (bpOSD_COMP_EXTENSION)(filep->private_data);
	
	return fasync_helper(fd, filep, mode, &p->async_queue); 
}

